#include "mainwin.h"
#include "_grafik.h"
#include "utils/msg.h"
#include "utils/utils.h"
#include "drawbefehle.h"
#include "steuerung.h"
#include "galerie.h"


extern tMsgBox *MsgBox;  // MsgBox

// class tGrafik >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

tGrafik::tGrafik(QWidget *parent): tTurtle(parent, this),
 ScriptEngine(0),
 DiaRestwert(0)
{
 DiaStart=true;
}

tGrafik::~tGrafik()
{ delEngine();

}


QString tGrafik::Anleitung()
{ return "Basisklasse tGrafik\n";
}


void tGrafik::draw()           // Default
{
  init();

  if (!begin()) return;        // Painter.begin(), Setup anwenden

  setFont("Arial", 2.5);       // Beschriftung 2.5 mm
  setPen("black");             // Stift, Schwarz
  drawKoordSystem();           // Koordinatenachsen, Beschriftung=true
  drawRaster(10);              // Raster
 }


void tGrafik::setEngine(QScriptEngine *aScriptEngine)
{ delete ScriptEngine;
  ScriptEngine=aScriptEngine;
  PaintAll=true;                     // alles neu zeichnen
  DiaStart=true;
}

void tGrafik::delEngine()
{ delete ScriptEngine; ScriptEngine=0;
}

bool tGrafik::hasErr(QScriptEngine *engine, bool show)
{ if (!engine) return false;
  if (engine->hasUncaughtException())
  { if (show)      
    { int Zeile=engine->uncaughtExceptionLineNumber();
      if (Zeile==1)
      {
        MsgBox->addLn( "Kein Script gefunden!", tMsgBox::err);
      }
      else
      MsgBox->addLn( "Scriptzeile " + Int(Zeile,0)+": "+
                      engine->uncaughtException().toString(),
                      tMsgBox::err);
    }
    return true;
  }
  return false;
}


QString tGrafik::getScriptVar(const QString &VarName, const QString &Default)
{ QString s;
  if (hasScript())
  { s=ScriptEngine->evaluate(VarName).toString();
    if (!hasErr(ScriptEngine,false)) return s;
  }
  return Default;
}

int tGrafik::getScriptVar(const QString &VarName, int Default, int min, int max)
{ int i=Default;
  if (hasScript())
  { i=ScriptEngine->evaluate(VarName).toInteger();
    if (hasErr(ScriptEngine,false)) i=Default;
  }
  if (i<min) i=min;  if (max<i) i=max;
  return i;
}

qreal tGrafik::getScriptVar(const QString &VarName, qreal Default)
{ qreal x=Default;
  if (hasScript())
  { x=ScriptEngine->evaluate(VarName).toNumber();
    if (hasErr(ScriptEngine,false)) x=Default;
  }
  return x;
}


// ----------------------------------------------------------------------------
bool tGrafik::initScript(QScriptValue &thisScript)
{
  init(); // tTurtle init
  setIdentity();

  if (!hasScript()) return false;
  if (!thisScript.isValid())
  {  thisScript=ScriptEngine->newQObject(this);

     // Alle 'public slots:' von tTurtle und tGrafik Slots
     // sind im Script unter "t" verfügbar
     ScriptEngine->globalObject().setProperty("t",thisScript);
  }

  return callScript(thisScript, ScriptInit);
}


// ----------------------------------------------------------------------------
bool tGrafik::drawScrip(const QScriptValue &thisScript)
{ return callScript(thisScript, ScriptDraw);
}


// ----------------------------------------------------------------------------
bool tGrafik::callScript(const QScriptValue &thisScript,const QString &Bezeichner )
{  // Script-Funktion Bezeichner() von C++ aufrufen

   if (!thisScript.isValid())
   { MsgBox->addLn("C++ Err: init(thisScript,this) fehlt in draw()!", tMsgBox::err);
     return false;
   }

  QScriptValue ScriptFnkt = ScriptEngine->evaluate(Bezeichner);
  if (!hasErr(ScriptEngine,false))
  {  if (!ScriptFnkt.isFunction())
      { MsgBox->addLn("Err: '"+Bezeichner+"()' ist keine Funktion!", tMsgBox::err);
        return false;
      }
     ScriptFnkt.call(thisScript);
     if (hasErr(ScriptEngine))
      { emit markScriptLine(ScriptEngine->uncaughtExceptionLineNumber());
        return false;
      }     
     return true;
  }
  else MsgBox->addLn("Hinweis: Keine Funktion '"+Bezeichner+"()'!", tMsgBox::err);
  return false;
}

// ----------------------------------------------------------------------------
bool tGrafik::callBefehl(const QString &BefehlsCode, const QString &Befehl)
{ if (!hasScript()) { return false; }

  ScriptEngine->evaluate(BefehlsCode);

  QScriptValue Fnkt=ScriptEngine->evaluate(Befehl);
  if (!Fnkt.isFunction())
   { MsgBox->addLn("Err: '"+Befehl+"()' ist keine Funktion!", tMsgBox::err);
     return false;
   }
   Fnkt.call(QScriptValue());
   return true;
}


// ----------------------------------------------------------------------------
QString tGrafik::ScriptSource(const QString Bezeichner)
{
  if (!hasScript()) { return ""; }

  QString s;
  QScriptValue ScriptObj = ScriptEngine->evaluate(Bezeichner);
  if (!hasErr(ScriptEngine,false))
  {
    s=ScriptObj.toString();
    if (hasErr(ScriptEngine))
      { emit markScriptLine(ScriptEngine->uncaughtExceptionLineNumber());
        s.clear();
      }
  }
  else MsgBox->addLn("Err: Keine Bezeichner '"+Bezeichner+"' im Script!", tMsgBox::err);
  return s;
}


// ----------------------------------------------------------------------------
void setRaster(qreal &x, int d=-10)
{ if (d==0) d=-10;
  int i=x/abs(d);
  x= i*abs(d)+d;
}


void tGrafik::drawKoordSystem(bool Beschriftung, qreal Einheiten_mm, qreal PSize)
{ qreal xmin,xmax,ymin,ymax; bool drawEinheiten=true;
  qreal x0=lastX(), y0=lastY();

  if (Einheiten_mm<=0) {Einheiten_mm=10; drawEinheiten=false;}
  //qreal d= FontSize_mm()+PSize*1.5;
  qreal d= PSize+ FontSize_mm()*0.6;

  TurtleSetup.getP0P1().getCoords(&xmin,&ymax,&xmax,&ymin);
  setRaster(xmin,-Einheiten_mm); setRaster(xmax,+Einheiten_mm);
  setRaster(ymin,-Einheiten_mm); setRaster(ymax,+Einheiten_mm);

  QLineF l(xmin,0,xmax,0); Painter.drawLine(l);
  l.setLine(0,ymin,0,ymax); Painter.drawLine(l);

  if (!drawEinheiten) goto fertig;
  for (qreal x=xmin ; x<=xmax; x=x+Einheiten_mm)
   { l.setLine(x,-PSize,x,+PSize); Painter.drawLine(l);
   }

  for (qreal y=ymin; y<=ymax; y=y+Einheiten_mm)
   { l.setLine(-PSize,y,+PSize,y); Painter.drawLine(l);
   }

  if (!Beschriftung) goto fertig;

  for (qreal y=ymin; y<=ymax; y=y+Einheiten_mm)
   { text(d, y, Int(y,2));
   }

  d+= FontSize_mm();
  for (qreal x=xmin ; x<=xmax; x=x+Einheiten_mm)
   { text(x,-d,Int(x,2));
   }
 fertig:
  goTo(x0,y0);
}

//-----------------------------------------------------------------------------
void tGrafik::drawRaster(qreal Einheiten_mm, qreal Psize)
{ qreal xmin,xmax,ymin,ymax;

  TurtleSetup.getP0P1().getCoords(&xmin,&ymax,&xmax,&ymin);
  setRaster(xmin,-Einheiten_mm); setRaster(xmax,+Einheiten_mm);
  setRaster(ymin,-Einheiten_mm); setRaster(ymax,+Einheiten_mm);

  QLineF l;
  for (qreal x=xmin ; x<=xmax; x=x+Einheiten_mm) {
    for (qreal y=ymin; y<=ymax; y=y+Einheiten_mm)
    {
      l.setLine(x-Psize,y,x+Psize,y); Painter.drawLine(l);
      l.setLine(x,y-Psize,x,y+Psize); Painter.drawLine(l);
    }
  }
  goTo(0.0,0.0);
}



//-----------------------------------------------------------------------------
void tGrafik::defDraw(const QString Name, int mode)
{ if (RecDrawName==Name) return;

  if (getFnktText)
   { getFnktText->append(QString("t.defDraw(\""+Name+"\",%1);\n").arg(mode));}


  if (mode==0)
    { // Befehl Polygon neu definieren
      tDrawBefehl &bef=tMainWin::BefehleLst.defBefehl(Name);            
      if (!bef.isNil()) { bef.saveStyle(this);  bef.setPoly(this);}
      return;
    }

  if (mode==3)
    { // Polygon in Master einpassen
      tDrawBefehl &master=tMainWin::BefehleLst.Befehl(Name);
      master.insertPoly(this);
      return;
    }

  // Subpolygon anhängen
  tDrawBefehl &bef=tMainWin::BefehleLst.addBefehl(Name);

  if (!bef.isPolygon()) return;

  bef.saveStyle(this);

  if (mode==2)
    { //Polygon an Master.Polygon Klippen
      tDrawBefehl &master=tMainWin::BefehleLst.Befehl(Name);
      bef.setPoly(this, &master.getPolygon() );
      return;
    }

  bef.setPoly(this);

}

//-----------------------------------------------------------------------------
bool tGrafik::defScript(const QString Flag, const QString Grafikname)
{ if (RecDrawName==Flag) return false;
  if (getFnktText)
   { getFnktText->append(QString("t.defScript(\""+Flag+"\",\""+Grafikname+"\");\n"));}

  if (tMainWin::BefehleLst.exists(Flag)) return true;

  tDrawBefehl &bef=tMainWin::BefehleLst.defBefehl(Flag);
  if (bef.isNil()) return false;

  bef.setFnktText("function "+Flag+"() { t.msg(\"Flag: "+Grafikname+"\");");

  // pop Aufruf Grafikname und Rücksprung nach 'this' auf den Stack
  tMainWin::Steuerung.call(this, Grafikname);

  return false;
}


//-----------------------------------------------------------------------------
void tGrafik::defCall(const QString Name, const QString Befehle, int mode)
{
  if (getFnktText)
   { getFnktText->append(QString("t.defCall(\""+Name+"\",\""+Befehle+"\",%1);\n").arg(mode));}

  if (mode==0)
  { // Befehl Funktion neu definieren
    tDrawBefehl &bef=tMainWin::BefehleLst.defBefehl(Name);
   if (!bef.isNil())
    {   bef.setFnktText("function ")->append(Name).append("() {\n");
        bef.getFnktText()->append(Befehle).append("\n");
    }
   return;
  }

  tDrawBefehl &bef=tMainWin::BefehleLst.Befehl(Name);
  if (bef.isFunction()) bef.getFnktText()->append(Befehle).append("\n");
}


//-----------------------------------------------------------------------------
void tGrafik::call(const QString Name)
{
  if (getFnktText)
  { getFnktText->append(QString("t.call(\""+Name+"\");\n"));
  }

  tTurtleStatus Status=TurtleStatus(false,true,false);
    // keine Detail-Aufzeichnung!
    // Turtle nicht speichern

  tMainWin::BefehleLst.Befehl(Name).draw(this);

  setTurtleStatus(Status);
}


//-----------------------------------------------------------------------------
void tGrafik::defPoint(const QString Name)
{ if (RecDrawName==Name) return;

  if (getFnktText)
   { getFnktText->append(QString("t.defPoint(\""+Name+"\");\n"));}

  tMainWin::BefehleLst.defBefehl(Name).setLast(this);
}

//-----------------------------------------------------------------------------
void tGrafik::defPoint(const QString Name, int Index)
{ if (RecDrawName==Name) return;

  if (getFnktText)
   { getFnktText->append(QString("t.defPoint(\""+Name+"\",%1);\n").arg(Index));}

  QPointF oldLast=Last;

  int size=Poly.size();
    if (size>0)
    { if (Index<0) Index=size+Index;
      if (Index<0) Index=0; if (Index>=size) Index=size-1;
      Last=Poly.at(Index);
    }
  tMainWin::BefehleLst.defBefehl(Name).setLast(this);
  Last=oldLast;
}



//-----------------------------------------------------------------------------
void tGrafik::defPoint(const QString Name, const QString P1, const QString P2, qreal t)
{ if (RecDrawName==Name) return;

  if (getFnktText)
   { getFnktText->append(QString("t.defPoint(\""+Name+"\",\""+P1+"\",\""+P2+"\", %1);\n").arg(t,0,'g',8));}

  QPointF oldLast=Last;
  QPointF A=tMainWin::BefehleLst.Befehl(P1).getFirstP();
  QPointF B=tMainWin::BefehleLst.Befehl(P2).getFirstP();

  Last=A+t*(B-A);
  tMainWin::BefehleLst.defBefehl(Name).setLast(this);
  Last=oldLast;
}


//-----------------------------------------------------------------------------
void tGrafik::defVector(const QString Name, const QString P1, const QString P2, qreal t)
{ if (RecDrawName==Name) return;

  if (getFnktText)
   { getFnktText->append(QString("t.defVektor(\""+Name+"\",\""+P1+"\",\""+P2+"\",%1);\n").arg(t,0,'g',8));}

  QPointF oldLast=Last;
  QPointF A=tMainWin::BefehleLst.Befehl(P1).getFirstP();
  QPointF B=tMainWin::BefehleLst.Befehl(P2).getFirstP();

  Last=t*(B-A);
  tMainWin::BefehleLst.defBefehl(Name).setLast(this);
  Last=oldLast;
}



//-----------------------------------------------------------------------------
void tGrafik::defLine(const QString Name, const QString P1, const QString P2)
{ if (RecDrawName==Name) return;

  if (getFnktText)
   { getFnktText->append(QString("t.defLine(\""+Name+"\",\""+P1+"\",\""+P2+"\");\n"));}

  QPolygonF temp;
  temp.append(tMainWin::BefehleLst.Befehl(P1).getFirstP());
  temp.append(tMainWin::BefehleLst.Befehl(P2).getFirstP());

  tDrawBefehl &bef=tMainWin::BefehleLst.defBefehl(Name);
  if (!bef.isNil()) { bef.saveStyle(this->Stift,this->Pinsel, NoCloseNoFill);
                      bef.setPoly(temp);}
}

//-----------------------------------------------------------------------------
void tGrafik::defLine(const QString Name,  qreal l)
{ if (RecDrawName==Name) return;

  if (getFnktText)
   { getFnktText->append(QString("t.defLine(\""+Name+"\",%1);\n").arg(l,0,'g',8));
   }

  QPolygonF temp;
  tTurtleStatus Status=TurtleStatus(false,true);
  temp.append(Last); setNoStyle(); move(l);  temp.append(Last);
  setTurtleStatus(Status);

  tDrawBefehl &bef=tMainWin::BefehleLst.defBefehl(Name);
  if (!bef.isNil()) { bef.saveStyle(this->Stift,this->Pinsel, NoCloseNoFill);
                      bef.setPoly(temp);}
}


//-----------------------------------------------------------------------------
bool tGrafik::exists(const QString Name)
{ if (RecDrawName==Name) return false;

  if (getFnktText) { getFnktText->append(QString("t.exists(\""+Name+"\");\n"));}

   return tMainWin::BefehleLst.exists(Name);
}


//-----------------------------------------------------------------------------
void tGrafik::shift(const QString Name,  qreal t)
{ if (RecDrawName==Name) return;

  if (getFnktText) { getFnktText->append(QString("t.shift(\""+Name+"\",%1);\n").arg(t,0,'g',8) );}

  tDrawBefehl &bef=tMainWin::BefehleLst.Befehl(Name);
  bef.shift(this, t);
}


//-----------------------------------------------------------------------------
void tGrafik::recDraw(QString Name)
{
  getFnktText=0; // stop
  if (Name.isEmpty()){ RecDrawName.clear(); }
  else
  {
    tDrawBefehl &bef=tMainWin::BefehleLst.defBefehl(Name);
    if (bef.isNil()) return;

    RecDrawName=bef.Bezeichner();
    getFnktText=bef.setFnktText("function ");
    getFnktText->append(Name).append("() {\n");
  }
}

//-----------------------------------------------------------------------------
void tGrafik::clrDraw()
{ if (getFnktText)
    MsgBox->addLn("Recorder aktiv! clrDraw() nicht möglich! !",tMsgBox::err);
  else
   tMainWin::BefehleLst.clear();
}


//-----------------------------------------------------------------------------
void tGrafik::clrDraw(const QString &Name)
{ if (getFnktText)
    MsgBox->addLn("Recorder aktiv! clrDraw() nicht möglich! !",tMsgBox::err);
  else
    tMainWin::BefehleLst.clrBefehl(Name);
}



//-----------------------------------------------------------------------------
void tGrafik::setRotate(qreal Winkel,bool reset)
{ if (getFnktText)
    getFnktText->append(QString("t.setRotate(%1,%2);\n").arg(Winkel,0,'g',8).arg(reset));
  if (reset) drawMatrix.reset();
  drawMatrix.rotate(Winkel);
}

//-----------------------------------------------------------------------------
void tGrafik::setRotate(const QString Punkt, qreal Winkel, bool reset)
{ if (getFnktText)
    getFnktText->append(QString("t.setRotate(\""+Punkt+"\",%1,%2);\n").arg(Winkel,0,'g',8).arg(reset));

  QPointF P=tMainWin::BefehleLst.Befehl(Punkt).getP0();
  QMatrix m;
  m.translate(P.x(),P.y());
  m.rotate(Winkel);
  m.translate(-P.x(),-P.y());
  if (reset) drawMatrix=m;
  else drawMatrix=drawMatrix*m;
}


//-----------------------------------------------------------------------------
void tGrafik::setMirror(QPointF A, QPointF B, bool reset)
{
  QPointF oldLast=Last;
  Last=B-A; qreal w=getW(); Last=oldLast;
  QMatrix m;
  m.translate(A.x(),A.y());
  m.rotate(w);
  m.scale(1,-1);
  m.rotate(-w);
  m.translate(-A.x(),-A.y());

  if (reset) drawMatrix=m;
  else drawMatrix=drawMatrix*m;
}


//-----------------------------------------------------------------------------
void tGrafik::setMirror(qreal x0, qreal y0, qreal x1, qreal y1 , bool reset)
{
  if (getFnktText)
      getFnktText->append(QString("t.setMirror(%1,%2,%3,%4,%5);\n").arg(x0).arg(y0).arg(x1).arg(x1).arg(reset));

  QPointF A(x0,y0),B(x1,y1);
  setMirror(A,B,reset);
}

//-----------------------------------------------------------------------------
void tGrafik::setMirror(const QString Punkt1, const QString Punkt2, bool reset)
{ if (getFnktText)
    getFnktText->append(QString("t.setMirror(\""+Punkt1+"\",\""+Punkt2+"\",%1);\n").arg(reset));

  setMirror(tMainWin::BefehleLst.Befehl(Punkt1).getP0(), tMainWin::BefehleLst.Befehl(Punkt2).getP0(), reset);
}


//-----------------------------------------------------------------------------
void tGrafik::setScale(qreal sx, qreal sy, bool reset)
{ if (getFnktText)
    getFnktText->append(QString("t.setScale(%1,%2,%3);\n").arg(sx,0,'g',8).arg(sy,0,'g',8).arg(reset));
  if (reset) drawMatrix.reset();
  drawMatrix.scale(sx,sy);
}


//-----------------------------------------------------------------------------
void tGrafik::tGrafik::setShear(qreal sh, qreal sv, bool reset)
{ if (getFnktText)
    getFnktText->append(QString("t.setShear(%1,%2,%3);\n").arg(sh,0,'g',8).arg(sv,0,'g',8).arg(reset));
  if (reset) drawMatrix.reset();
  drawMatrix.scale(sh,sv);
}


//-----------------------------------------------------------------------------
void tGrafik::setTranslate(qreal dx, qreal dy, bool reset)
{ if (getFnktText)
    getFnktText->append(QString("t.setTranslate(%1,%2,%3);\n").arg(dx,0,'g',8).arg(dy,0,'g',8).arg(reset));
  if (reset) drawMatrix.reset();
  drawMatrix.translate(dx - drawMatrix.dx(), dy - drawMatrix.dy());
}



//-----------------------------------------------------------------------------
void tGrafik::setIdentity()
{ if (getFnktText)
    getFnktText->append(QString("t.setIdentity();\n"));
  drawMatrix.reset();
}



//-----------------------------------------------------------------------------
void tGrafik::setPolygon(int Start, int Anzahl)
{
  if (getFnktText)
    getFnktText->append(QString("t.setPolygon(%1,%2);\n").arg(Start).arg(Anzahl));

  if (Poly.isEmpty()) return;

  QPolygonF temp=drawMatrix.map(Poly);
  Poly.clear();

  if (Start>=0)
  { int anf=Start, end;
    if(anf>=temp.size()) return;
    if (Anzahl<0) end=temp.size(); else end=Start+Anzahl;
    if (end>temp.size()) end=temp.size();
    for (int i=anf; i<end; i++) Poly.append( temp.at(i));
    return;
  }

  if (Start<0)
  { int anf=Start+temp.size(), end;
    if (anf<0) return;
    if (Anzahl<0) end=-1; else end = anf-Anzahl;
    if (end<-1) end=-1;
    if (end>temp.size()) end=temp.size();
    for (int i=anf;  i>end; i--) Poly.append( temp.at(i));
  }
}



//-----------------------------------------------------------------------------
void tGrafik::addPolygon(const QString Name, int mode)
{
  if (Name.isEmpty()) return;
  if (getFnktText) getFnktText->append(QString("t.addPolygon(\""+Name+"\",%1);\n").arg(mode));

  tDrawBefehl &item=tMainWin::BefehleLst.Befehl(Name);
  if (item.isPolygon())
   {
     QPointF P0, P1, P01;
     QPolygonF temp=drawMatrix.map(item.getPolygon());

     if (Poly.size()>0) P0=Poly.last();

     if (temp.size()>0)
      { if (mode==0) P1=temp.first();
        else P1=temp.last();
      }
     // Schiebvektor P1-P0
     P01=P1-P0;
     QMatrix m;  m.translate(-P01.x(), -P01.y());
     temp=m.map(temp);

     if (mode==-1) for (int i=temp.size()-1; i>=0; i--) Poly.append( temp.at(i));
     else          for (int i=0; i<temp.size();    i++) Poly.append( temp.at(i));
   }
}





//-----------------------------------------------------------------------------
void tGrafik::setPolygon(const QString Name, int mode, qreal dt)
{
  if (Name.isEmpty()) return;
  if (getFnktText) getFnktText->append(QString("t.setPolygon(\""+Name+"\",%1,%2);\n").arg(mode).arg(dt,0,'g',8));

  tDrawBefehl &item=tMainWin::BefehleLst.Befehl(Name);
  if (item.isPolygon()||item.isPunkt())
    { QPolygonF temp=drawMatrix.map(item.getPolygon());

      switch (mode)
      {
        case -1:
        case  0:
           Poly.clear();
           if (mode==-1) for (int i=temp.size()-1; i>=0; i--) Poly.append( temp.at(i));
           else          for (int i=0; i<temp.size();    i++) Poly.append( temp.at(i));

           return;

        case 1:
           Poly=Poly.intersected( temp );
           return;

        case 2:
           Poly=Poly.united( temp );
           return;

        case 3:
           if (PolyFillMode==NoCloseNoFill)  cutPolyLine(Poly,temp);
           else Poly=Poly.subtracted( temp );
           return;

        case 4:
           PolyFillMode=  item.getFillMode();
           spline(temp, Poly, PolyFillMode, dt);
           return;

        default: return;
      }
    }
  MsgBox->addLn("Polygon '"+Name+"' nicht gefunden!");
}



//-----------------------------------------------------------------------------
void tGrafik::draw(const QString Name, bool UseStyle)
{
  if (getFnktText)
  { getFnktText->append(QString("t.draw(\""+Name+"\", %1);\n").arg(UseStyle));
  }

  tTurtleStatus Status=TurtleStatus(false,true); // keine Detail-Aufzeichnung!

  tMainWin::BefehleLst.Befehl(Name).draw(this,UseStyle);

  setTurtleStatus(Status); 
}


//-----------------------------------------------------------------------------
void tGrafik::drawName(const QString Name, qreal dx, qreal dy)
{
  if (getFnktText)  // keine Detail-Aufzeichnung!
  { getFnktText->append(QString("t.drawName(\""+Name+"\",%1,%2);\n").arg(dx,0,'g',8).arg(dy,0,'g',8));
  }

  tTurtleStatus Status=TurtleStatus(false,true); // keine Detail-Aufzeichnung!
  tMainWin::BefehleLst.Befehl(Name).drawName(this, dx, dy);

  setTurtleStatus(Status);
}


//-----------------------------------------------------------------------------
void tGrafik::msgDraw(const QString Name)
{ MsgBox->addLn( drawInfo(Name),tMsgBox::inf1);
}


//-----------------------------------------------------------------------------
QString tGrafik::drawInfo(const QString Name)
{
  if (Name.isEmpty())  return tMainWin::BefehleLst.Info();

  tDrawBefehl &item=tMainWin::BefehleLst.Befehl(Name);

  if (item.Bezeichner()!=Name)
   return "Name '"+Name+"' nicht gefunden!";

  return item.Info();
}



// ----------------------------------------------------------------------------
qreal tGrafik::showDia(qreal ms, qreal Startwert, qreal Decrement)
{ //qDebug()<<"DiaRestwert="<<DiaRestwert<<"DiaStart"<<DiaStart;

  //if   {  DiaRestwert=0; return DiaRestwert;}
  if ((ms<=0) || (!DiaStart)) {  DiaRestwert=0; return DiaRestwert;}

  if (Decrement<=0) Decrement=1;

  setAutoPaint(true);
  if (DiaRestwert<=0)
  { DiaRestwert=Startwert-Decrement;  // Neustart
    //qDebug()<<"DiaRestwert<0"<<DiaRestwert<<Decrement;
  }
  else
  { DiaRestwert= DiaRestwert - Decrement;
    //qDebug()<<"DiaRestwert>=0"<<DiaRestwert<<Decrement;
  }

 if (DiaRestwert>0) //Update
 { QTimer::singleShot(ms, this, SLOT(update()));
   //qDebug()<<"singleShot"<<DiaRestwert<<Decrement;
 }
 else
 { DiaRestwert=-1;
   DiaStart=false;
   //qDebug()<<"fertig="<<DiaRestwert<<Decrement;
 }

 return DiaRestwert;
}

// ----------------------------------------------------------------------------
qreal tGrafik::rnd(qreal max)
{  return Random.qnext(max);
}

// ----------------------------------------------------------------------------
qreal tGrafik::rndW(qreal min, qreal max)
{ if (Random.qnext(2.0)>1)
        return -min-Random.qnext(max-min);
  else  return  min+Random.qnext(max-min);
}


QString tGrafik::rndRGB1()
{  return Random.rgb1();
}

QString tGrafik::rndRGB(qreal rnd)
{  return Random.rgbRGB(rnd);
}

void tGrafik::setRndRGB(const QString &RGB1, const QString &RGB2)
{ Random.setRGB(RGB1,RGB2);
}

void tGrafik::setRndHeller(int Heller, int rndHeller)
{ Random.setLighter(Heller,rndHeller);
}

QString tGrafik::rndRGB2()
{ return Random.rgb2();
}

QString tGrafik::rgbName(int rot, int gruen, int blau)
{ return QColor(rot,gruen,blau).name();
}




void tGrafik::msg(QVariant s, const QString rgb)
{ MsgBox->addLn(s.toString(), QColor(rgb).rgb());
}

void tGrafik::clrMsg()
{ MsgBox->clear();
}


//-----------------------------------------------------------------------------
void tGrafik::msgPolygon(bool Spline)
{
  QString s(
        "function defPoly()\n"
        "{t.setNoStyle();\n t.beginPolygon();\n");

  for(int i=0; i<Polygon().size(); i++)
   { if (i==0) s+=" t.goTo"+Punkt(Polygon().at(i))+";\n";
     else      s+=" t.moveTo"+Punkt(Polygon().at(i))+";\n";
   }

  s+=" t.endPolygon("+Int(PolyFillMode,0)+");\n";
  if (Spline) s+=" t.splinePolygon();\n";
  s+="}\n";

  MsgBox->addLn(s, tMsgBox::inf1);
}


//-----------------------------------------------------------------------------
void tGrafik::msgScript(bool showWerte)
{
  MsgBox->addLn(infoScript(showWerte),tMsgBox::inf1);
}


//-----------------------------------------------------------------------------
QString tGrafik::infoScript(bool showWerte)
{QString s="'"+objectName()+"': Objekte";

  if (showWerte)  s+=" und Werte\n";

  if (!ScriptEngine)  { s+="\nKein Script aktiv!"; return s;}

  QScriptValue object=ScriptEngine->globalObject();

  QScriptValueIterator it(object);
  while (it.hasNext())
    { it.next();
      // Systemobjekte ausblenden
      if (it.flags() & QScriptValue::SkipInEnumeration) continue;
      if (it.name()=="print") continue;
      if (it.name()=="gc") continue;
      if (it.name()=="version") continue;
      if (showWerte) s+="\n"+it.name()+": " +it.value().toString();
      else           s+="\n"+it.name();
    }
  s+="\n";
  return s.replace( QChar(0xA),"\n").replace( QChar(8233),"\n");
}


// Ende tGrafik <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





