#include "_turtle.h"
#include "setup.h"
#include "utils/msg.h"
#include "utils/utils.h"
#include "_grafik.h"
#include "drawbefehle.h"



extern tMsgBox   *MsgBox;
extern tSetup    *Setup;


// tTurtle >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//
tTurtle::tTurtle(QWidget *parent, tGrafik *grafik) : QWidget(parent),
  TurtleSetup(Setup->TurtleDefaults()),
  Last(0,0),
  Richtung(0),
  PaintAll(true),
  AutoPaint(true),
  DiaStart(false),  
  PolyAdd(false),
  PolyFillMode(NoCloseNoFill),
  getFnktText(0),
  ToolRand(0),
  beginUseSetup(false),
  PDFPrinter(0),
  Grafik(grafik)
{
  // Defaultwerte aus Setup kopieren
  //TurtleSetup= Setup->TurtleDefaults();

}

tTurtle::~tTurtle() {}

QSize tTurtle::minimumSizeHint() const
{ return QSize(100, 141); //dinA4quer
}

QSize tTurtle::sizeHint() const
{ return QSize(705,500); //dinA4quer
}


// ----------------------------------------------------------------------------
void tTurtle::setNoStyle()
{ setPen("", -1, Qt::NoPen);
  setBrush("", Qt::NoBrush);
}


// ----------------------------------------------------------------------------
void tTurtle::setPen(QPen Pen)
{ Stift=Pen;
  if (Painter.isActive()) Painter.setPen(Stift);
}


// ----------------------------------------------------------------------------
void tTurtle::setPen(const QString &Rgb, qreal width, int aStyle)
{
  if (aStyle<0)
  {  Stift.setStyle(LastStiftStyle);
  }
  else
  { Qt::PenStyle style=  (Qt::PenStyle) aStyle;
    if (style==Qt::NoPen)
     { if (Stift.style()!=Qt::NoPen)
        { LastStiftStyle=Stift.style();
        }
     }
    else  LastStiftStyle=style;
    Stift.setStyle(style);
  }

  if (Rgb!="") {Stift.setColor(Rgb);}

  if (width>=0) Stift.setWidthF(width);

  if (getFnktText) getFnktText->append(QString(
  "t.setPen(\"%1\",%2,%3);\n").arg(Rgb).arg(width).arg(aStyle));

  setPen(Stift);
}



// ----------------------------------------------------------------------------
void tTurtle::setPenWidth(qreal width, int CapStyle)
{ if (width>=0) Stift.setWidthF(width);

  if (CapStyle>=0) { Stift.setCapStyle((Qt::PenCapStyle) CapStyle); }

  //Stift.setJoinStyle(Qt::MiterJoin); // Qt::BevelJoin,Qt::RoundJoin

  if (getFnktText) getFnktText->append(QString(
  "t.setPenWidth(%1,%2);\n").arg(width).arg(CapStyle));

  setPen(Stift);
}



// ----------------------------------------------------------------------------
void tTurtle::setBrush(QBrush Brush)
{ Pinsel=Brush;
  if (Painter.isActive()) Painter.setBrush(Pinsel);
}


// ----------------------------------------------------------------------------
void tTurtle::setBrush(const QString &Rgb, int aStyle, int Alpha)
{
  if (aStyle<0)
  { Pinsel.setStyle(LastPinselStyle);
  }
  else
  { Qt::BrushStyle style= (Qt::BrushStyle) aStyle;
    if (style==Qt::NoBrush)
    { if (Pinsel.style()!=Qt::NoBrush)
       { LastPinselStyle=Pinsel.style();
       }
    }
    else  LastPinselStyle=style;
    if (style==Qt::LinearGradientPattern)
    { Pinsel= QBrush (LGradient);
    }
    Pinsel.setStyle(style);
  }

  if (Alpha>-1)
  { QColor c(Rgb); c.setAlpha(Alpha);
    Pinsel.setColor(c);
  }
  else  if (Rgb!="") { Pinsel.setColor(Rgb); }

  if (getFnktText) getFnktText->append(QString(
  "t.setBrush(\"%1\",%2,%3);\n").arg(Rgb).arg(aStyle).arg(Alpha));

  if (Painter.isActive()) Painter.setBrush(Pinsel);
}


// ----------------------------------------------------------------------------
void tTurtle::setGradient(qreal x1, qreal y1, qreal x2, qreal y2,
                          const QString &Rgb1,const QString &Rgb2)
{ LGradient.setStops( QGradientStops ());
  LGradient.setStart(x1,y1);
  LGradient.setFinalStop(x2,y2);

  LGradient.setColorAt(0, Rgb1);
  LGradient.setColorAt(1, Rgb2);
  //LGradient.setCoordinateMode(QGradient::StretchToDeviceMode);
  //LGradient.setCoordinateMode(QGradient::ObjectBoundingMode);
}

// ----------------------------------------------------------------------------
void tTurtle::setGradient(qreal t, const QString &Rgb)
{ LGradient.setColorAt(t, Rgb);
}

// ----------------------------------------------------------------------------
void tTurtle::turn(qreal w)
{ Richtung +=w*TwoPi/360;
  if ((Richtung < 0))  while (Richtung < -TwoPi) Richtung += TwoPi;
  else                 while (Richtung >  TwoPi) Richtung -= TwoPi;
  if (getFnktText) getFnktText->append(QString("t.turn(%1);\n").arg(w,0,'g',8));
  }

// ----------------------------------------------------------------------------
void tTurtle::turnTo(qreal w)
{ Richtung=w*TwoPi/360;
  if ((Richtung < 0))  while (Richtung < -TwoPi) Richtung += TwoPi;
  else                 while (Richtung >  TwoPi) Richtung -= TwoPi;
  if (getFnktText) getFnktText->append(QString("t.turnTo(%1);\n").arg(w,0,'g',8));
}


// ----------------------------------------------------------------------------
void tTurtle::turnTo(qreal x, qreal y)
{ Richtung=wAlpha(QPointF(x,y) -Last)*TwoPi/360;
  if ((Richtung < 0))  while (Richtung < -TwoPi) Richtung += TwoPi;
  else                 while (Richtung >  TwoPi) Richtung -= TwoPi;
  if (getFnktText) getFnktText->append(QString("t.turnTo(%1,%2);\n").arg(x,0,'g',8).arg(y,0,'g',8));
}


// ----------------------------------------------------------------------------
void tTurtle::turnTo(const QString Name)
{ QPointF P=tMainWin::BefehleLst.Befehl(Name).getP0();
  turnTo(P.x(), P.y());
}

// ----------------------------------------------------------------------------
void tTurtle::move(qreal mm)
{ QPointF Neu( ::cos(Richtung)*mm, ::sin(Richtung)*mm);
  Neu=Last+Neu;
  if (Painter.isActive()) Painter.drawLine(Last,Neu);
  Last=Neu;
  if (PolyAdd) Poly.append(Last);
  if (getFnktText) getFnktText->append(QString("t.move(%1);\n").arg(mm,0,'g',8));
}


// ----------------------------------------------------------------------------
void tTurtle::moveTo(qreal x, qreal y)
{ QPointF Neu;
  Neu.setX(x);
  Neu.setY(y);
  if (Painter.isActive()) Painter.drawLine(Last,Neu);
  Last=Neu;
  if (PolyAdd) Poly.append(Last);
  if (getFnktText) getFnktText->append(QString("t.moveTo(%1,%2);\n").arg(x,0,'g',8).arg(y,0,'g',8));
}

//-----------------------------------------------------------------------------
void tTurtle::moveTo(const QString Name)
{
  QPointF P=tMainWin::BefehleLst.Befehl(Name).getP0();
  moveTo(P.x(), P.y());
}


// ----------------------------------------------------------------------------
void tTurtle::goTo(qreal x, qreal y)
{ Last.setX(x); Last.setY(y);
  if (PolyAdd) Poly.append(Last);
  if (getFnktText) getFnktText->append(QString("t.goTo(%1,%2);\n").arg(x,0,'g',8).arg(y,0,'g',8));
}

//-----------------------------------------------------------------------------
void tTurtle::goTo(const QString Name, int Index)
{
  QPointF P=tMainWin::BefehleLst.Befehl(Name).getP0(Index);
  goTo(P.x(), P.y());
}


//-----------------------------------------------------------------------------
void tTurtle::drawPoint(qreal x, qreal y)
{ if (getFnktText) getFnktText->append(QString("t.drawPoint(%1,%2);\n").arg(x,0,'g',8).arg(y,0,'g',8));

  if (Painter.isActive())
  { Painter.save();
    Painter.setBrush(Qt::NoBrush);
    //Setup.setFangRect(FangRect);
    Painter.drawRect(FangRect.translated(x,y));
    Painter.restore();
  }

}

// ----------------------------------------------------------------------------
qreal tTurtle::lastX()
{ if (getFnktText) getFnktText->append(QString("t.lastX();\n"));
  return Last.x();
}

qreal tTurtle::lastY()
{ if (getFnktText) getFnktText->append(QString("t.lastY();\n"));
  return Last.y();
}

qreal tTurtle::lastW()
{ if (getFnktText) getFnktText->append(QString("t.lastW();\n"));
  return Richtung * 180 /Pi;
}


// ----------------------------------------------------------------------------
double tTurtle::getL()
{ if (getFnktText) getFnktText->append(QString("t.getL();\n"));
  return Betrag(Last);
}


// ----------------------------------------------------------------------------
qreal tTurtle::getW()
{ if (getFnktText) getFnktText->append(QString("t.getW();\n"));
  return wAlpha(Last);
}


// ----------------------------------------------------------------------------
void tTurtle::msgTurtle()
{ QString s
      ( "Turtle:\n"
        "t.setPen(\""+Stift.color().name()+"\", "
                     +Real(Stift.widthF())+", "
                     +Int(Stift.style(),0)+");\n"
        "t.setBrush(\""+Pinsel.color().name()+"\", "
                       +Int(Pinsel.style(),0)+", "
                       +Int(Pinsel.color().alpha(),0)+");\n"
        "t.goTo("+Real(lastX())+","+Real(lastY())+");\n"
        "t.turnTo("+Real(lastW())+");\n"
        "t.setFont(\""+Painter.font().family()+"\", "
                    +Real(Painter.font().pointSizeF()*FontScale )+", "
                    +Bool(Painter.font().fixedPitch())+");\n"
        "t.setAutoPaint("+Bool(AutoPaint)+");\n"
        "t.setAntiAliasing("+Bool(TurtleSetup.AntiAliasing)+");\n"

        );
  MsgBox->addLn(s,tMsgBox::inf1);
}


// ----------------------------------------------------------------------------
void tTurtle::rect(QRectF r)
{ if (!Painter.isActive()) return;
  qreal grad=Richtung*180/Pi;
  Painter.save();
  Painter.translate(Last.x(),Last.y());
  Painter.rotate(grad);
  Painter.drawRect(r);
  Painter.restore();
}

// ----------------------------------------------------------------------------
void tTurtle::rectangle(qreal dx, qreal dy)
{ rect( QRectF (0,0,dx,dy));
  if (getFnktText) getFnktText->append(QString(
   "t.rectangle(%1,%2);").arg(dx).arg(dy));
}


// ----------------------------------------------------------------------------
void tTurtle::clrPolygon()
{ Poly.clear();
  PolyAdd=false;
  PolyFillMode=NoCloseNoFill;
  if (getFnktText) getFnktText->append(QString("t.clrPolygon();\n"));
}

// ----------------------------------------------------------------------------
qreal tTurtle::distanceToPolygon(qreal maxl)
{
  if (getFnktText) getFnktText->append(QString("t.distanceToPolygon(%1);\n").arg(maxl));

  QPointF Neu( ::cos(Richtung)*maxl, ::sin(Richtung)*maxl);

  return boundedIntersection(Poly, Last, Last+Neu);
}


// ----------------------------------------------------------------------------
void tTurtle::rectPolygon(qreal dx, qreal dy, int FillMode)
{
  qreal x0=lastX() ,y0=lastY(),w0=lastW();
  beginPolygon();
  goTo(x0,y0);
  move(dx); turn(90);
  move(dy); turn(90);
  move(dx);
  endPolygon(FillMode);
  goTo(x0,y0); turnTo(w0);

  // if (getFnktText)
  // getFnktText->append(QString("t.rectPolygon(%1,%2,%3);\n").arg(dx,0,'g',8).arg(dy,0,'g',8).arg(Fill));
}


// ----------------------------------------------------------------------------
void tTurtle::trianglePolygon(qreal c, qreal wb, qreal b, int FillMode)
{
  qreal x0=lastX() ,y0=lastY(),w0=lastW();
  beginPolygon();
  goTo(x0,y0);
  move(c); turn(180-wb); move(b);
  endPolygon(FillMode);
  goTo(x0,y0); turnTo(w0);

}

// ----------------------------------------------------------------------------
void tTurtle::isoPolygon(qreal r, int n, int FillMode)
{
  if ((-epsilon<r) && (r<epsilon)) return;

  qreal x0=lastX() ,y0=lastY(),w0=lastW();

  if (n<0) { n= ceil(r-10.0*n); if(n<3) n=3; }
  double w=360.0/(double)n;
  double l=2.0*r*sin(Pi/(double) n);

  goTo(x0,y0);
  beginPolygon();
  move(r); turn(90+w/2);
  for(int i=1; i<n; i++) {move(l); turn(w);}
  endPolygon(FillMode);
  goTo(x0,y0); turnTo(w0);
}


// ----------------------------------------------------------------------------
void tTurtle::splinePolygon(qreal dt)
{
  QPolygonF temp(Poly);
  spline(temp,Poly,PolyFillMode,dt);

  if (getFnktText) getFnktText->append(QString("t.splinePolygon(%1);\n").arg(dt,0,'g',8));
}


// ----------------------------------------------------------------------------
void tTurtle::cPolygon()
{ if (getFnktText) getFnktText->append(QString("cPolygon();\n"));

  if (Poly.size()<1) return;

  QPointF Q=Poly.last();

  QMatrix m;
  m.translate(Q.x(),Q.y()); m.rotate(180); m.translate(-Q.x(),-Q.y());
  for (int i=Poly.size()-2; i>=0; i--)
      Poly.append( m.map(Poly.at(i)));
}

// ----------------------------------------------------------------------------
void tTurtle::beginPolygon()
{ clrPolygon();
  PolyAdd=true;
  PolyFillMode=NoCloseNoFill;

  if (getFnktText) getFnktText->append(QString("t.beginPolygon();\n"));
}


// ----------------------------------------------------------------------------
void tTurtle::endPolygon(int Fill)
{ PolyAdd=false;
  PolyFillMode=tFillMode(Fill);

  if (getFnktText) getFnktText->append(QString(
  "t.endPolygon(%1);\n").arg(Fill));

  switch (Poly.size())
   { case 0:
       return;

     case 1:
     case 2:
       if (Fill<3) MsgBox->addLn("Err: Polygon nicht definiert!", tMsgBox::err);
       return;

     default:
       if (Fill<=CloseNoFill)
        {  // Polygon schliessen
          if (Poly.first()!=Poly.last()) Poly.append(Poly.first());
        }
       return;
   }
}


// ----------------------------------------------------------------------------
void tTurtle::drawPolygon(const QPolygonF &aPoly)
{ 
  switch (PolyFillMode)
   {
    case OddEvenFill:
        Painter.drawPolygon(aPoly, Qt::OddEvenFill);
        break;

     case WindingFill:
        Painter.drawPolygon(aPoly, Qt::WindingFill);
        break;

     case NoCloseNoFill:
     case CloseNoFill:
     default:
         Painter.drawPolyline(aPoly);
         break;
   }
}


//-----------------------------------------------------------------------------
void tTurtle::movePolygon(qreal x, qreal y)
{ if (getFnktText)
    getFnktText->append(QString("t.movePolygon(%1,%2);\n").arg(x,0,'g',8).arg(y,0,'g',8));

  if (Poly.size()==0) return;

  QMatrix m;

  Poly=m.translate(x - Poly.first().x(),
                   y - Poly.first().y()).map(Poly);

}


// ----------------------------------------------------------------------------
void tTurtle::drawPolygon()
{ if (getFnktText) getFnktText->append(QString("t.drawPolygon();\n"));
  drawPolygon(lastX(), lastY(),
              lastW(),
              Poly);
}



// ----------------------------------------------------------------------------
void tTurtle::drawPolygon(qreal dx, qreal dy, qreal grad, const QPolygonF &aPoly)
{
  Painter.save();
  Painter.translate( dx, dy);
  Painter.rotate(grad);   // in Grad!
  drawPolygon(aPoly);
  Painter.restore();
}


// ----------------------------------------------------------------------------
void tTurtle::text(const QString &s)
{  text(Last.x(), Last.y(), s);

}

// ----------------------------------------------------------------------------
void tTurtle::text(qreal x, qreal y, const QString &s)
{ if (!Painter.isActive()) return;
  Last.setX(x); Last.setY(y);

  Painter.save();
  Painter.translate(x,y);
  Painter.rotate(lastW());
  Painter.scale(1,-1);
  Painter.translate(-x,-y);
  Painter.drawText(x,y, s );
  Painter.restore();
}

void tTurtle::text(qreal x, qreal y, qreal x1, qreal y1, const QString &s)
{ if (!Painter.isActive()) return;
  Last.setX(x); Last.setY(y);
  QRectF R(x,-y,x1-x,y-y1);
  Painter.save();
  Painter.rotate(lastW());
  Painter.scale(1,-1);
  Painter.drawText(R,Qt::AlignLeft | Qt::TextWordWrap, s );
  Painter.restore();
}


//-----------------------------------------------------------------------------
void tTurtle::setFont(const QString &FontName, qreal size, bool FixPitch, int Weight)
{  QFont f(FontName);
   f.setFixedPitch(FixPitch);
   f.setPointSizeF(size/FontScale);
   f.setLetterSpacing(QFont::AbsoluteSpacing,0);
   f.setWeight(Weight);
   TurtleSetup.FontName=FontName;
   TurtleSetup.setFontSize_mm(size);
   if (Painter.isActive()) Painter.setFont(f);
}

//-----------------------------------------------------------------------------
void tTurtle::setWeight(int Weight)
{  QFont f(Painter.font());
   f.setWeight(Weight);
   if (Painter.isActive()) Painter.setFont(f);
}

//-----------------------------------------------------------------------------
qreal tTurtle::bild(const QString &DateiName, qreal w_mm, qreal h_mm)
{ qreal l=0;
  if (!Painter.isActive()) return l;
  if ((w_mm<0) && (h_mm<0)) return l;

  QPixmap pixmap(DateiName);
  if (pixmap.isNull()) {MsgBox->addLn("Err Bild '"+DateiName+"' nicht gefunden!",tMsgBox::err); return l;}

  QRectF source(0.0, 0.0, pixmap.width(), pixmap.height());

  if (h_mm<0) { h_mm= w_mm * pixmap.height()/ pixmap.width();  l=h_mm;}
  if (w_mm<0) { w_mm= h_mm * pixmap.width() / pixmap.height(); l=w_mm;}

  QRectF ziel(0,0,w_mm, h_mm);
  ziel.translate(0,-h_mm);

  Painter.save();
  Painter.scale(1,-1);
  Painter.translate(Last.x(),-Last.y());
  Painter.rotate(-lastW());
  Painter.drawPixmap(ziel, pixmap, source);

  Painter.restore();
  setBrush("",-1,Qt::NoBrush);

  ziel.translate(0,h_mm);
  rect(ziel);
  return l;
}


// ----------------------------------------------------------------------------
void tTurtle::setPage(qreal x0, qreal y0, int din)
{ if (!Painter.isActive())
  { TurtleSetup.setPage(x0,y0, din);
    //TurtleSetup.setFangRect(FangRect);
    beginUseSetup=true;    
  }
  if (getFnktText) getFnktText->append(QString(
  "t.setPage(%1,%2,%3);\n").arg(x0).arg(y0).arg(din));
}

// ----------------------------------------------------------------------------
void tTurtle::setPage(int din)
{ if (!Painter.isActive())
  { TurtleSetup.setPage( din);
    //TurtleSetup.setFangRect(FangRect);
    beginUseSetup=true;
  }
  if (getFnktText) getFnktText->append(QString(
  "t.setPage(%1);\n").arg(din));
}

// ----------------------------------------------------------------------------
void tTurtle::setMargin(qreal RandX, qreal RandY)
{ if (!Painter.isActive())
  { TurtleSetup.setMargin(RandX, RandY);
    beginUseSetup=true;
  }
  if (getFnktText) getFnktText->append(QString(
  "t.setMargin(%1,%2);\n").arg(RandX).arg(RandY));

}

// ----------------------------------------------------------------------------
qreal tTurtle::P0x()
{ return TurtleSetup.getP0P1().topLeft().x();
}
// ----------------------------------------------------------------------------
qreal tTurtle::P0y()
{ return TurtleSetup.getP0P1().topLeft().y();
}
// ----------------------------------------------------------------------------
qreal tTurtle::P1x()
{ return TurtleSetup.getP0P1().bottomRight().x();
}
// ----------------------------------------------------------------------------
qreal tTurtle::P1y()
{ return TurtleSetup.getP0P1().bottomRight().y();
}


// ----------------------------------------------------------------------------
void tTurtle::setAutoPaint(bool on)
{  AutoPaint=on;

}

// ----------------------------------------------------------------------------
void tTurtle::setAntiAliasing(bool on)
{  TurtleSetup.AntiAliasing=on;
   if (getFnktText) getFnktText->append(QString(
   "t.setAntiAliasing(%1);\n").arg(on));
}


qreal tTurtle::FontSize_mm()
{ return Painter.font().pointSizeF()*FontScale;
}


void tTurtle::rahmen()
{   setBrush("",-1,Qt::NoBrush);
    Painter.setClipping(false);
    Painter.drawRect(TurtleSetup.getPlotRect());
    Painter.setClipping(true);
}


QTransform const &tTurtle::getWinMatrix() const
{ if (!Painter.isActive()) MsgBox->addLn("Err: WinMatrix ungültig",tMsgBox::err);
  return WinMatrix;
}
//-----------------------------------------------------------------------------
const QTransform &tTurtle::setWinMatrix()
{ QRectF P0P1= TurtleSetup.getP0P1();

  qreal scale = (width()-ToolRand) / P0P1.width();
  qreal scale1= height() / -P0P1.height();
  if (scale>scale1) scale=scale1;

  WinMatrix.reset();
  WinMatrix.scale(scale, -scale);
  WinMatrix.translate( -P0P1.left() , -P0P1.top());
  return WinMatrix;
}


//-----------------------------------------------------------------------------
void tTurtle::setPDFMatrix(QPrinter *Printer)
{
  qreal scale= Printer->resolution()/TurtleSetup.getPDF_DPI();
  // Laserjet 4: Dots-Per-Inch / 25.4mm
  // LserJet Pro 400: Dots-Per-Inch / 24.32mm
  QRectF P0P1= TurtleSetup.getP0P1();

  //QRectF rect= Printer->pageRect(QPrinter::Millimeter);
  //QRectF rect= Printer->paperRect(QPrinter::Millimeter);
  //MsgBox->addLn(QString("Drucker dpi: %1").arg(Printer->resolution()));
  //qDebug()<<"fullPag="<<Printer->fullPage();
  //qDebug()<<"pageRect="<<rect<<rect.left()<<rect.top();
  //qreal left, top, right, bottom;
  //Printer->getPageMargins(&left, &top, &right, &bottom,QPrinter::Millimeter);
  //qDebug()<<"getPageMargins"<<left<<top<<right<<bottom;

  PDFMatrix.reset();
  PDFMatrix.scale(scale, -scale);

  PDFMatrix.translate( -P0P1.left()- TurtleSetup.getPDF_P0_dX(),
                       -P0P1.top() + TurtleSetup.getPDF_P0_dY() );

  //qDebug()<<"PDF_DPI_mm  ="<<TurtleSetup.getPDF_DPI();
  //qDebug()<<"PDF_P0_dX_mm="<<TurtleSetup.getPDF_P0_dX();
  //qDebug()<<"PDF_P0_dY_mm="<<TurtleSetup.getPDF_P0_dY();
}


//-----------------------------------------------------------------------------
void tTurtle::setPDFPage(QPrinter *)
{ int din= (tSetupTurtle::tDin) TurtleSetup.getDin();

  //QPrinter::PageSize PageSize;
  QPrinter::PaperSize PageSize;
  switch (din)
   {case tSetupTurtle::dinA0quer:
    case tSetupTurtle::dinA0hoch: PageSize=QPrinter::A0; break;

    case tSetupTurtle::dinA1quer:
    case tSetupTurtle::dinA1hoch: PageSize=QPrinter::A1; break;

    case tSetupTurtle::dinA2quer:
    case tSetupTurtle::dinA2hoch: PageSize=QPrinter::A2; break;

    case tSetupTurtle::dinA3quer:
    case tSetupTurtle::dinA3hoch: PageSize=QPrinter::A3; break;

    case tSetupTurtle::dinA5quer:
    case tSetupTurtle::dinA5hoch: PageSize=QPrinter::A5; break;

    case tSetupTurtle::dinA6quer:
    case tSetupTurtle::dinA6hoch: PageSize=QPrinter::A6; break;

    case tSetupTurtle::dinA7quer:
    case tSetupTurtle::dinA7hoch: PageSize=QPrinter::A7; break;

    case tSetupTurtle::dinA8quer:
    case tSetupTurtle::dinA8hoch: PageSize=QPrinter::A8; break;

    case tSetupTurtle::dinA9quer:
    case tSetupTurtle::dinA9hoch: PageSize=QPrinter::A9; break;

    case tSetupTurtle::dinA4quer:
    case tSetupTurtle::dinA4hoch:
    default:                      PageSize=QPrinter::A4;
  }

  PDFPrinter->setFullPage(true);
  //PDFPrinter->setPageSize(PageSize);
  PDFPrinter->setPaperSize(PageSize);

  if (din>=0x0a) PDFPrinter->setOrientation(QPrinter::Portrait);
  else {PDFPrinter->setOrientation(QPrinter::Landscape); }

  //qDebug()<<"din"<<din<<"PageSize"<<PageSize<<"Orientation"<<PDFPrinter->orientation();
}

//-----------------------------------------------------------------------------
void tTurtle::init()
{
  TurtleSetup= ::Setup->TurtleDefaults();
  LastStiftStyle=TurtleSetup.Stift.style();
  LastPinselStyle=TurtleSetup.Pinsel.style();
  beginUseSetup=false;
  getFnktText=0;
  Poly.clear();
  Random.clear();
}



//-----------------------------------------------------------------------------
bool tTurtle::begin()
{ if (!beginUseSetup) { Stift=TurtleSetup.Stift; Pinsel=TurtleSetup.Pinsel;}

  if (PDFPrinter!=0)
  { setPDFPage(PDFPrinter);

    if (!Painter.begin(PDFPrinter)) return false;

    setPDFMatrix(PDFPrinter);
    Painter.setWorldTransform(PDFMatrix);
    Painter.setRenderHint(QPainter::Antialiasing, TurtleSetup.AntiAliasing);

    Painter.setPen(Stift);
    Painter.setBrush(Pinsel);
    setFont(TurtleSetup.FontName, TurtleSetup.getFontSize_mm());

    Painter.drawRect(TurtleSetup.getPlotRect());
    Painter.setClipRect(TurtleSetup.getPlotRect());
    TurtleSetup.setFangRect(FangRect);
  }
  else
  { if (!Painter.begin(this)) return false;

    //setWinMatrix();
    Painter.setWorldTransform( setWinMatrix() );
    Painter.setRenderHint(QPainter::Antialiasing, TurtleSetup.AntiAliasing);

    Painter.setPen(Stift);
    Painter.setBrush(Pinsel);
    setFont(TurtleSetup.FontName, TurtleSetup.getFontSize_mm());

    Painter.drawRect(TurtleSetup.getPlotRect());
    Painter.setClipRect(TurtleSetup.getPlotRect());
    TurtleSetup.setFangRect(FangRect);

    if (!PaintAll)
     { text(TurtleSetup.getInfoPosX(),TurtleSetup.getInfoPosy(),
            "No AutoPaint: Doppelklick oder PDF Ausgabe");
       return false;
     }
  }
  Last.setX(0); Last.setY(0); Richtung=0;
  Stift=TurtleSetup.Stift; Pinsel=TurtleSetup.Pinsel;
  return true;
}


//-----------------------------------------------------------------------------
void tTurtle::paintEvent(QPaintEvent * event)
{
  if ((Painter.isActive()) || (PDFPrinter!=0))
  { event->ignore(); return;
  }

  QApplication::setOverrideCursor(Qt::WaitCursor);

  //PDFPrinter=0;
  FontScale=1;
  Pinsel=TurtleSetup.Pinsel; // Defaultwerte
  Stift=TurtleSetup.Stift;

  Grafik->draw();

  PaintAll=AutoPaint;
  Painter.end();
  QApplication::setOverrideCursor(Qt::ArrowCursor);
  event->accept();

  emit DrawEnde(objectName());
}


//-----------------------------------------------------------------------------
void tTurtle::printPDF(QPrinter *Printer)
{ QApplication::setOverrideCursor(Qt::WaitCursor);
  PDFPrinter=Printer;
  FontScale=PDFFontScale;

  Stift=TurtleSetup.Stift;
  Pinsel=TurtleSetup.Pinsel; // default

  Grafik->draw();

 // Painter.endNativePainting();
  Painter.end();
  PDFPrinter=0;
  QApplication::setOverrideCursor(Qt::ArrowCursor);
}


//-----------------------------------------------------------------------------
void tTurtle::mousePressEvent(QMouseEvent *event)
{  if (event->type()==QEvent::MouseButtonPress)
  { QPointF P(setWinMatrix().inverted().map(event->posF()));
    QApplication::clipboard()->setText(
      QString(" t.moveTo(%1,%2);").arg(P.rx(),0,'g',2).arg(P.ry(),0,'g',2));
   }
  if (event->type()==QEvent::MouseButtonDblClick)
   { PaintAll=true; DiaStart=true; update();}

}


//-----------------------------------------------------------------------------
void tTurtle::spline(QPolygonF &qPoly, QPolygonF &SplinePoly, int FillMode, qreal dt)
{
  SplinePoly.clear();

  int i=0, imax=qPoly.size()-1;
  QPointF A,B,C, P0;

  if (imax<2) return;
  if ((dt<0) && (dt<epsilon)) dt=epsilon;

  if (FillMode<NoCloseNoFill)
  { // Polygon schliessen
    if (qPoly.first()!=qPoly.last())
     { qPoly.append(qPoly.first()); imax++;}

    // den neuen Startpunkt P0 Berechnen
    P0=0.5*(qPoly.at(0)+qPoly.at(1));
    C=P0; i=1;
  }
  else C=qPoly.at(0); i=1;

  while (i<=imax)
  { A=C;
    B=qPoly.at(i); i++;
    if (i<imax)
     { C=0.5*(B+qPoly.at(i)); }
    else
     { if (FillMode==NoCloseNoFill)
       { C=qPoly.at(imax); i++; }
       else
       { if (i==imax) C=0.5*(B+qPoly.at(imax));
         else { C=P0; i++;}
       }
     }

    // 0<=t<1, Zwischenpunkte berechnen
    SplinePoly.append(A); qreal t=dt;
    while (t<1)
    { SplinePoly.append( ((1-t)*(1-t))*A + (2-2*t)*t*B+ t*t*C );
      t+=dt;
    }
  }
  SplinePoly.append(C); // letzter Punkt
}


// tTurtleStatus >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//
tTurtleStatus tTurtle::TurtleStatus(bool savePoly, bool stopRec, bool saveTurtle)
{ tTurtleStatus status;
  status.saveTurtle=saveTurtle;
  status.savePoly=savePoly;

  if (saveTurtle)
  { status.Last=Last;
    status.Richtung=Richtung;
    status.Stift=Stift;
    status.LastStiftStyle=LastStiftStyle;
    status.Pinsel=Pinsel;
    status.LastPinselStyle=LastPinselStyle;

    status.PolyAdd=PolyAdd;
    status.PolyFillMode=PolyFillMode;
  }

  if (savePoly) status.Poly=Poly;

  if (stopRec)
  { // Aufzeichnung stop
    status.RecDrawName=RecDrawName;
    getFnktText=0;
    RecDrawName.clear();
  }
  else status.RecDrawName.clear();

  return status;
}

void tTurtle::setTurtleStatus(const tTurtleStatus &status)
{ if (status.saveTurtle)
  { Last=status.Last;
    Richtung=status.Richtung;
    setBrush(status.Pinsel);
    LastStiftStyle=status.LastStiftStyle;
    setPen(status.Stift);
    LastPinselStyle=status.LastPinselStyle;
    PolyAdd=status.PolyAdd;

    PolyFillMode=status.PolyFillMode;
  }

  if (status.savePoly) Poly=status.Poly;

  if (!status.RecDrawName.isEmpty())
  { // Aufzeichnung fortsetzen
    RecDrawName=status.RecDrawName;
    getFnktText=tMainWin::BefehleLst.getFnktText(RecDrawName);
  }
}



