#ifndef TURTLE_H
#define TURTLE_H
#include <QtGui>
#include "setup.h"
#include "utils/utils.h"

class tGrafik;
class tSetupTurtle;
class tTurtleStatus;
class QScriptValue;
class tRndFarbe;

QString subRgb(int cyan, int magenta, int yellow);


// tTurtle >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// Zeichenbefehle für Turtlegrafiken und Painter.
// Zeichnet im Widget oder in eine PDF-Datei.
// Alle 'public slots:' können in Scripts verwendet werden.
//
class tTurtle : public QWidget
{  Q_OBJECT
public:
  tTurtle(QWidget *parent, tGrafik *grafik);
  virtual ~tTurtle();

  QSize minimumSizeHint() const;  // Hinweise fuer Widget
  QSize sizeHint() const;

  enum tFillMode { OddEvenFill   =Qt::OddEvenFill,
                   WindingFill   =Qt::WindingFill,
                   CloseNoFill   =Qt::WindingFill+1,
                   NoCloseNoFill =CloseNoFill+1};

public slots:
   /*! Begin class tTurtle Javascript ************************************/
   /*! |t.Turtle|*********************************************************/
    //Basisbefehle für die Turtle

  /*! |Einstellungen|****************************************************/
       //Einstellungen sind nur in init() wirksam.
       //Alle setPage() Befehle setzen Pen und Brush.

   void setPage(qreal x0, qreal y0, int dinA=255);
       //Papierformat und Koordinatenursprung einstellen.
       //Linken obere Ecke: (x0,y0) in mm.
       //dinA: 0-9 Querformat. 10-19 Hochformat. 255 Setup.

  void setPage(int din=255);
       //Koordinatenursprung in Blattmitte
       //dinA: 0-9 Querformat. 10-19 Hochformat. 255 Setup.

  void setMargin(qreal RandX=-1, qreal RandY=-1);
       //RandX=-1 oder RandY=-1: Rand aus Setup.
       //RandX=-2 oder RandY=-2: Zeichenrechteck quadratisch.

  void setAutoPaint(bool on=true);
       //Automatisches Neuzeichnen abschalten.
       //true:  Grafik immer automatisch neu zeichen.
       //false: Grafik erst nach Mausklick neu zeichnen.

  void setAntiAliasing(bool on=true) ;
       //Antialaising. On: feinere Linien, langsamer.

  /*! |Pen und Brush|****************************************************/
       //Zeichenstift und Malfarbe

  void setNoStyle();
       //Turtle zeichnet unsichtbar.
       //Aktuellen Pen und Brush merken.
       //Pen=0 und Brush=0 setzen.

  void setPen(const QString &Farbe="", qreal Breite=-1, int Style=-1);
       //Farbe siehe Hilfe Farben oder Tool: Farbmischung.
       //Breite: in mm, 0 für ein Pixel.
       //Style: 0-6, siehe Tool: Penstyle.
       //Defaults: Letzten Stift verwenden.

  void setPenWidth(qreal Breite, int CapStyle=-1);
       //Breite -1: nur CapStyle setzen.
       //CapStyle: Linenenden Flat=0, Square=16 oder Rounded=32 setzen.

  void setBrush(const QString &Farbe="", int Style=-1, int Alpha=-1);
       //Farbe siehe Hilfe Farben oder Tool: Farbmischung.
       //Style 0-24, siehe Tools: Brushstyle.
       //Transparenz 0-255.
       //Defaults: Letzten Brush verwenden.

  void setGradient(qreal x1, qreal y1, qreal x2, qreal y2, const QString &Rgb1, const QString &Rgb2);
       //Linearen Farbverlauf von Farbe1 in P1(x1,y1) nach Farbe2 in P2(x2,y2) festlegen.
       //Verwendung nur mit setBrush("",15);

  void setGradient(qreal t, const QString &Rgb);
       //0<t<1: Farbe im Zwischenpunkt P1 + t*(P2-P1) festlegen.
       //Verwendung mit setBrush("",15).


  /*! |Drehen|***********************************************************/
       //Winkel in Grad. Uhrzeigersinn ist negativ.

  void turn(qreal w);
       //Richtung der Turtle realtiv ändern.
  void turnTo(qreal w);
       //Richtung absolut einstellen, x-Achse hat 0°.
  void turnTo(qreal x, qreal y);
       //Richtung (lastX(),lastY()) nach P(x,y)
  void turnTo(const QString Name);
       //Richtung (lastX(),lastY()) nach Punkt "Name"

  /*! |Bewegen|**********************************************************/
      //Bewegungen mit aktuellem Pen. Einheit mm.

  void move(qreal mm);
       //Turtle um mm in aktueller Richtung bewegen.
  void moveTo(qreal x, qreal y);
       //Turtle auf Punkt (x,y) bewegen
  void moveTo(const QString Name);
       //Turtle auf Punkt "Name" bewegen

  void goTo(qreal x, qreal y);
       //Turtle ohne Pen auf (x,y)
  void goTo(const QString Name, int Index=0);
       //Turtle ohne Pen auf Punkt "Name"[Index].
       //Index= 0: erster Polygonpunkt, usw.
       //Index=-1: letzter Polygonpunkt.
       //Index=-2: vorletzter Polygonpunkt, usw.

  /*! |Polygon|**********************************************************/
  //Die Turtle besitzt ein Polygon. Das Turtle-Polygon wird
  //mit beginPolygon() und endPolygon(...) erzeugt.

  void beginPolygon();
       //Polygon löschen, Polygonmodus einschalten. Die Endpunkte
       //jeder Turtlebewegung werden ans Polygon angehängt.

  void endPolygon(int FillMode=0);
       //Polygonmodus beenden.
       //FillMode 0: Polygon schliessen, Qt::OddEvenFill.
       //FillMode 1: Polygon schliessen, Qt::WindingFill.
       //FillMode 2: Polygon schliessen, nicht füllen.
       //FillMode 3: Polygon nicht schliessen, nicht füllen.

  void drawPolygon();
        //Polygon auf Turtlepos mit -richtung zeichnen.

  void movePolygon(qreal x, qreal y);
       //Polygonpunkte verschieben.
       //Punkt Polygon[0] auf Punkt (x,y).

  void rectPolygon(qreal dx, qreal dy, int FillMode=0);
       //Rechteckiges Polygon auf Turtlepos mit -richtung.

  void trianglePolygon(qreal c, qreal w_ca, qreal a,  int FillMode=0);
       //Dreieck ABC mit A(lastX(),lastY())
       //c in Richtung lastW();
       //w_ca ist Winklel ca

  void isoPolygon(qreal r, int n=-1, int FillMode=0);
       //Polygon mit n-Ecken, Radius r, Mittelpunkt Last(), Richtung lastW()
       //n=-1: Kreisnäherung, n=-2 höhere Auflösung für Kreis

  void splinePolygon(qreal dt=0.1);
       //Polygon durch ein Spline-Polygon ersetzen.

  void cPolygon();
       //Polygon durch C-Polygon ersetzen.
       //Polygon am Endpunkt spiegeln

  void clrPolygon();
       //Polygon löschen und Polygonmodus beenden

  qreal distanceToPolygon(qreal maxl);
       // Entfernung l der Turtle vom Polygon.
       // Von lastX(),lastY() in Richtung lastW()
       // Rückgabe: 0<l<=maxl in mm
       // -1 kein Schnittpunkt


  /*! |Text|*************************************************************/
  void text(const QString &s);
       //Text an Turtlepos in -richtung anzeigen.
  void text(qreal x, qreal y, const QString &s);
       //Text an P(x,y) in -richtung anzeigen.
  void text(qreal x, qreal y, qreal x1, qreal y1, const QString &s);
       //Text in Rechteck P(x,y) und Q(x1,y1) schreiben.

  void setFont(const QString &Font, qreal Size, bool Fix=true, int Weight=50);
       //Font: "Times","Courrier","Arial" usw.
       //Size: in mm
       //Fix: Proportional
       //Weight: 0-99, Normal=50, Bold=75

  void setWeight(int Weight);
       //Weight: 0-99, Normal=50, Bold=75

  /*! |Rechtecke|********************************************************/
  void rahmen();
       //Blattrahmen neu zeichnen.
  void rectangle(qreal dx, qreal dy);
       //Rechteck (dx,dy) in Turtlerichtung zeichnen.


  /*! |Punkt|************************************************************/
  void drawPoint(qreal x, qreal y);
       //Punktmarke zu Punkt(x,y) zeichnen.


  /*! |Bilder|***********************************************************/

  qreal bild(const QString &DateiName, qreal w_mm=-1, qreal h_mm=-1);
        //Bild an Turtleposition in -richtung zeichnen.
        //Breite w_mm, Hoehe h_mm. Ist ein Wert -1: proportional.
        //Rueckgabe: 0 oder die fehlende Breite oder Höhe

  /*! |Abfragen|*********************************************************/
      //Die aktuellen Einstellungen der Turtle abfragen.
  qreal  lastX();
         //Letzte x-Koordinate
  qreal  lastY();
         //Letzte y-Koordinate

  qreal  lastW();
         //Letzte Richtung in °

  qreal  getW();
         //Richtung des Vektors (0,0)->(lastX(),lastY())
  double getL();
         //Betrag des Vektors (0,0)->(lastX(), lastY())

  void   msgTurtle();
         //Einstellungen der Turtle in der MsgBox ausgeben

  qreal  P0x();
         //Plotrechteck: x,links,oben
  qreal  P0y();
         //Plotrechteck: y,links,oben
  qreal  P1x();
         //Plotrechteck: x,rechts,unten
  qreal  P1y();
         //Plotrechteck: y,rechts,unten

  /*! End Javascript **************************************************/

signals:
   void DrawEnde(QString Titel);

 public:
  void  rect(QRectF r);             // Rechteck in Position mit Richtung zeichnen

  QPointF last()  {return Last;}    // letzte Position der Turtle
 // qreal   x();                    // letzte x-Koordinate der Turtle
 // qreal   y();                    // letzte y-Koordinate der Turtle

  qreal   FontSize_mm();          // FontSize in mm



  void drawPolygon(const QPolygonF &Poly);
        // Polygon im PolyFillMode zeichnen
        // PolyFillMode 0: Qt::OddEvenFill
        // PolyFillMode 1: Qt::WindingFill
        // sonst            : nicht füllen

   void drawPolygon(qreal dx, qreal dy, qreal grad, const QPolygonF &aPoly);
        // 1. Polygon um Punkt (0,0) drehen, Drehwinkel grad
        // 2. Polygon um (dx,dy) verschieben
        // 3. Polygon im PolyFillMode zeichnen

  QPolygonF &Polygon() {return Poly;}

  void  printPDF(QPrinter *Printer);  // PDF-Ausgabe

  bool  begin(); // Einstellungen setzen: Widget oder PDF
                 // Papiergroesse, Rand, Koordinatensystem,
                 // Stift, Pinsel, Klipping, letzter Punkt(0,0), Richtung 0 Grad;
                 // Painter starten: Painter.begin(...)

  QTransform  const &getWinMatrix() const; // Abbildungsmatrix fuer das aktuelle Fenster

  tSetupTurtle  TurtleSetup; // Welt P0/P1, Format, Stift, Hintergrund
  QPainter      Painter;     // aktueller Painter

  tTurtleStatus TurtleStatus(bool savePoly=false, bool stopRec=false, bool saveTurtle=true);
                // Turtlestatus sichern
                // savePoly: Polygon sichern
                // stopRec: Recording unterbrechen
                // saveTurtle: Postion, Stift, Pinsel und Polygonmode sichern

   void         setTurtleStatus(const tTurtleStatus &Status);
                // Turtlestatus setzen
                // Polgon und Recording beachten

  friend class tDrawBefehl;

 protected:

  QPen         Stift;        // aktueller Stift
  QBrush       Pinsel;       // aktueller Pinsel
  QLinearGradient LGradient; // Gradient
  QPointF      Last;         // letzte Position der Turtle
  qreal        Richtung;     // aktuelle Richtung der Turtle im Bogenmass

  void setPen(QPen Pen);
  void setBrush(QBrush Brush);

  bool         PaintAll;   // true : bei paintEvent() die ganze Grafik neu zeichnen
                           // false: bei paintEvent() nur Rahmen und Hinweis zeichen
  bool         AutoPaint;  // automatisches Neuzeichnen
  bool         DiaStart;   //

  void         init();          // Anfangswerte der Turtle setzen

  QPolygonF    Poly;            // Turtle-Polygon
  bool         PolyAdd;         // Last() ans Polygon anhängen
  tFillMode    PolyFillMode;    // Polygon Füllmodus

                                // Splinepolygon aus Polygon berechnen
  void         spline(QPolygonF &qPoly,       // Quelle
                      QPolygonF &SplinePoly,  // Ergebnis
                      int        FillMode,    // offen oder geschlossen
                      qreal      dt=0.1 );    // Zwischenpunkte


                             // Datenrecorder
  QString     *getFnktText;  // !=0: Befehlsliste RecDrawName,
  QString      RecDrawName;  //      Turtle aufzeichnen.
                             // ==0: keine Aufzeichnung,
                             //      RecDrawName isEmpty.

  int          ToolRand;     // Rand für Buttons
  QRectF       FangRect;     // Fangrechteck

  QTransform const &setWinMatrix(); // Abbildungsmatrix zum Fenster berechnen

  tRndFarbe      Random;

  void paintEvent(QPaintEvent *event); // ruft Grafik->draw()
          // der aktuellen Grafik mit Default-Stift und -Pinsel auf.
          // Ablauf in draw():
          // 1. Papierformat, Stift, Pinsel usw. waehlen, Grafikberechnung vorbereiten.
          // 2. mit begin() werden die Einstellungen uebernommen und der Painter gestartet.
          // 3. Grafik mit Turtle- oder Painterbefehlen zeichnen.

  void mousePressEvent(QMouseEvent *); // bei AutoPaint=true: Grafik neu zeichen
           /*bool event(QEvent *e);*/  // Debug: zum Azeigen von Events



private:

  bool           beginUseSetup;  // Style in begin(): true=Turtle, false=Setup

  Qt::PenStyle   LastStiftStyle;   // letzter Style fuer Stift
  Qt::BrushStyle LastPinselStyle; // letzter Style fuer Pinsel

  QTransform     WinMatrix;      // Abbildungsmatrix fuer das aktuelle Fenster

  void setPDFMatrix(QPrinter *Printer);  // Abbildungsmatrix fuer PDF berechnen
  void setPDFPage(QPrinter *Printer);    // PDF-Page setzen
  QTransform     PDFMatrix;              // Abbildungsmatrix fuer PDF
  QPrinter      *PDFPrinter;             // PDF-Ausgabedevice oder 0 fuer Widget

  tGrafik       *Grafik;       // aktuelle Grafik mit Zeichenfunktion draw()

  qreal          FontScale;    // Scale fuer Fonts

  qreal getScale() {return Painter.transform().m11();} // aktueller Scale aus der Abbildungsmatrix


  friend class tTurtleStatus;
};


// tTurtleStatus >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//
// Zustand der Turtle
//
class tTurtleStatus
{
public:
  //tTurtleStatus(){}
  bool               saveTurtle;      // Postion, Stift, Pinsel und Polygonmode sichern
  QPointF            Last;            // letzter Position der Turtle
  qreal              Richtung;        // aktuelle Richtung der Turtle
  QPen               Stift;           // aktueller Stift
  Qt::PenStyle       LastStiftStyle;  // letzter Style fuer Stift
  QBrush             Pinsel;          // aktueller Pinsel
  Qt::BrushStyle     LastPinselStyle; // letzter Style fuer Pinsel

  bool               savePoly;        // true Polygon speichern
  QPolygonF          Poly;            // Polygon
  bool               PolyAdd;         // Last() ans Polygon anhängen
  tTurtle::tFillMode PolyFillMode;    // Polygon Füllmodus

  QString            RecDrawName;     // !="" Name der Aufzeichnung
};


#endif // TURTLE_H
