#include "drawbefehle.h"
#include "utils/msg.h"
#include "_turtle.h"
#include "_grafik.h"


extern tMsgBox   *MsgBox;

// ----------------------------------------------------------------------------
tDrawBefehl::tDrawBefehl():
    bezeichner("0"),
    typ(BNil),
    FillMode(tTurtle::OddEvenFill),
    SubBefehl(0)
{
}


// ----------------------------------------------------------------------------
void tDrawBefehl::clear()
{
   FnktText.clear();
   Polygon.clear();
   if (SubBefehl) {delete SubBefehl; SubBefehl=0;} // rekursiv löschen
}


// ----------------------------------------------------------------------------
bool tDrawBefehl::isValid(const QString &Bez) const
{
  if (Bez.isEmpty()) return false;

  if (Bez.at(0).isDigit()) return false;

  for (int i=0; i<Bez.length(); i++)
      if ((!Bez.at(i).isLetterOrNumber()) &&
           (Bez.at(i)!='_')) return false;

  return true;
}

// ----------------------------------------------------------------------------
void tDrawBefehl::setBezeichner(const QString &Bez)
{
  if (isNil())   // noch kein Bezeichner
  {
    if (!isValid(Bez))
    { MsgBox->addLn("Err, Bezeichner \""+Bez+"\" ungültig",tMsgBox::err);
      return;
    }
    bezeichner=Bez;
    typ=BLeer;
  }
}

// ----------------------------------------------------------------------------
tDrawBefehl::~tDrawBefehl()
{
 //qDebug()<<"~tDrawBefehl()"<<Bezeichner;

  if (SubBefehl) {delete SubBefehl; SubBefehl=0;}
}


// ----------------------------------------------------------------------------
QString *tDrawBefehl::setFnktText(const QString &Text)
{ if (isNil()) return 0;

  typ=BFunction;
  clear();

  FnktText.append(Text);
  return &FnktText;
}



// ----------------------------------------------------------------------------
void tDrawBefehl::saveStyle(QPen &aPen, QBrush &aBrush, int aFillMode)
{ Pen     =aPen;
  Brush   =aBrush;
  FillMode= tTurtle::tFillMode(aFillMode);
}



// ----------------------------------------------------------------------------
void tDrawBefehl::saveStyle(tTurtle *Turtle)
{ Pen     =Turtle->Stift;
  Brush   =Turtle->Pinsel;
  FillMode=Turtle->PolyFillMode;
}


// ----------------------------------------------------------------------------
void tDrawBefehl::setPoly(tTurtle *Turtle, const QPolygonF *KlippPoly)
{ if (isNil()) return;

  typ=BPolygon;
  clear();

  if (Turtle->Polygon().size()<=0) Polygon.clear();
  else if (KlippPoly)
            Polygon=KlippPoly->intersected(Turtle->Polygon());
       else Polygon=Turtle->Polygon();
}


// ----------------------------------------------------------------------------
void tDrawBefehl::setPoly(const QPolygonF &aPoly)
{ if (isNil()) return;

  typ=BPolygon;
  clear();
  Polygon=aPoly;
}



// ----------------------------------------------------------------------------
void tDrawBefehl::insertPoly(tTurtle *Turtle)
{ if (!isPolygon()) return;

  QPolygonF &Quelle=Turtle->Poly;

  if ((Quelle.size()==0)||(Polygon.size()==0)) return;

  QPointF A=Polygon.first(), B=Polygon.last();
  QMatrix m;
  setInsertMatrix(m, Quelle.first(),Quelle.last(), A,B);

  Polygon.clear();
  Polygon=m.map(Quelle);
  Polygon.first()=A;
  Polygon.last()=B;

}



// ----------------------------------------------------------------------------
void tDrawBefehl::restoreStyle(tGrafik *Turtle, bool UseStyle) const
{ if (UseStyle)
  { Turtle->setPen(Pen);
    Turtle->setBrush(Brush);
  }
  Turtle->PolyFillMode=FillMode;
}



// ----------------------------------------------------------------------------
tDrawBefehl &tDrawBefehl::newSubBefehl()
{
  if (typ!=BPolygon) return *this;

  tDrawBefehl *p=this;
  int i=1;

  while (p->SubBefehl!=0)  {
    p=p->SubBefehl;
    i++;
  }


  p->SubBefehl= new tDrawBefehl();
  p->SubBefehl->setBezeichner("Poly"+Int(i,0));
  p->SubBefehl->typ=BPolygon;
  return *p->SubBefehl;
}


// ----------------------------------------------------------------------------
void tDrawBefehl::setLast(tTurtle *Turtle)
{ if (isNil()) return;

  typ=BPunkt;
  clear();

  Polygon.append(Turtle->Last);    // Schiebvektor
  saveStyle(Turtle);
  if (Pen.style()==Qt::NoPen) Pen.setStyle(Qt::SolidLine);
}


// ----------------------------------------------------------------------------
QPointF tDrawBefehl::getFirstP()const
{
    QPointF P;
    if (Polygon.size()>0) P=Polygon.first();
    return P;
}


// ----------------------------------------------------------------------------
QString tDrawBefehl::InfoSubBefehl( bool detail) const
{
  QString s;

  if (SubBefehl)
   { s =", "+SubBefehl->Bezeichner();
     if (detail)  s+=":"+ Int(Polygon.size(),0) + "P";
     s+=SubBefehl->InfoSubBefehl(detail);
   }
   return s;
}


// ----------------------------------------------------------------------------
QString tDrawBefehl::Info() const
{  QString s;

   switch (typ)
   { case BNil:
       s="Neu";
       break;

     case BFunction:
       s=FnktText+"}";
       break;

     case BPunkt:
       s="\""+bezeichner+"\": ";
       if (Polygon.size()>0)
        { s+="Punkt (%1,%2);\n";
          s=s.arg( Polygon.at(0).x() ,0,'g',8)
             .arg( Polygon.at(0).y()	,0,'g',8);
        }
       else s+="Punkt 0";
       break;

     case BPolygon:
       s="\""+bezeichner+"\": Polygon: FillMode="+
           Int(FillMode,0)+", "+
           Int(Polygon.size(),0)+"P"
         + InfoSubBefehl(true);
       break;

     case BLeer:
     default:  s=s="\""+bezeichner+"\": Leer";
   }

   return s;
}

// ----------------------------------------------------------------------------
void tDrawBefehl::draw(tGrafik *Turtle, bool UseStyle) const
{ const tDrawBefehl *p=this;

  switch (typ)
  { case BFunction:
      Turtle->callBefehl(FnktText+"}", bezeichner );
      return;

    case BPunkt:
      if (p->Polygon.size()>0)
      { QPointF L=Turtle->Last;
        QPointF P= L+ Turtle->drawMatrix.map(p->Polygon.first());
        p->restoreStyle(Turtle,UseStyle);
        Turtle->drawPoint( P.x(), P.y());
      }
      return;

    case BPolygon:
      do
      { if (p->Polygon.size()>0)
         { // Turtle setzen und  Polygon Zeichnen
           QMatrix &matrix=Turtle->drawMatrix;
           p->restoreStyle(Turtle,UseStyle);
           if (matrix.isIdentity())
             Turtle->drawPolygon( Turtle->lastX(), Turtle->lastY(),
                                  Turtle->lastW(),
                                  p->Polygon);
           else
           {
             Turtle->drawPolygon( Turtle->lastX(), Turtle->lastY(),
                                  Turtle->lastW(),
                                  matrix.map( p->Polygon) );
           }
         }
      p=p->SubBefehl;
      } while (p!=0);
      return;

    case BLeer:
    default: return;
  }
}


// ----------------------------------------------------------------------------
void tDrawBefehl::drawName(tGrafik *Turtle, qreal dx, qreal dy)
{
  if (Polygon.isEmpty()) return;

  QPointF L=Turtle->Last;
  QPointF P= L+ Turtle->drawMatrix.map(Polygon.first());
          Turtle->text(P.x()+dx, P.y()+dy, bezeichner);
          Turtle->Last=L;
}



// ----------------------------------------------------------------------------
void tDrawBefehl::draw(tGrafik *Turtle, QScriptValue &morphX, QScriptValue &morphY, bool UseStyle) const
{
  const tDrawBefehl *p=this;

  switch (typ)
  {
    case BFunction:
      Turtle->callBefehl(FnktText+"}", bezeichner );
      return;

    case BPunkt:
      if (p->Polygon.size()>0)
      { QPointF L=Turtle->Last;
        QPointF P= L+ Turtle->drawMatrix.map(p->Polygon.first());
        p->restoreStyle(Turtle,UseStyle);
        Turtle->drawPoint( P.x(), P.y());
      }
      return;

   case BPolygon:
     do
     { if (p->Polygon.size()>0)
         { // Turtle setzen und  Polygon Zeichnen
           QMatrix &matrix=Turtle->drawMatrix;
           p->restoreStyle(Turtle,UseStyle);
           QPolygonF poly;

           if (matrix.isIdentity()) poly=p->Polygon;
           else poly=matrix.map( p->Polygon);

           QScriptValueList args; args<<0<<0;
           // morphen
           QMatrix morph;
           //morph.translate(Turtle->Last.x(),Turtle->Last.y()).rotate(Turtle->lastW());
           morph.rotate(Turtle->lastW()).translate(Turtle->Last.x(),Turtle->Last.y());

           for(int i=0; i<poly.size(); i++)
           { QPointF P(morph.map(poly.at(i)));
             //morph.map(poly.at(i));

             args[0]=P.x(); args[1]=P.y();
             P.setX(morphX.call(QScriptValue(),args).toNumber());
             P.setY(morphY.call(QScriptValue(),args).toNumber());
             // poly.replace(i,P-Turtle->Last);
             poly.replace(i,P);
           }

           Turtle->drawPolygon( 0,0,//Turtle->lastX(), Turtle->lastY(),
                                0,//Turtle->lastW(),
                                poly);
         }
       p=p->SubBefehl;
     } while (p!=0);

    case BLeer:
    default: return;
  }
}


// ----------------------------------------------------------------------------
void tDrawBefehl::shift(tGrafik *Turtle, qreal t) const
{
  if (typ!=BPunkt) return;
  QMatrix m;
  QPointF P= t * Polygon.at(0);
  P= Turtle->Last+ m.rotate(Turtle->lastW()).map(P);

  Turtle->moveTo(P.x(), P.y());
}

// ----------------------------------------------------------------------------
QPointF tDrawBefehl::getP0(int Index) const
{ int size=Polygon.size();
  if (size>0)
  { if (Index<0) Index=size+Index;
    if (Index<0) Index=0; if (Index>=size) Index=size-1;
    return Polygon.at(Index);
  }
  return QPointF();
}



// ----------------------------------------------------------------------------
void tDrawBefehl::refine(qreal maxL)
{
  if (typ!=BPolygon) return;

  tDrawBefehl *p=this;  if (maxL<epsilon) maxL=epsilon;
  do
  { QPointF P0,P1,P01;
    QPolygonF &Poly=p->Polygon;
    QPolygonF neuPoly;
    if (Poly.size()>1)
     { // Polygon verfeinern
       P0=Poly.at(0); neuPoly.append(P0);

       int i=1;
       while (i<Poly.size())
       {
         P1=Poly.at(i);
         P01=P1-P0; qreal l= sqrt(P01.x()*P01.x()+P01.y()*P01.y());

         if (l>maxL)               // Punkteabstand zu gross
         { int n=ceil(l/maxL);     // Anzahl der Teile
           P01 /=qreal(n);         // Einheitsvektor/n
           for(int k=1; k<n; k++)
           { neuPoly.append( P0+ qreal(k)*P01 );
           }
         }
         neuPoly.append(P1);
         P0=P1;
         i++;
       }
       Poly=neuPoly;
       neuPoly.clear();
     }
    p=p->SubBefehl;
  } while (p!=0);


}
// Ende tDrawBefehl <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<



// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// tBefehleLst >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

tBefehleLst::tBefehleLst()
{}

tBefehleLst::~tBefehleLst()
{}



// ----------------------------------------------------------------------------
tDrawBefehl &tBefehleLst::defBefehl(const QString &Bezeichner)
{
  tDrawBefehl bef;
  bef.setBezeichner(Bezeichner);      // Versuch, default "0"
  Db.insert(bef.Bezeichner(),bef);    // item "Bezeichner" oder "0"

  tDrawBefehl &item=Db[bef.Bezeichner()]; // item suchen

  item.clear();                       // Befehl löschen

  return item;                        // "Bezeichner" oder "0"
}


// ----------------------------------------------------------------------------
tDrawBefehl &tBefehleLst::addBefehl(const QString &Bezeichner)
{  
  if (!Db.contains(Bezeichner))
  { MsgBox->addLn("Err: \""+Bezeichner+"\") nicht gefunden!", tMsgBox::err);
    return defBefehl("0"); // BNil
  }

  tDrawBefehl &item=Db[Bezeichner];

  if (item.isPolygon())
    return item.newSubBefehl();
  else
  { MsgBox->addLn("Err: \""+Bezeichner+"\" ist kein Polygon!", tMsgBox::err);
    return defBefehl("0");
  }
}


// ----------------------------------------------------------------------------
void tBefehleLst::clrBefehl(const QString &Bezeichner)
{
   QHash<QString, tDrawBefehl>::iterator i=Db.begin();

   while (i !=Db.end())
   { if (i.value().Bezeichner()==Bezeichner) i=Db.erase(i);
     else i++;
   }
}


// ----------------------------------------------------------------------------
tDrawBefehl &tBefehleLst::Befehl(const QString &Bezeichner)
{ return Db[Bezeichner];
}


// ----------------------------------------------------------------------------
QString *tBefehleLst::getFnktText(const QString &Bezeichner)
{ if (Bezeichner.isEmpty()) return 0;
  tDrawBefehl &item=Db[Bezeichner];

  if (item.isFunction())  return item.getFnktText();
  else return 0;

}


// ----------------------------------------------------------------------------
QString tBefehleLst::Info() const
{
  QString s="Eigene Befehle:\n";

  if (Db.size()==0)   return s+="Keine\n";

  QHashIterator<QString, tDrawBefehl> i(Db);

  //if (!i.hasNext()) s+="Keine\n";

  while (i.hasNext())
   { i.next();
     s+="\""+i.value().Bezeichner()+"\": ";

     if (i.value().isNil())
        {s+="nil\n"; continue;}

     if (i.value().isFunction())
        { s+="Funktion\n"; continue; }

     if (i.value().isPunkt())
        { s+="Punkt\n"; continue; }

     if (i.value().isPolygon())
        { s+="Polygon" + i.value().InfoSubBefehl()+"\n"; continue;}

     s+="Leer\n"; continue;

   }
  return s;
}


// ----------------------------------------------------------------------------
QString tBefehleLst::PolygonLst() const
{ QString s;
  if (Db.size()==0)   return s;

  QHashIterator<QString, tDrawBefehl> i(Db);

  while (i.hasNext())
   { i.next();

    if (i.value().isPolygon())
    { if (s.isEmpty()) s=i.value().Bezeichner();
      else s+=SepLst+i.value().Bezeichner();
    }
   }
  return s;
}
