#ifndef DRAWBEFEHL_H
#define DRAWBEFEHL_H

#include "_turtle.h"

class tTurtle;

// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//
// tDrawBefehl beschreibt Grafikelemente.
//
// Ein tDrawBefehl kann mit draw("Bezeichner") gezeichnet werden.
//
// Es gibt drei Typen:
//   Typ Funktion : Funktionsdefinition
//   Typ Punkt    : Punkt
//   Typ Polygon  : Verschachtelte Polygone mit Style informationen
//
class tDrawBefehl
{
public:
   tDrawBefehl();
   ~tDrawBefehl();
   void clear();

   enum BType{ BNil,        // Befehl Not in List
               BLeer,       // Bezeichner ok, undefiniert
               BPolygon,
               BFunction,
               BPunkt};

   void     setBezeichner(const QString &Bez);
            // Befehlsname setzen. Funktioniert nur mit BNil
   bool     isValid(const QString &Bezeichner) const;
            // Bezeichner ist gültig

   bool     isNil() const      { return typ==BNil;}
   bool     isFunction() const { return typ==BFunction;}
   bool     isPolygon() const  { return typ==BPolygon;}
   bool     isPunkt() const    { return typ==BPunkt;}

   QString  Bezeichner() const {return bezeichner;}
   QString  Info() const;

   void     draw(tGrafik *Turtle, bool UseStyle=true) const;   
   void     draw(tGrafik *Turtle, QScriptValue &morphX, QScriptValue &morphY, bool UseStyle=true) const;

   void     drawName(tGrafik *Turtle, qreal dx, qreal dy);
            // Name anzeigen

   void     shift(tGrafik *Turtle, qreal t) const;
   QPointF  getP0(int Index=0) const;

   // Typ BFunction -------------------------------------------------
   QString *setFnktText(const QString &Text);
            // Typ=BFunction setzen, FnktText löschen

   QString *getFnktText() {return &FnktText; }


   // Typ BPolygon --------------------------------------------------
   void     setPoly(tTurtle *Turtle, const QPolygonF *KlippPoly=0);
            // Typ=BPolygon setzen, Turtlepolygon merken
            // KlippPoly!=0: Polygon vorher klippen.

   void     setPoly(const QPolygonF &aPoly);
            // Typ=BPolygon setzen, aPoly merken

   void     insertPoly(tTurtle *Turtle);
            // Turtle-Polygon in Polygon einpassen
            // siehe defDraw( mode=3 )

   void     saveStyle(tTurtle *Turtle);
   void     saveStyle(QPen &aPen, QBrush &Brush, int aFillMode);
            // Turtlestyle und Fillmode merken

   void     restoreStyle(tGrafik *Turtle, bool UseStyle=true) const;
            // UseStyle==true : Turtlestyle und Fillmode setzen
            // UseStyle==false: nur Fillmode setzen

   void     refine(qreal maxL);
            // Polygon verfeinern

   tDrawBefehl &newSubBefehl();
            // für Typ==BPolygon: SubBefehl vom Typ=BPolygon erzeugen

   const QPolygonF   &getPolygon() const  { return Polygon; }
   tTurtle::tFillMode getFillMode() const { return FillMode;}


   QPointF  getFirstP()const ;

   // Typ BPunkt -----------------------------------------------------
   void     setLast(tTurtle *Turtle);
            // Typ=BPunkt setzen, (lastX(), lastY()) speichern

   // ---------------------------------------------------------------
private:
  QString       bezeichner;   // Befehlsbezeichner
  BType         typ;          // Befehlstyp

  // Typ Polygon ----------------------------------------------------
  QPolygonF     Polygon;         // leer oder Polygon
  tTurtle::tFillMode FillMode;   // aktueller Füllmodus
  QPen          Pen;
  QBrush        Brush;

  tDrawBefehl  *SubBefehl;  // 0 oder weiterer Befehl (rekursiv)

  QString       InfoSubBefehl(bool detail=false) const;

  // Typ Funktion ---------------------------------------------------
  QString       FnktText;     // leer oder aufgezeichnete Funktion

  friend class  tBefehleLst;
};
// Ende tDrawBefehl <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<



// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// tBefehleLst
//
// Datenbank für tDrawBefehl Grafikelemente.
//
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
class tBefehleLst
{
public:
  tBefehleLst();
  ~tBefehleLst();

  tDrawBefehl &defBefehl(const QString &Bezeichner);
              // Befehls item "Bezeichner" neu anlegen, alten löschen
              // Rückgabe: item oder item.isNil()==true

  tDrawBefehl &addBefehl(const QString &Bezeichner);
              // SubBefehl an item "Bezeichner" anhängen
              // Rückgabe: item oder item.isNil()==true

  tDrawBefehl &Befehl   (const QString &Bezeichner);
              // Bezeichner suchen

  QString     *getFnktText(const QString &Bezeichner);
              // Recorder-Liste oder 0

  void        clrBefehl(const QString &Bezeichner);
              // Bezeichner löschen
  void        clear() { Db.clear(); }
              // Befehlsliste löschen

  bool        exists(const QString &Bezeichner) {return Db.contains(Bezeichner);}
              // Befehl existiert

  QString     Info() const;
              // Befehle

  QString     PolygonLst() const;

private:
  QHash <QString, tDrawBefehl> Db;
};


#endif // DRAWBEFEHL_H
