#include "galerie.h"
#include "utils/msg.h"
#include "_grafik.h"
#include "_turtle.h"
#include "steuerung.h"
#include "rundgang.h"
#include "grafiken/script_escher.h"
#include "grafiken/script_turtle.h"
#include "grafiken/baum.h"
#include "grafiken/ekurve.h"
#include "grafiken/escher.h"
#include "grafiken/gosper.h"
#include "grafiken/hilbert.h"
#include "grafiken/hilbert1.h"
#include "grafiken/peano.h"
#include "grafiken/purzel.h"
#include "grafiken/script_fxy.h"
#include "grafiken/script_line.h"
#include "grafiken/script_lsystem.h"
#include "grafiken/script_messwerte.h"
#include "grafiken/sierpinski.h"
#include "grafiken/tests.h"
#include "grafiken/toolbrush.h"
#include "grafiken/toollinien.h"
#include "grafiken/toolmischen.h"
#include "grafiken/toolsplines.h"
#include "grafiken/demo.h"

extern tMsgBox   *MsgBox;
extern tSetup    *Setup;


//tGalerie >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//
tGalerie::tGalerie(): QObject(0)
{
  setObjectName("Galerie");

  // C++ Scripvorlagen -----------------
  infos.append(new tScript_Info);
  infos.append(new tScript_f_Info);
  infos.append(new tScriptLSystem_Info);
  infos.append(new tScriptEscher_Info);
  infos.append(new tScript_line_Info);
  infos.append(new tScript_Messwerte_Info);

  // C++ Grafiken ----------------------
  infos.append(new tDemo_Info);
  infos.append(new tBaum_Info);
  infos.append(new tHilbert1_Info);
  infos.append(new tSierpinski_Info);
  infos.append(new tEKurve_Info);
  infos.append(new tGosper_Info);
  infos.append(new tPeano_Info);
  infos.append(new tEscher_Info);
  infos.append(new tPurzel_Info);
  infos.append(new tHilbert_Info);

  // C++ Grafiktools -------------------
  infos.append(new tToolMischen_Info);
  infos.append(new tToolLinien_Info);
  infos.append(new tToolMuster_Info);
  infos.append(new tToolSplines_Info);

  // C++ Testrrafiken ------------------
  infos.append(new tTestGrafik_Info);
  infos.append(new tTestBild_Info);
  infos.append(new tTestRandom_Info);
}

//-----------------------------------------------------------------------------
tGalerie::~tGalerie()
{// qDebug()<<"~tGalerie()";
  for(int i=0; i<infos.size(); i++)  delete(infos.at(i));
}


//-----------------------------------------------------------------------------
void tGalerie::setMainWin(tMainWin *Win)
{ mainWin=Win;
}


//-----------------------------------------------------------------------------
void tGalerie::Dialog(tMainWin *w)
{ tGalerieDialog * d=new tGalerieDialog(w);
  d->exec();
}


//-----------------------------------------------------------------------------
tGrafik_Info *tGalerie::GInfo(int i) const
{ return infos.at( checkI(i) );
}


//-----------------------------------------------------------------------------
tGrafik::GType tGalerie::Type(int i) const
{ if (i==checkI(i)) return infos.at( checkI(i) )->Type();
  else return tGrafik::GNoType;
}


//-----------------------------------------------------------------------------
QString tGalerie::Titel(int i) const
{ if (i==checkI(i)) return infos.at( checkI(i) )->Titel();
  else  return "";
}


//-----------------------------------------------------------------------------
QString tGalerie::Info()
{ return QString(
     "Galerie:\n"
     "Name:"+objectName()+
     "\nGrafikanzahl:"+Int(infos.size())+"\n");

}


// ----------------------------------------------------------------------------
bool tGalerie::isVisible(int i) const
{ return 0!=mainWin->findChild<tGDock *>( Titel(i) );
}


// ----------------------------------------------------------------------------
int tGalerie::size() const
{ return infos.size();
}

// ----------------------------------------------------------------------------
int tGalerie::getI(const QString &Titel) const
{ for(int i=0; i<infos.size(); i++)
  {  if (Titel==infos.at(i)->Titel()) return i;
  }
 return -1;
}

// ----------------------------------------------------------------------------
tGDock *tGalerie::getGDock(const QString &Titel)
{
  tGDock *p=mainWin->findChild<tGDock *>(Titel);
  return p;
}


// ----------------------------------------------------------------------------
tGDock *tGalerie::show(tDock *Dock, QString EndMarke)
{
  MsgBox->addLn("\nScript ausführen:" );

  if (!Dock)
  { MsgBox->addLn("Err: Script fehlt !", tMsgBox::err);
    return 0;
  }
  tGDock *GDock=0; QString Titel;

  Dock->getCursorPos(EndMarke);
  // Cursorposition setzen. Legt das Script im Dock fest

  QScriptEngine *Engine=new QScriptEngine();
  // Script-Engine erzeugen.
  // Die Engine wird von tGrafik übernommen und verwaltet

  Engine->evaluate(Dock->getScriptText(EndMarke));
  // Script einlesen

  if (tGrafik::hasErr(Engine))
  { // Fehlerzeile markieren
    Dock->markScriptLine( Engine->uncaughtExceptionLineNumber());
    goto CancelShow;
  }

  Titel=Engine->evaluate(ScriptVarGrafik).toString();
  // Titel der Grafik aus dem Script ermitteln
  if (tGrafik::hasErr(Engine))
  { MsgBox->addLn("Err: Zeile 'var Grafik=\"Titel\"' fehlt !", tMsgBox::err);
    goto CancelShow;
  }

  // Grafikliste.
  if (Titel.indexOf(SepLst)>=0)
  { // Grafikliste z.B. "xxx:Baum; yyy:Peano;" anzeigen
    // closeAllWithout();
    int sec=Engine->evaluate(ScriptVarSec).toInteger();
    tMainWin::Rundgang.setBilder(Titel,sec);
    goto CancelShow;
  }

  // Einzelne Grafik anzeigen und Scriptengine setzen
  GDock= show(Titel, Engine);
  if (GDock)
  { MsgBox->add(" Ok! \nC++ Grafik \"" + Titel +"\"");
    MsgBox->addLn(Dock->getPfad()+EndMarke);
    return GDock;
  }
  else
  { MsgBox->addLn("Err: Grafik \"" + Titel +"\" nicht in der Galerie!", tMsgBox::err);
    goto CancelShow;
  }

  CancelShow:
  delete Engine;
  tMainWin::Steuerung.ClrDrawEnde();
  return 0;
}



// ----------------------------------------------------------------------------
tGDock *tGalerie::show(int i)
{ return show( Titel(i),0);
}



// ----------------------------------------------------------------------------
tGDock *tGalerie::show(const QString &Titel, QScriptEngine *ScriptEngine)
{
  int i=getI(Titel);   if (i<0) return 0; //Grafik nicht vorhanden

  // Andere GDocks löschen. Ausnahme Tools und i.
  closeAllExceptGCToolsAnd(i);


  // Ist GDock bereits vorhanden?
  tGDock *GDock= mainWin->findChild<tGDock *>( Titel );

  if (GDock==0)
  {  // GDock ist noch nicht vorhanden
     // GDock mit Grafik i neu anlegen
     tGrafik *Grafik=infos.at(i)->New();
     Grafik->setObjectName(Titel);

     if (ScriptEngine) Grafik->setEngine(ScriptEngine);
       // Script aktiv, ScriptEngine übergeben
     else
     { // Kein Script aktiv, C++ Grafik
       if (infos.at(i)->Type()==tGrafik::GCTool)
       { // Tool. Anleitung in MsgBox anzeigen
         MsgBox->addLn(Grafik->Anleitung(), tMsgBox::inf1);
       }
       else
       { // Kein Tool. Also Anleitung im Dock anzeigen
         mainWin->setScriptText(Grafik->Anleitung());
       }
     }

     // Grafikfenster fuer Grafik i neu anlegen
     GDock= new tGDock( Titel, mainWin, Grafik);
     tMainWin::Steuerung.SetDrawEnde( GDock->getGrafik());
  }
  else
  { // GDock updaten
    if (ScriptEngine) GDock->setEngine(ScriptEngine);
    else  GDock->delEngine();

    tMainWin::Steuerung.SetDrawEnde( GDock->getGrafik());
    // FlagSetDrawEnde==true: Slot für DrawEnde setzen

    GDock->update();
  }
  return GDock;
}



// ----------------------------------------------------------------------------
bool tGalerie::close(const QString &Titel) const
{
  tGDock * GDock=mainWin->findChild<tGDock *>( Titel );
   if (GDock!=0)
    { return GDock->close();
    }
   return false;
}


// ----------------------------------------------------------------------------
void tGalerie::closeAllExceptGCToolsAnd(int i)
{
  if (Type(i)==tGrafik::GCTool) return;

  for(int j=0; j<infos.size(); j++)
    { if ((i!=j) && (Type(j)!=tGrafik::GCTool)) close( Titel(j));
    }
}


// ----------------------------------------------------------------------------
void tGalerie::stopSignals(const QString &Titel) const
{
  tGDock * GDock=mainWin->findChild<tGDock *>( Titel );
   if (GDock!=0) GDock->getGrafik()->disconnect();

}



// ----------------------------------------------------------------------------
int tGalerie::checkI(int i) const
{  if (i<0) i=0;
   if ((i<0) || (i>=infos.size())) i=0;
   return i;
}


//End tGalerie <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<




//tGDock >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//
tGDock::tGDock(const QString &title, tMainWin *parent, tGrafik * grafik):
  QDockWidget(title, parent)
{ //Grafik-Dock

  setAllowedAreas(Qt::AllDockWidgetAreas);
  setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
  setFeatures( QDockWidget::DockWidgetMovable|
               QDockWidget::DockWidgetFloatable|
               QDockWidget::DockWidgetClosable);

  setAttribute(Qt::WA_DeleteOnClose);
  setObjectName(title);
  setFloating(true);
  parent->addDockWidget(Qt::TopDockWidgetArea, this);

  QPalette palette;
  QBrush brush(BackcolorGrafik);
  brush.setStyle(Qt::SolidPattern);
  palette.setBrush(QPalette::Window, brush);
  setPalette(palette);
  setAutoFillBackground(true);

  Grafik=grafik;
  setWidget(Grafik);
  QObject::connect(Grafik,        SIGNAL(markScriptLine(int)),
                   parent->Dock, SLOT(markScriptLine(int)));

  Setup->restore(this);
}


tGDock *tGDock::setEngine(QScriptEngine *ScriptEngine)
{ Grafik->setEngine(ScriptEngine);
  return this;
}

void tGDock::delEngine()
{ Grafik->delEngine();
}


void tGDock::printPDF(QString Path)
{  QPrinter *prt= new QPrinter(QPrinter::HighResolution);
   prt->setOutputFileName(Path);
   prt->setOutputFormat(QPrinter::PdfFormat);
   prt->setDocName(objectName());

   Grafik->printPDF(prt);
   //printer.abort();
   MsgBox->addLn("PDF-Datei '"+ Path + "' erzeugt" );

   delete prt;
}

void tGDock::saveGrafik(QSettings *Setting)
{ Grafik->save(Setting);
}

void tGDock::restoreGrafik(QSettings *Setting)
{  Grafik->restore(Setting);
}


tGDock::~tGDock()
{// qDebug()<<"~tGDock()";
  delete Grafik;
}


void tGDock::closeEvent(QCloseEvent *event)
{ event->accept();
  Setup->save(this);
}


//End tGDock <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// tGalerieDialog >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

tGalerieDialog::tGalerieDialog(tMainWin * w):
  QDialog(w),
  ui(new Ui::GalerieDialog)
{ tGalerieDialog::w=w;
  setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
  ui->setupUi(this);


  ui->GrafikTree->setIconSize( QSize(20,20));


  //.................................................................
  // Geschützte Scripte
  QTreeWidgetItem *TopItem = new  QTreeWidgetItem(ui->GrafikTree);
  TopItem->setText(0,"Rundgang, Demos, Kurse, Hilfe");
  TopItem->setText(1,Setup->getDirDemoRO());
  TopItem->setIcon(0, QIcon(":/demo.ico"));
  TopItem->setBackgroundColor(0,ColorDemoLine);

  QString Path=Setup->getDirDemoRO();
  addDir(Path, PathDemo, TopItem, false);


  //.................................................................
  // C++ Grafiken
  QTreeWidgetItem *TopItemC = new  QTreeWidgetItem(ui->GrafikTree);
  TopItemC->setText(0,"Grafiken");
  TopItemC->setIcon(0, QIcon(IconCPlus));
  TopItemC->setBackgroundColor(0,ColorHeadLine );

  QTreeWidgetItem *TopItemD = new  QTreeWidgetItem(TopItemC);
  TopItemD->setText(0,"Test");
  TopItemD->setIcon(0, QIcon(":/ordner.ico" ));
  TopItemD->setBackgroundColor(0,ColorLine);


  QTreeWidgetItem *TopItemT = new  QTreeWidgetItem(ui->GrafikTree);
  TopItemT->setText(0,"Grafiktools");
  TopItemT->setIcon(0, QIcon(IconCPlus));
  TopItemT->setBackgroundColor(0,ColorHeadLine+ColorHeller );

  QTreeWidgetItem *TopItemS = new  QTreeWidgetItem(ui->GrafikTree);
  TopItemS->setText(0,"Grafiken für Scripte");
  TopItemS->setIcon(0, QIcon(IconCPlus));
  TopItemS->setBackgroundColor(0,ColorHeadLine);


  QTreeWidgetItem *item=0;

  for(int i=0; i<tMainWin::Galerie.size(); i++)
  {
    switch (tMainWin::Galerie.GInfo(i)->Type())
    {
      case tGrafik::GCPlus:
        item = new QTreeWidgetItem(TopItemC);
        break;

      case tGrafik::GCTool:
        item = new QTreeWidgetItem(TopItemT);
        break;

      case tGrafik::GScript:
        item = new QTreeWidgetItem(TopItemS);
        break;

      case tGrafik::GCTest:
      default:
        item = new QTreeWidgetItem(TopItemD);
        break;
    }

    item->setText(0,tMainWin::Galerie.GInfo(i)->Titel());
    item->setData(0,Qt::UserRole,i);
    if (i % 2)  item->setBackgroundColor(0,ColorLine);
    item->setText(1,PathCplus);

  }

  //................................................
  // Eigene Scripte
  TopItem = new  QTreeWidgetItem(ui->GrafikTree);
  TopItem->setText(0,"Eigene Scripte");
  TopItem->setIcon(0, QIcon(":/script.ico"));
  TopItem->setBackgroundColor(0,ColorScriptLine);

  Path=Setup->getDirScripte();
  TopItem->setText(1,Path);
  addDir(Path, PathEigene, TopItem, true);

  //................................................
  // PDF
  TopItem = new  QTreeWidgetItem(ui->GrafikTree);
  TopItem->setText(0,"Eigene PDF's");
  TopItem->setIcon(0, QIcon(":/pdf.ico"));
  TopItem->setBackgroundColor(0,ColorScriptLine);

  Path=Setup->getDirPDF();
  TopItem->setText(1,Path);
  addDir(Path, PathPDF, TopItem, true);


 Setup->restore(this);

}



//-----------------------------------------------------------------------------
void tGalerieDialog::addDir(QString Path, const QString &PathBez,
                            QTreeWidgetItem *Item, bool allFiles, int Tiefe)
{
  Tiefe--; if (Tiefe<0) return;

  QDir dir(Path);

  // Ordner listen
  QStringList lst=dir.entryList(QDir::AllDirs | QDir::NoDot | QDir::NoDotDot,
                                QDir::Name | QDir::DirsFirst);

  QTreeWidgetItem * newItem;
  for(int i=0; i<lst.size(); i++)
  {
    newItem= new QTreeWidgetItem(Item);
    newItem->setText(0,lst.at(i));
    newItem->setIcon(0, QIcon(":/ordner.ico" ) );
    if (i % 2) newItem->setBackgroundColor(0,ColorLine);

    addDir(Path+lst.at(i)+"/", PathBez, newItem, Tiefe-1, true);
  }

  // Files listen
  if (!allFiles) { allFiles=true; return;}

  dir.setNameFilters( QStringList()<<"*"+ScriptSuffix<<"*"+PDFSuffix );
  lst=dir.entryList(QDir::NoDot | QDir::NoDotDot | QDir::Files,
                                  QDir::Name | QDir::DirsFirst);

  for(int i=0; i<lst.size(); i++)
  {
    newItem= new QTreeWidgetItem(Item);
    newItem->setText(0,lst.at(i));
    newItem->setText(1,PathBez);
    newItem->setData(0,Qt::UserRole,-1);   // Script
    if (i % 2) newItem->setBackgroundColor(0,ColorLine);
  }
}


//-----------------------------------------------------------------------------
QString tGalerieDialog::makePfad(QTreeWidgetItem *currenItem, bool Datei)
{
  QString s;
  while (currenItem)
  {
    if (currenItem->parent())
     { if (!s.isEmpty()) s="/"+s;  // Ordner
       s=currenItem->text(0)+s;    // Datei
     }
    else
    {  if (Datei) s=currenItem->text(1)+s;   // Dir
       else s=currenItem->text(0)+"/"+s;         //
       return s;
     }
    currenItem=currenItem->parent();
  }
  return s;
}


//-----------------------------------------------------------------------------
void tGalerieDialog::setDialogPath(const QString &DialogPath)
{
  //if (DialogPath.isEmpty()) return;

  QStringList lst= DialogPath.split(SepPfad);

  if (lst.size()<1) return;

  //qDebug()<<lst;

  QTreeWidget     *Tree(ui->GrafikTree);
  QTreeWidgetItem *Item(0);
  int n=0;
  for(int i=0; i<Tree->topLevelItemCount(); i++)
  {
     Item=Tree->topLevelItem(i);
     if (Item->text(0)==lst.at(n))
     { Tree->setCurrentItem(Item);
       Item->setExpanded(true);
       n++;
       break;
     }
  }

  if (n==0) return;
  while (n<lst.size())
  {
    for(int i=0; i<Item->childCount(); i++)
    { if (Item->child(i)->text(0)==lst.at(n))
      { Item=Item->child(i);
        Tree->setCurrentItem(Item);
        Item->setExpanded(true);
        break;
      }
    }
    n++;
  }
}


//-----------------------------------------------------------------------------
void tGalerieDialog::accept()
{

  QTreeWidgetItem &item=*ui->GrafikTree->currentItem();
  QVariant         data( item.data(0,Qt::UserRole));

  Setup->save(this, makePfad(&item, false));

  if (!data.isValid())
   { // Ordner gewählt
     QDialog::reject(); return;
   }

  QDialog::accept();



  int i=data.toInt();
  if (i<0)
  { // Script oder PDF laden
    tMainWin::Steuerung.load(makePfad(&item));
  }
  else
    // C++ Grafik i laden
    tMainWin::Steuerung.load(PathCplus+ tMainWin::Galerie.Titel(i));




}


//-----------------------------------------------------------------------------
void  tGalerieDialog::reject()
{ Setup->save(this,"");
  QDialog::reject();
}


/*
void tGalerieDialog::mouseDoubleClickEvent(QMouseEvent * event)
{ if (Qt::LeftButton==event->button())
  { //event->accept();
    qDebug()<<"<>";
  }
//  event->ignore();
}
*/



