/******************************************************************************
turtle: turlegrafik

Copyright (C) 2011  Günther Schardinger,
                    Forstweg 12,
                    8793-Trofaiach, Austria
                    g.schardinger@aon.at

Dieses Programm ist freie Software. Sie können es unter den Bedingungen der
GNU General Public License, wie von der Free Software Foundation veröffentlicht,
weitergeben und/oder modifizieren, entweder gemäß Version 3 der Lizenz oder
jeder späteren Version.

Die Veröffentlichung dieses Programms erfolgt in der Hoffnung, daß es Ihnen von
Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite
Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK.
Details finden Sie in der GNU General Public License.
GNU General Public License siehe <http://www.gnu.org/licenses/>.
*******************************************************************************/
#ifndef GALERIE_H
#define GALERIE_H
//#include "mainwin.h"
#include "_grafik.h"
#include "ui_galeriedialog.h"


class tGrafik;
class tGrafik_Info;
class tGDock;


// tGalerie >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//
// Galerie verwaltet/startet Grafiken (class tGrafik):
//
//  * Grafiken ueber Auswahldialog in GDocks anzeigen.
//  * Grafiken ueber Scripte in GDocks anzeigen.
//  * Grafiken als PDF-Datei speichern.
//
class tGalerie : public QObject
{
  Q_OBJECT
public:

  tGalerie();
  ~tGalerie();

  void setMainWin(tMainWin *Win); // Haupfenster setzen

  void Dialog(tMainWin *w); // Grafikdialog zur Wahl von Grafiken anzeigen

  // Katalog, Infos zur i-ten Grafik abfragen -----------------------

  tGrafik_Info  *GInfo(int i) const;     // i-tes Grafikinfo
  tGrafik::GType Type(int i) const;      // i-ter Grafiktyp
  QString        Titel(int i) const;     // Name der i-ten Grafik
  bool           isVisible(int i) const; // Ist Grafik i sichtbar?

  int            getI(const QString &Titel) const;
                 // Index i zum Titel oder -1

  tGDock        *getGDock(const QString &Titel);
                 // GDock zum Index oder 0

  QString        Info();

  // ----------------------------------------------------------------
  // C++ Grafiken und Scriptgrafiken in GDock-Fenster anzeigen

  tGDock *show(const QString &Titel, QScriptEngine *ScriptEngine=0);
          // C++ Grafik 'Titel' im GDock anzeigen.
          // ScriptEngine!=0 : Engine als Parent übernehmen.
          // Rückgabe: GDock oder 0.
          //
          // Verhalten:
          // GDock existiert: GDock nur updaten
          //            else: GDock neu anlegen
          // GDock != GCTool: alle anderen GDocks löschen
          // GDock == GCTool: nichts löschen
          // if (Steuerung.FlagSetDrawEnde)
          //            Signal Grafik.DrawEnde(Titel) mit
          //            Slot Steuerung.DrawEnde(Titel) verbinden

  tGDock *show(int i);
          // i-te C++ Grafik im GDock anzeigen.
          // Rückgabe: GDock oder 0.
          // Verwendet show(Titel,0).

  tGDock *show(tDock *Dock, QString EndMarke="");
          // C++ Grafik/en zum Script im Text-Dock im GDock anzeigen.
          // Rückgabe: GDock oder 0.
          //
          // Ablauf:
          // 1. Scriptengine anlegen.
          //    EndMarke==""      : Script an Cursorpositon ausführen
          //    EndMarke=="#marke": Script mit "ScriptEnde#marke" ausführen
          //
          // 2. Scriptzeile 'var Grafik="Name";' auswerten
          //    Name: "Titel": C++ Grafik 'fenster'Titel' laden und mit
          //                   show(Titel, Scriptengine) ausführen.
          //    Name="Grafikname1; Grafikname2; ..": Liste mit Grafiken.
          //                   Grafiken mit Rundgang anzeigen.

  // ----------------------------------------------------------------
  bool    close(const QString &Titel) const;
          // GDock 'Titel' schließen
  void    closeAllExceptGCToolsAnd(int i=-1);
          // alle GDocks ausser GCTools und i schließen

  void    stopSignals(const QString &Titel) const;
          // disconnect Grafik Signals

  void    printPDF(tDock *Dock);
          // aktuelle Grafik als PDF ausgeben

  int     size() const;
          // Anzahl der Grafiken

  tMainWin *MainWin() {return mainWin;}

private:

  QList  <tGrafik_Info* > infos;         // Alle Grafikinfos der Galereie
  tMainWin               *mainWin;       // Hauptfenster

  int checkI(int i) const;           // Grafik-Index i korrigieren, default

  friend class tSetup;
};
// End tGalerie <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<






// tGDock >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//
// Grafik-Dock-Fenster zum Anzeigen von tGrafik-Widgets
//
class tGDock : public QDockWidget  //Grafik-Dock
{ Q_OBJECT

public:
  tGDock(const QString &title, tMainWin *parent, tGrafik * Grafik);
  ~tGDock();

  tGDock *setEngine(QScriptEngine *ScriptEngine=0);
          // Scriptengine der Grafik übergeben

  void    delEngine();
          // Scriptengine aus Grafik löschen

  void printPDF(QString Path);
  // Grafik als PDF-Datei speichern
  // Legt QPrinter-Objekt für PDF an.

  void saveGrafik(QSettings *Setting);     // ruft Grafik->save()
  void restoreGrafik(QSettings *Setting);  // ruft Grafik->restore()
  // Die beiden Funktionen werden von Setup->save(tGDock*) bzw.
  // Setup->restore(tGDock*) gerufen um eventuelle Einstellungen der
  // Grafik unter der Gruppe des GDock' zu speichern

  tGrafik *getGrafik() {return Grafik;}

protected:
  void closeEvent(QCloseEvent *event);     // GDock Setup aufrufen

private:
// tMainWin  *MainWin;  // Hauptfenster
 tGrafik   *Grafik;   // Zeichenobjekt

};
// End tGDock <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<



// tGalerieDialog >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//
// Grafikliste anzeigen und Grafiken auswaehlen
//
class tGalerieDialog : public QDialog
{ Q_OBJECT
  public:
    tGalerieDialog(tMainWin * w);
    ~tGalerieDialog() { delete ui; }

    void setDialogPath(const QString &DialogPath);
    // Item DialogPath aufklappen

    void accept();  // Ok, Script oder Grafik laden
    void reject();  // Dialog mit ESC beendet


protected:
   // void mouseDoubleClickEvent(QMouseEvent *event);

private:
    // Ordner und Dateien rekursiv in QTreeWidgetItem einlesen
    void addDir(QString  Path,          // Startordner
          const QString &PathBez,       // Pfadabkürzuen
        QTreeWidgetItem *Item,          // Topitem
                   bool  allFiles=true, // Files auch im Startordner anzeigen
                    int  Tiefe=15);     // Rekursionstiefe

    QString makePfad(QTreeWidgetItem * currenItem, bool Datei=true);
    // Datei=true:  Dateipfad basteln
    // Datei=false: Dialogpfad zum Item basteln

    tMainWin          *w;
   Ui::GalerieDialog  *ui;             // UI vom Qt-Designer
};
// End tGalerieDialog <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





#endif // GALERIE_H
