#include "_grafik.h"
#include "utils/msg.h"
#include "grafiken/script_fxy.h"

extern tMsgBox *MsgBox;

// tScript_fxy >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//


tScript_fxy::tScript_fxy(QWidget *parent):tGrafik(parent)
{}


QString tScript_fxy::Anleitung()
{ return QString
  (
       "Script Grafik: \""+objectName()+"\"\n\n"
       "Basisgrafik für alle Scripte mit\n"
       "Turtle und Funktionen.\n\n"
       +ScriptHinweis+"\n"+ScriptLine+"\n\n"
       +ScriptBegin+"\n"
       " var Grafik = \""+objectName()+"\";\n\n"

       "var min = -120;\n"
       "var max =  120;\n"
       "var dt  =  0.5;\n\n"

       "function fx(t){ return 0.3*t*Math.cos(t); }\n"
       "function fy(t){ return 0.2*t*Math.sin(t); }\n\n"

       "function init(){ }\n\n"

       "function draw()\n"
       "{ t.drawKoordSystem()\n\n"

       "  t.drawGraf(\"fx\");\n\n"

       "  t.setPen(\"red\");\n"
       "  min= 0; max= 340 ; dt= 0.05;\n"
       "  t.drawGraf(\"fx\",\"fy\");\n"
       "}\n"
       +ScriptEnd+"\n\n");
}


// ----------------------------------------------------------------------------
void tScript_fxy::drawGraf(QString FName1, QString FName2, qreal dx, qreal dy)
{ QScriptValueList args;

  qreal   umin   = getScriptVar("min", 0);
  qreal   umax   = getScriptVar("max",5000.0);
  qreal   dt     = getScriptVar("dt",0.02);

  QScriptValue Xu= Script()->evaluate(FName1);
  if (!Xu.isFunction()) { MsgBox->addLn("Err: "+FName1+"()", tMsgBox::err); return;}

  QScriptValue Yu= Script()->evaluate(FName2);
  if (!Yu.isFunction()) { MsgBox->addLn("Err: "+FName2+"()", tMsgBox::err); return;}

  if ((-epsilon<dt) && (dt<epsilon))
  { dt=epsilon;
    MsgBox->addLn("abs(dt) < "+Real(epsilon)+"! dt="+Real(dt),tMsgBox::err);
  }

  qreal u;  if (dt>0) u=umin; else u=umax;

  args<<u;
  goTo( dx+Xu.call(thisScript, args ).toNumber(),
        dy+Yu.call(thisScript, args ).toNumber());
  u=u+dt;
  while ((umin<=u) && (u<=umax))
  { args[0]=u;
    moveTo( dx+Xu.call(thisScript, args ).toNumber(),
            dy+Yu.call(thisScript, args ).toNumber());
    u=u+dt;
  }

  if (dt>0)
  {    if (u>umax) {  args[0]=umax;  moveTo( dx+Xu.call(thisScript, args ).toNumber(),
                                             dy+Yu.call(thisScript, args ).toNumber()); }
  }
  else if (u<umin) {  args[0]=umin;  moveTo( dx+Xu.call(thisScript, args ).toNumber(),
                                             dy+Yu.call(thisScript, args ).toNumber());}

}


// ----------------------------------------------------------------------------
void tScript_fxy::drawGraf(QString FName,qreal dx, qreal dy)
{ QScriptValueList args;
  qreal   umin   = getScriptVar("min", TurtleSetup.getP0P1().x()-5);
  qreal   umax   = getScriptVar("max", umin + TurtleSetup.getP0P1().width()+10);
  qreal   dt     = getScriptVar("dt",0.2);

  QScriptValue f= Script()->evaluate(FName);
  if (!f.isFunction()) { MsgBox->addLn("Err: "+FName+"()", tMsgBox::err); return;}

  if ((-epsilon<dt) && (dt<epsilon))
  { dt=epsilon;
    MsgBox->addLn("abs(dt) < "+Real(epsilon)+"! dt="+Real(dt),tMsgBox::err);
  }

  qreal x; if (dt>0) x=umin; else x=umax;

  goTo(dx+x, dy+f.call(thisScript, args<<x ).toNumber());
  x=x+dt;

  while ((umin<=x) && (x<=umax))
  { args[0]=x;
    moveTo(dx+x, dy+f.call(thisScript, args ).toNumber() );
    x=x+dt;
  }

  if (dt>0)
  {    if (x>umax) {  args[0]=umax;    moveTo(dx+umax, dy+f.call(thisScript, args ).toNumber());  }
  }
  else if (x<umin) {  args[0]=umin;    moveTo(dx+umin, dy+f.call(thisScript, args ).toNumber());  }
}


//-----------------------------------------------------------------------------
void tScript_fxy::draw()
{
  initScript(thisScript); // Script-Funktion init()
  //setAutoPaint(false);

  if (!begin()) return;   // Painter.begin()
                          // Transformationsmatrix und Klipping setzen,
                          // Pinsel, Stift, Hintergrund und Rahmen zeichnen

  if (!hasScript()) { text(0,0, ScriptHinweis);  return; }

  drawScrip(thisScript); // Script-Funktion draw()

}
