#include "grafiken/script_lsystem.h"
//#include "_grafik.h"
#include "utils/msg.h"
#include "utils/utils.h"
#include "lsystem.h"

extern tMsgBox *MsgBox;

// tScriptLSystem >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//

//-----------------------------------------------------------------------------
QString tScriptLSystem::Anleitung()
{ return QString
  (
    "Script Grafik: \""+objectName()+"\"\n\n"
    "Basisgrafik für alle Scripte mit\n"
    "Turtle- und Lindenmayerfunktionen.\n\n"
    +ScriptHinweis+"\n"+ScriptLine+"\n\n"
    +ScriptBegin+"\n"
    " var Grafik = \""+objectName()+"\";\n"
    " var Tiefe   = 2;\n"
    " var Alpha   = 90.0;\n"
    " var Schritt = 5.0;\n"
    " var Axiom   = \"F-F-F-F\";\n"
    " var Regeln  = \"F()\";\n\n"

    "function F() { return \"F-F+F+FF-F-F+F\";}\n\n"

    "function init(){ t.makeLSystem(); }\n\n"

    "function draw()\n"
    "{ t.msg(t.showLInfo());\n"
    "  t.goTo(-50,40);\n\n"
    "  t.drawLSystem();\n"
    "}\n"
    +ScriptEnd+"\n\n");
}


//-----------------------------------------------------------------------------
void tScriptLSystem::draw()
{
  initScript(thisScript); // Script-Funktion init()

  if (!begin()) return;   // Painter.begin()
                          // Transformationsmatrix und Klipping setzen,
                          // Pinsel, Stift, Hintergrund und Rahmen zeichnen

  if (!hasScript()) { text(0,0, ScriptHinweis);  return; }

  drawScrip(thisScript);  // Script-Funktion draw()
}


//-----------------------------------------------------------------------------
bool tScriptLSystem::callScriptLRegel(const tModul &Modul, QString &Ergebnis)
{
  if (!hasScript()) { return false; }

  QScriptValue ScriptRegel = Script()->evaluate(Modul.Bezeichner());
  if (!hasErr(Script(),false))
  { if (!ScriptRegel.isFunction())
    {MsgBox->addLn("Err: '"+Modul.Info()+"' ist keine Regelfunktion!", tMsgBox::err); return false;}
    QScriptValueList args;

    for(int i=0; i<Modul.ArgumentAnz(); i++){ args << Modul.Argument(i);}

    Ergebnis=ScriptRegel.call(thisScript, args).toString();  if (hasErr(Script())) return false;
    return true;
  }
  else MsgBox->addLn("Err: Keine Regelfunktion '"+Modul.Info()+"' definiert!", tMsgBox::err);
  return false;
}


//-----------------------------------------------------------------------------
void tScriptLSystem::makeLSystem()
{
  if (!LSystem.readLDefinitionen(this)) return;
  LSystem.makeModuleStr();
}


//-----------------------------------------------------------------------------
void tScriptLSystem::drawLSystem()
{
  if (!LSystem.isValid())
  {
    MsgBox->addLn("Err: LSystem nicht definiert!", tMsgBox::err);
    return;
  }
  const QString &s=LSystem.ModuleStr();

  setPen();
  StiftIsDown=true;
  Stack.clear();

  int pos(0); tModul m;

  while (pos<s.length())
  {
    if (!m.read(s,pos))
    { MsgBox->addLn("Err in drawLSystem(): Fehler bei Lesen eines Moduls!", tMsgBox::err);
      return;
    }
    drawModul(m);
  }
}

//-----------------------------------------------------------------------------
void tScriptLSystem::drawModul(tModul &Modul)
{

  switch (Modul.Bezeichner().toAscii())
  {
  case DRAW:
  case DRAW1:
      if (!StiftIsDown) { setPen(); StiftIsDown=true;}
      if (Modul.ArgumentAnz()>0) { move(LSystem.Schritt()*Modul.Argument(0)); break;}
      move(LSystem.Schritt());
      break;

  case MOVE:
      if (StiftIsDown) {setNoStyle(); StiftIsDown=false;}
      if (Modul.ArgumentAnz()>0) { move(LSystem.Schritt()*Modul.Argument(0)); break;}
      move(LSystem.Schritt());
      break;

  case TURNPLUS:
      if (Modul.ArgumentAnz()>0) { turn(Modul.Argument(0)); break;}
      turn(LSystem.Alpha());
      break;
  case TURNMINUS:
      if (Modul.ArgumentAnz()>0) { turn(-Modul.Argument(0)); break;}
      turn(-LSystem.Alpha());
      break;

  case PUSH:
     Stack.push(TurtleStatus());
     break;
  case POP:
     if (Stack.isEmpty())
       MsgBox->addLn("Err in drawModul(): Stack leer! Überzählige ']' ", tMsgBox::err);
     else
       setTurtleStatus(Stack.pop());
     break;

  case PENWIDTH:
     if (Modul.ArgumentAnz()>0)
     {setPenWidth(Modul.Argument(0));}
     break;
  }
}

//-----------------------------------------------------------------------------
QString tScriptLSystem::showLModule()
{ return LSystem.ModuleStr();
}

//-----------------------------------------------------------------------------
QString tScriptLSystem::showLInfo()
{ return LSystem.Info(this);
}

