#ifndef SCRIPTS_LSYSTEM_H
#define SCRIPTS_LSYSTEM_H
#include "_grafik.h"
#include "lsystem.h"


// tScriptLSystem >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//
// "Script Lindenmayer":
// Scripte zum Erstellen und Zeichnen von Lindenmayer-Grafiken (L-Systeme)
//
//
class tScriptLSystem : public tGrafik
{
  Q_OBJECT
public:
  tScriptLSystem(QWidget *parent=0) :tGrafik(parent) {}
  QString Anleitung();
  void    draw();

  // Passende Script-Funktion für ein L-Modul auswerten
  bool callScriptLRegel(const tModul &Modul, QString &Ergebnis);

public slots:
                            // Folgende Funktionen im Script aufgerufen werden
  void    makeLSystem();    // L-System Definitionen aus dem Script lesen
                            // und Modulstring erzeugen
  QString showLInfo();      // Definitionen des L-Systems
  void    drawLSystem();    // Modulstring zeichnen
                            // PolyMode 0 oder 1, siehe tTurtle
  QString showLModule();    // Modulstring zeigen



private:
  QScriptValue thisScript;  // Zugriff des Scripts auf public slots
                            // Beispiel: t.showLInfo();
  tLSystem     LSystem;     // L-System, Definitionen und Modulstring

  void         drawModul(tModul &Modul);
               // ruft Turtlebefehle für ein L-Module auf
               // wird von drawLSystem() gerufen

  bool  StiftIsDown; // Hilfsvariable für drawModul

  QStack <tTurtleStatus> Stack;
};


class tScriptLSystem_Info : public tGrafik_Info
{
public:
  QString  Titel()                {return "Script Lindenmayer";}
  tGrafik::GType  Type()          {return  tGrafik::GScript;}
  tGrafik *New(QWidget *parent=0) {return new tScriptLSystem(parent);}
};
// Ende tScriptLSystem <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<



#endif // SCRIPTS_LSYSTEM_H
