#include "_grafik.h"
#include "utils/msg.h"
#include "utils/utils.h"
#include "script_messwerte.h"

extern tMsgBox *MsgBox;

// tScript_Messwerte >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//


QString tScript_Messwerte::Anleitung()
{ return QString
   (
      "Script Grafik: \""+objectName()+"\"\n"
      +ScriptHinweis+"\n\n"
      "Analyse vom Messwerten-Paaren aus einer Datei.\n"
      "Dateiformat:\n"
      "  Zwei real Messwerte u und v pro Zeile. Trennzeichen Blank.\n"
      "  Textzeilen werden in Msgbox angezeigt.\n"
      +ScriptLine+"\n\n"

      +ScriptBegin+"\n"
      " var Grafik = \""+objectName()+"\";\n"
      " var Messwerte = \"/home/guenther/c/arduino/sun_9xx/sunkalibF1MitT6.txt\";\n"
      " var PunktSize = 0.08;  // Punktrechteck\n\n"

      " // Messwerte darstellen mit t.drawMesswerte():\n"
      " var k = 0.595; var d = -155.0;\n"
      " function Fx(u,v){ return u; }       // x-Koordinate\n"
      " function Fy(u,v){ return k*v + d; } // y-Koordinate\n\n"

      " // Option: Vergleichsfunktion t.drawGraf():\n"
      " function f(x){ return x; } // Vergleichsfunktion \n"
      " //var min=0; var max=100; var dt=0.2;\n\n"

      "function init()\n"
      "{ t.setPage(-25, 97, 6); // dinA6\n"
      "  t.setFont(\"\", 1.6);    // Höhe der Beschriftung\n"
      "}\n\n"

      "function draw()\n"
      "{ t.drawKoordSystem(true, 10, 0.2); t.drawRaster(10, 0.2);\n"
      "  t.setPen(\"red\");\n"
      "  t.drawMesswerte(Messwerte,\"Fx\",\"Fy\",0,-1);\n\n"

      "  t.setPen(\"green\");\n"
      "  t.drawGraf(\"f\",0,10);\n\n"

      "  t.setPen(\"black\");\n"
      "  t.setFont(\"\",2,true,50);\n"
      "  x=60; y=31;\n"
      "  t.text(x,   y  , t.Zeile1());\n"
      "  t.text(x,   y-5,\"x-Achse: Temperatur in °C\");\n"
      "  t.text(x,   y-9,\"y-Achse: k * Sensor + d\");\n"
      "  t.text(x+16,y-14,\"k = \"+k);\n"
      "  t.text(x+16,y-18,\"d = \"+d);\n"
      "}\n"
      +ScriptEnd+"\n\n");
}


//-----------------------------------------------------------------------------
QString tScript_Messwerte::Zeile1(){
  return zeile1;
}


//-----------------------------------------------------------------------------
void tScript_Messwerte::drawMesswerte(QString Pfad, QString Fx, QString Fy, int VonZeile, int BisZeile)
{
  MsgBox->addLn(QString("Messwerte: %1").arg(Pfad)); // Dateiname anzeigen

  QFile f(Pfad);                                     // Datei öffnen
  if (!f.open(QIODevice::ReadOnly))
  { MsgBox->addLn(QString("Datei '%1' nicht gefunden!").arg(Pfad), tMsgBox::err);
    return;
  }
  QTextStream ds(&f);

  zeile1=ds.readLine(); // Überschrift lesen und merken
  MsgBox->addLn(QString("Überschrift: "+zeile1));

  qreal  PunktSize = getScriptVar("PunktSize", 0.08);
  TurtleSetup.setFangRect(FangRect,-PunktSize);  // <0: Punktrechteck absolut setzen


  QScriptValueList args;
  QScriptValue Xuv= Script()->evaluate(Fx);
  if (!Xuv.isFunction()) { MsgBox->addLn("Err: "+Fx+"()", tMsgBox::err); return;}

  QScriptValue Yuv= Script()->evaluate(Fy);
  if (!Yuv.isFunction()) { MsgBox->addLn("Err: "+Fy+"()", tMsgBox::err); return;}


  qreal u=0, v=0; QString s; qint64 Zeile=0; bool ok;
  args << u << v;
  ds.seek(Zeile);
  MsgBox->addLn( QString("Anzeigen von Zeile %1 bis %2").arg(Zeile).arg(BisZeile));

  while (!ds.atEnd())
  { s=ds.readLine();    
    Zeile++;
    if (Zeile<=VonZeile) continue;
    QString us=stripFirst(s ,' ');
    u=us.toFloat(&ok);
    if (ok)
    { v=s.toFloat();
      args[0]=u; args[1]=s.toFloat();
      drawPoint(Xuv.call(thisScript, args ).toNumber(),
                Yuv.call(thisScript, args ).toNumber());
    }
    else MsgBox->addLn(QString("Textzeile %1: "+us+" "+s).arg(Zeile));

    if ((BisZeile>-1) && (Zeile>BisZeile)) break;
  }

}


// ----------------------------------------------------------------------------
void tScript_Messwerte::drawGraf(QString FName,qreal dx, qreal dy)
{ QScriptValueList args;
  qreal   umin   = getScriptVar("min", TurtleSetup.getP0P1().x()-5);
  qreal   umax   = getScriptVar("max", umin + TurtleSetup.getP0P1().width()+10);
  qreal   dt     = getScriptVar("dt",0.2);

  QScriptValue f= Script()->evaluate(FName);
  if (!f.isFunction()) { MsgBox->addLn("Err: "+FName+"()", tMsgBox::err); return;}

  if ((-epsilon<dt) && (dt<epsilon))
  { dt=epsilon;
    MsgBox->addLn("abs(dt) < "+Real(epsilon)+"! dt="+Real(dt),tMsgBox::err);
  }

  qreal x; if (dt>0) x=umin; else x=umax;

  goTo(dx+x, dy+f.call(thisScript, args<<x ).toNumber());
  x=x+dt;

  while ((umin<=x) && (x<=umax))
  { args[0]=x;
    moveTo(dx+x, dy+f.call(thisScript, args ).toNumber() );
    x=x+dt;
  }

  if (dt>0)
  {    if (x>umax) {  args[0]=umax;    moveTo(dx+umax, dy+f.call(thisScript, args ).toNumber());  }
  }
  else if (x<umin) {  args[0]=umin;    moveTo(dx+umin, dy+f.call(thisScript, args ).toNumber());  }
}


//-----------------------------------------------------------------------------
void tScript_Messwerte::draw()
{
  setAutoPaint(false);
  initScript(thisScript); // Script-Funktion init() aufrufen



  if (!begin()) return;   // Painter.begin()
                          // Transformationsmatrix und Klipping setzen,
                          // Pinsel, Stift, Hintergrund und Rahmen zeichnen

  if (!hasScript()) { text(0,0, ScriptHinweis);  return; } // Script vorhanden?


  drawScrip(thisScript);                       // Script-Funktion draw()

}
