#include "_grafik.h"
#include "utils/msg.h"
#include "utils/utils.h"
#include "grafiken/toolmischen.h"

extern tMsgBox *MsgBox;  // MsgBox


// Farbmischung >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

tToolMischen::tToolMischen(QWidget *parent):tGrafik(parent)
{ cyan=255; magenta=255; yellow=255;
  Rgb2="#000000"; 
}



QString tToolMischen::Anleitung()
{ return ScriptLine+"\n"
         "Subtraktive Farbmischung\n\n"
         "Felder Grundfarben:\n"
         "  * Mischen mit Scrollrad oder Mausklick.\n\n"

         "Feld Subtraktive Mischung:\n"
         "  * Scrollrad ändert die Helligkeit.\n"
         "  * Mausklick links : Farbe in die Zwischenablage\n"
         "  * Mausklick rechts: Komplementärfarbe\n\n"

      "Feld Vergleichsfarbe:\n"
         "  * Mausklick vertauscht die Farben.\n"
         +ScriptLine;
}

void tToolMischen::draw()
{ init();
  setPage(tSetupTurtle::dinA4hoch);  // A4, Hochformat
  setMargin(0,0);                    // kein Rand

  if (!begin()) return;              // Painter.begin(), Setup anwenden


  QTransform Matrix(getWinMatrix()); // für Mauspositionen

  setFont("Courrier", 4, true);

  goTo(0.0,0.0);

  setBrush(subRgb(cyan,0,0));
  QRectF r(-30,0,60,27 );
  r.translate(-64,110);
  rect(r); c_rect=Matrix.mapRect(r).toRect();
  text(-90,115, "Cyan:" +Hex(cyan,2));

  text(-94,140, "Grundfarben:");

   goTo(0.0,0.0);
  setBrush(subRgb(0,magenta,0));
  r.moveTo(-30,0); r.translate(0,110);
  rect(r); m_rect=Matrix.mapRect(r).toRect();
  text(-26,115, "Magenta:" +Hex(magenta,2));

  goTo(0.0,0.0);
  setBrush(subRgb(0,0,yellow));
  r.moveTo(-30,0); r.translate(64,110);
  rect(r); y_rect=Matrix.mapRect(r).toRect();
  text(40,115, "Yellow:" +Hex(yellow,2));

  QString Rgb1=subRgb(cyan,magenta,yellow);
  text(-94,103, "Subtraktive Mischung: "+Rgb1);
  goTo(0.0,0.0);
  setBrush(Rgb1);
  r.setRect(-94, -20 , 188, 120);
  rect(r); rgb1_rect=Matrix.mapRect(r).toRect();

  goTo(0.0,0.0);
  setBrush(Rgb2);
  r.setRect(-94, -140 , 188, 120);
  rect(r); rgb2_rect=Matrix.mapRect(r).toRect();
  text(-94,-146, "Vergleichsfarbe: "+Rgb2);

  rahmen();   // Rahmen neu zeichnen
}// Ende Testbild 1 <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<



void tToolMischen::scrollFarbe(bool &isUpdate, int &Farbe, int delta)
{ delta =delta/120;
  if(abs(delta)>1) Farbe+=11*delta;
  else Farbe+=delta;
  if (Farbe<0) Farbe=0;  if (Farbe>255) Farbe=255; 
  isUpdate=true; 
}

void tToolMischen::scrollHelligkeit(bool &isUpdate, int delta)
{ if((delta)>0)
  { if ((cyan<255) && (magenta<255) && (yellow<255))
      { cyan++; magenta++; yellow++;}
  } else
  { if ((cyan>0) && (magenta>0) && (yellow>0))
      { cyan--; magenta--; yellow--;}
  }
  isUpdate=true;
}


void tToolMischen::save(QSettings *Setting)
{ Setting->setValue("Cyan", cyan);
  Setting->setValue("Magenta", magenta);
  Setting->setValue("Yellow", yellow);
  Setting->setValue("Rgb2", Rgb2);
}

void tToolMischen::restore(QSettings *Setting)
{ cyan  =Setting->value("Cyan").toInt();
  magenta=Setting->value("Magenta").toInt();
  yellow =Setting->value("Yellow").toInt();
  Rgb2   =Setting->value("Rgb2").toString();
}


void tToolMischen::wheelEvent(QWheelEvent *event)
{ 
  bool isUpdate=false;
  if (c_rect.contains(event->pos())) scrollFarbe(isUpdate, cyan, event->delta());
  if (m_rect.contains(event->pos())) scrollFarbe(isUpdate, magenta, event->delta());
  if (y_rect.contains(event->pos())) scrollFarbe(isUpdate, yellow, event->delta());
  if (rgb1_rect.contains(event->pos())) scrollHelligkeit(isUpdate, event->delta());
  if (isUpdate) update();

  event->accept();
}

//-----------------------------------------------------------------------------
void tToolMischen::mousePressEvent(QMouseEvent *event)
{

  if (rgb1_rect.contains(event->pos()))
  {
    //if (event->type()==QEvent::MouseButtonDblClick)
    if (event->button()==Qt::LeftButton)
    { // Farbe Mischung ins Clipboard
      QApplication::clipboard()->setText("\""+subRgb(cyan,magenta,yellow)+"\"");
      event->accept();
      return;
    }
    else
    { // Komplementärfarbe
      subRgb( komplRgb( subRgb(cyan,magenta,yellow)),cyan, magenta, yellow);
    }

  }

    if (rgb2_rect.contains(event->pos()))
    { // Farbwechsel
      QString s=Rgb2;
      Rgb2=subRgb(cyan,magenta,yellow);
      subRgb(s, cyan, magenta, yellow);
    }


    if (c_rect.contains(event->pos()))
    { // Farbe hochzählen
      if (event->button()==Qt::LeftButton)
      {  cyan+=16; if (cyan>255) cyan=0;}
      else
      {  cyan-=16; if (cyan<0) cyan=255;}
    }

    if (m_rect.contains(event->pos()))
    { if (event->button()==Qt::LeftButton)
      { // Farbe hochzählen
        magenta+=16; if (magenta>255) magenta=0;
      }
      else
      { magenta-=16; if (magenta<0) magenta=255;
      }
    }

    if (y_rect.contains(event->pos()))
    { if (event->button()==Qt::LeftButton)
      { // Farbe hochzählen
        yellow+=16; if (yellow>255) yellow=0;
      }
      else
      { yellow-=16; if (yellow<0) yellow=255;
      }
    }
    update();
    event->accept();
}

/*
bool tToolMischen::event(QEvent *ev)
{qDebug()<<"QEvent="<<ev->type();
 return QWidget::event(ev);
}
*/
