#include "toolsplines.h"
#include "utils/msg.h"
#include "utils/utils.h"
#include "mainwin.h"
#include "galerie.h"
#include "drawbefehle.h"

extern tMsgBox *MsgBox;  // MsgBox


// tToolSplines >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
tToolSplines::tToolSplines(QWidget *parent):tGrafik(parent),
  lineW(0.0),
  splineW(0.5),
  P_i(-1)
{
  Dlg=new tToolSplineDlg(this);
  Dlg->setDt(0.1,true);
  Dlg->setDl(1.0,true);
  Dlg->setFillMode(3,true);
  Dlg->newSpline();
  ToolRand=Dlg->width();
  Dlg->show();
}


//-----------------------------------------------------------------------------
void tToolSplines::resizeEvent(QResizeEvent * )
{ Dlg->move(width()-Dlg->width(),0);
}


//-----------------------------------------------------------------------------
void tToolSplines::mousePressEvent(QMouseEvent *event)
{  if (event->type()==QEvent::MouseButtonPress)
   { QPointF P(setWinMatrix().inverted().map(event->posF()));
     Raster(dl,P);
     Fang.moveCenter(P);  P_i=findPoint(Punkte,Fang);

     if (event->button()==Qt::LeftButton)
     { if (P_i<0)
        { // kein Punkt gefunden
          int i=findLine(Punkte,Fang);
          if ((i>=0) && (i<Punkte.count()-1))
            { // Zwischenpunkt einfügen
              Punkte.insert(i+1, P);
            }
      }
      event->accept();
      update();
      return;
     }
     if (event->button()==Qt::RightButton)
     {
       if (Punkte.count()<4) return;
       if (P_i>=0)
        { // Punkt löschen
          if (FillMode==tTurtle::NoCloseNoFill) Punkte.remove(P_i);
          else
          { if (Punkte.count()<5) return;
            if ((P_i==0) || (P_i==Punkte.count()-1))
            { Punkte.remove(0); Punkte.remove(Punkte.count()-1);
              if (Punkte.first()!=Punkte.last()) Punkte.append(Punkte.first());

            }
            else Punkte.remove(P_i);
          }
        }
       event->accept();
       update();
       return;
     }

   }

}


//-----------------------------------------------------------------------------
void tToolSplines::mouseMoveEvent(QMouseEvent *event)
{
  if (P_i>=0)
   {  QPointF P( setWinMatrix().inverted().map(event->posF()));
      Raster(dl,P);
      Punkte[P_i]=P;
      if (FillMode<tTurtle::NoCloseNoFill)
      {
        if (P_i==0) Punkte[Punkte.count()-1]=P;
        else if (P_i==Punkte.count()-1) Punkte[0]=P;
      }
      event->accept();
      update();
   }
}

//-----------------------------------------------------------------------------
int tToolSplines::findPoint(const QPolygonF &aPoly, const QRectF &FangRect)
{
  for(int i=0; i<aPoly.count(); i++)
  { if (FangRect.contains(aPoly.at(i))) return i;
  }
  return -1;
}



//-----------------------------------------------------------------------------
int tToolSplines::findLine(const QPolygonF &aPoly, const QRectF &FangRect)
{
  QLineF a(FangRect.bottomLeft(), FangRect.topRight());
  QLineF b(FangRect.bottomRight(), FangRect.topLeft());

  for(int i=0; i<aPoly.count()+1; i++)
  { QLineF line(aPoly.at(i),aPoly.at(i+1));
    if (QLineF::BoundedIntersection==line.intersect(a,0)) return i;
    if (QLineF::BoundedIntersection==line.intersect(b,0)) return i;
  }

  return -1;
}

//-----------------------------------------------------------------------------
void tToolSplines::makeFunktion()
{ Poly=Punkte;
  PolyFillMode=FillMode;
  msgPolygon(Dlg->ui->ShowSpline->isChecked());
}

//-----------------------------------------------------------------------------
QString tToolSplines::Anleitung()
{ return ScriptLine+"\n"
         "Splines definieren\n"
         "Neue Stützpunkte: Linker Mausklick auf eine Strecke.\n"
         "Stützpunkt bewegen: Mit der linken Maustaste.\n"
         "Stützpunkt löschen: Mit rechtem Mausklick.\n"
      +ScriptLine+"\n";
}



void tToolSplines::draw()
{
  init();
  TurtleSetup.setPage(Dlg->getDinA()); TurtleSetup.setMargin(0,0);
  TurtleSetup.setFangRect(Fang,dl);
  setAntiAliasing();

  if (!begin()) return;       // Painter.begin(), Setup anwenden

  drawKoordSystem();  drawRaster();

  setPen("black",lineW);
  Painter.drawRect(Fang);
  for (int i=0; i<Punkte.count(); i++)
  {  if (P_i==i) setPen("red");
     else setPen("black");
     drawPoint(Punkte.at(i).x(),Punkte.at(i).y());
  }
  if (Dlg->ui->ShowSpline->isChecked())
  { Painter.drawPolyline(Punkte);
    spline(Punkte,Spline,FillMode,dt);
    setPen("red",splineW); PolyFillMode=FillMode;
    setBrush("lightblue",1,125);
    drawPolygon(Spline);
  }
  else
  { if (FillMode<NoCloseNoFill)
    { // Polygon schliessen
      if (Punkte.first()!=Punkte.last()) Punkte.append(Punkte.first());
    }
    setPen("red",splineW); PolyFillMode=FillMode;
    setBrush("lightblue",1,125);
    drawPolygon(Punkte);
  }

  setPen("black",0); setBrush("black",1,50);
  for (int i=0; i<drawNames.count(); i++) tGrafik::draw(drawNames.at(i),false);
}

// Ende tToolSplines <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<




// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//
// Toolbox
//
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
tToolSplineDlg::tToolSplineDlg(tToolSplines *parent):
QDialog(parent,  Qt::SubWindow),
 ui(new Ui::Tools)
{ setAttribute(Qt::WA_DeleteOnClose);
  ui->setupUi(this);
  ui->FillMode->addItem("0 = OddEvenFill"  ,QVariant(tTurtle::OddEvenFill ));
  ui->FillMode->addItem("1 = WindingFill"  ,QVariant(tTurtle::WindingFill ));
  ui->FillMode->addItem("2 = CloseNoFill"  ,QVariant(tTurtle::CloseNoFill ));
  ui->FillMode->addItem("3 = NoCloseNoFill",QVariant(tTurtle::NoCloseNoFill ));
  ui->FillMode->setCurrentIndex(3);


  p=parent;
}


//-----------------------------------------------------------------------------
tToolSplineDlg::~tToolSplineDlg()
{ delete ui;
}


//-----------------------------------------------------------------------------
bool tToolSplineDlg::event(QEvent *e )
{
  if (e->type()==QEvent::WindowActivate) setPolygone();
  return false;
}

//-----------------------------------------------------------------------------
void tToolSplineDlg::setPolygone()
{
  ui->Polygone->clear();
  ui->Polygone->insertItems(0, tMainWin::BefehleLst.PolygonLst().split(SepLst));
  if (!p->Name.isEmpty())  ui->Polygone->setEditText(p->Name);
}


//-----------------------------------------------------------------------------
void tToolSplineDlg::drawPolygon()
{ p->update();
}


//-----------------------------------------------------------------------------
tSetupTurtle::tDin tToolSplineDlg::getDinA()
{
  return (tSetupTurtle::tDin)  ui->DinA->value();
}

//-----------------------------------------------------------------------------
void tToolSplineDlg::setDt(double dt, bool setDlg)
{ if (setDlg) ui->dt->setValue(dt);
  p->dt=dt;
  if (!setDlg) p->update();
}

//-----------------------------------------------------------------------------
void tToolSplineDlg::setDl(double dl, bool setDlg)
{ if (setDlg) ui->Raster->setValue(dl);
  p->dl=dl;
  //p->update();
}

//-----------------------------------------------------------------------------
void tToolSplineDlg::setFillMode(int i, bool setDlg)
{ if (setDlg) ui->FillMode->setCurrentIndex(i);
  p->FillMode=(tTurtle::tFillMode) ui->FillMode->itemData(i).toInt();
  if (!setDlg) p->update();
}


//-----------------------------------------------------------------------------
void tToolSplineDlg::newSpline()
{
  QPointF P(0.0,0.0);
  p->Punkte.clear();
  p->Punkte.append(QPointF(-20.0, 0.0) );
  p->Punkte.append(QPointF(  0.0,30.0) );
  p->Punkte.append(QPointF( 20.0, 0.0) );
  p->Name.clear();
  setPolygone();
  p->drawNames.clear();
  p->update();
}


//-----------------------------------------------------------------------------
void tToolSplineDlg::loadPolgon()
{ QString s=ui->Polygone->currentText();

  if (tMainWin::BefehleLst.exists(s))
  { p->Poly.clear();
    p->setPolygon(s);
    p->Punkte=p->Poly;
    p->FillMode=p->PolyFillMode;
    p->Name=s;
    p->update();
  }
  else
  { newSpline();
    MsgBox->addLn("Err, Polygon nicht gefunden",tMsgBox::err );
  }

}


//-----------------------------------------------------------------------------
void tToolSplineDlg::savePolgon()
{ QString s=ui->Polygone->currentText();
  p->Poly=p->Punkte;
  p->PolyFillMode=p->FillMode;
  p->Name=s;
  p->defDraw(s);
  setPolygone();
}


//-----------------------------------------------------------------------------
void tToolSplineDlg::makeFunktion()
{ p->makeFunktion();
}


//-----------------------------------------------------------------------------
void tToolSplineDlg::addDraw()
{ QString s=ui->Polygone->currentText();
  if (!s.isEmpty())
  { p->drawNames.append(s);
    p->update();
  }
}
