#include "lsystem.h"
#include "utils/msg.h"
#include "grafiken/script_lsystem.h"

extern tMsgBox   *MsgBox;



// ----------------------------------------------------------------------------
tModul::tModul():
  bezeichner(tLSystem::Blank)
{
}

// ----------------------------------------------------------------------------
bool tModul::isValid() const
{return bezeichner!=tLSystem::Blank;

}

// ----------------------------------------------------------------------------
qreal tModul::Argument(int i) const
{
  if ((i<0) || (i>=Argumente.count()))
  { MsgBox->addLn("Err: Argument("+Int(i) +") ungültig! ", tMsgBox::err);
    return 0;
  }
  return Argumente.at(i);
}

// ----------------------------------------------------------------------------
bool tModul::read(const QString &Quelle, int &pos)
{
  clear();  int p0;
  if (pos<0) pos=0;
  if (pos>=Quelle.length())
   { fehler="Ende der Module erreicht!"; goto readErr;}

  bezeichner= Quelle.at(pos);

  if (!tLSystem::isModulBezeichner(bezeichner))
   { fehler="'"+bezeichner+"' ist kein Modulbezeichner!"; goto readErr;}

  pos++; if (pos>=Quelle.length()) return true;

  if (tLSystem::KlammerAuf!=Quelle.at(pos)) return true;

  pos++; p0=pos;

  // Argumente lesen
  while (pos<Quelle.length())
  {

    // Keine geschachtelten Klammern
    if (tLSystem::KlammerAuf==Quelle.at(pos))
      {fehler="'"+tLSystem::KlammerAuf+"' !"; goto readErr; }

    // Klammer ')'
    if (tLSystem::KlammerZu==Quelle.at(pos))
      { if (tLSystem::KlammerAuf==Quelle.at(pos-1)) { pos++; return true; } // alle Argumente gelesen

        if (!appendArgument(Quelle,p0,pos-1)) goto readErr;

        pos++; return true; // alle Argumente gelesen

      }

    if (tLSystem::Trenner==Quelle.at(pos))
      { if (!appendArgument(Quelle,p0,pos-1)) goto readErr;
         p0=pos+1;
      }
    pos++;
  }

  fehler="Klammer ')' fehlt!";

readErr:
  fehler="Err in tModul.read(): " +fehler;
  clear();
  MsgBox->addLn(fehler,tMsgBox::err);
  return false;
}


// ----------------------------------------------------------------------------
bool tModul::isGleich(const tModul &modul)
{ return ((bezeichner==modul.bezeichner) &&
          (ArgumentAnz()==modul.ArgumentAnz()));}

// ----------------------------------------------------------------------------
void tModul::clear()
{ bezeichner=tLSystem::Blank;
  Argumente.clear();
}

// ----------------------------------------------------------------------------
QString tModul::Info() const
{ QString s;

  if (!isValid()) {s='?'; return s;}

  s=bezeichner;
  if (Argumente.count()>0)
  {
    s=s+'(';
    for (int i=0; i<Argumente.count(); i++)
    { if (i>0) s=s+tLSystem::Trenner;
      s=s+QString::number(Argumente.at(i));
    }
    s=s+')';
  }
  return s;
}

// ----------------------------------------------------------------------------
bool tModul::appendArgument(const QString &Quelle, int p0,int p1)
{ if (p0>p1) { fehler="Dezimalzahl fehlt!"; return false;}
  bool ok;
  qreal r=Quelle.mid(p0,p1-p0+1).toFloat(&ok);

  if (!ok) { fehler="Dezimalzahl!"; return false;}

  Argumente.append(r);
  return true;
}
// Ende tModul <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<




// tLSystem >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
tLSystem::tLSystem(): Grafik(0)
{ clear();
}


// ----------------------------------------------------------------------------
bool tLSystem::readLDefinitionen(tScriptLSystem *Grafik)
{ if (!Grafik->hasScript())
  { clear();
    MsgBox->addLn("Err in readLDefinitionen(): kein Script! ", tMsgBox::err);
    return false;
  }
  this->Grafik=Grafik;  // Zugriff auf Script merken

  Tiefe  =Grafik->getScriptVar(tLSystem::BezTiefe,1,0,20);
  alpha  =Grafik->getScriptVar(tLSystem::BezAlpha,90.0);
  schritt=Grafik->getScriptVar(tLSystem::BezSchritt,1.0);
  Axiom  =Grafik->getScriptVar(tLSystem::BezAxiom,"F").remove(tLSystem::Blank);

  QString Regeln=Grafik->getScriptVar(tLSystem::BezRegeln,"").remove(tLSystem::Blank);

  if (!setRegeln(Regeln))
  { MsgBox->addLn("Err: Definitionen des Lindenmayersystems fehlerhaft!", tMsgBox::err);
    return false;
  }
  else return true;
    ;
}


// ----------------------------------------------------------------------------
bool tLSystem::setRegeln(const QString &ReglnStr)
{ Regeln.clear();
  moduleStr.clear();
  IsValid=false;

  int pos=0; tModul m;
  while (pos<ReglnStr.length())
  {
    if (!m.read(ReglnStr,pos))
    { MsgBox->addLn("Err in var '"+tLSystem::BezRegeln+"'!", tMsgBox::err);
      return false;
    }

    if (findRegel(m.Bezeichner())==-1)
     { // m ist neue Regel
       Regeln.append(m);
     }
    else
      { // Regel existiert schon
      MsgBox->addLn("Err: Regel '"+m.Bezeichner()+"' doppelt!", tMsgBox::err);
        return false;
      }
  }

  IsValid=true; return true;
}


// ----------------------------------------------------------------------------
int tLSystem::findRegel(const QChar Bezeichner) const
{
  for(int i=0; i<Regeln.count(); i++)
   { if (Bezeichner==Regeln.at(i).Bezeichner()) return i;
   }
  return -1;
}


// ----------------------------------------------------------------------------
QString tLSystem::RegelFunktionen() const
{ QString s;

  for(int i=0; i<Regeln.count(); i++)
  { if (s.length()>0) s=s+" ";
    s=s+Regeln.at(i).Info();
    if (Regeln.at(i).ArgumentAnz()==0) s=s+"()";
  }
  return s;
}

// ----------------------------------------------------------------------------
bool tLSystem::ersetzeModule(QString &ModulStr)
{
  QString Vorgaenger(ModulStr);

  ModulStr.clear();
  tModul m; int pos=0;

  while (pos<Vorgaenger.length())
  {
    // nächstes Modul suchen
    if (!m.read(Vorgaenger,pos)) {return false;}

    // passende Regel suchen
    int i=findRegel(m.Bezeichner());

    if (i<0) ModulStr.append(m.Info()); // Modul übernehmen
    else
     { if (m.ArgumentAnz()!=Regeln.at(i).ArgumentAnz())
        { // ungültiges Modul
          MsgBox->addLn("Err in ersetzeModule: Keine Regel für Modul '"+m.Info()+"'!", tMsgBox::err);
          return false;
        }

       // Regel auf Modul anwenden
       QString ms;
       if (!Grafik->callScriptLRegel(m,ms)) return false;
       ModulStr.append(ms);
     }
  }
  return true;
}

// ----------------------------------------------------------------------------
bool tLSystem::makeModuleStr()
{
  if (!isValid())
  { MsgBox->addLn("Err in makeModuleStr(): LSystem nicht definiert! ", tMsgBox::err);
    return false;
  }

  moduleStr=Axiom;

  for(int i=0; i<Tiefe ; i++)
  {
    if(!ersetzeModule(moduleStr))
    { MsgBox->addLn("Err: Erstellen des Modulstrings!", tMsgBox::err);
      return false;
    }
  }
  return true;
}


// ----------------------------------------------------------------------------
void tLSystem::clear()
{ IsValid=false;
  Tiefe=0;
  alpha=90;
  schritt=1;
  Axiom="";
  Regeln.clear();
  Grafik=0;
}


// ----------------------------------------------------------------------------
QString tLSystem::Info(tScriptLSystem *Grafik) const
{ QString s;
  s=  "Lindenmayersystem:\n";
  s=s+"var "+ScriptVarGrafik+           " = \""+Grafik->objectName()+"\";\n";
  s=s+"var "+tLSystem::BezTiefe+  " = "+Int(Tiefe)+";\n";
  s=s+"var "+tLSystem::BezAlpha+  " = "+Real(alpha)+";\n";
  s=s+"var "+tLSystem::BezSchritt+" = "+Real(schritt)+";\n";
  s=s+"var "+tLSystem::BezAxiom+  " = \""+Axiom+"\";\n";
  s=s+"var "+tLSystem::BezRegeln+ " = \""+RegelFunktionen()+"\";\n";

  if (Grafik)
  {// Regelfunktionen auslesen
   s=s+"\n";
   for(int i=0; i<Regeln.count(); i++)
    { s=s+Grafik->ScriptSource(Regeln.at(i).Bezeichner())+"\n";
    }
  }
  return s+"\n";
}




// ----------------------------------------------------------------------------
bool  tLSystem::isModulBezeichner(QChar Bezeichner)
{
  QString Gueltig= tLSystem::KlammerAuf+
                   tLSystem::KlammerZu+
                   tLSystem::Trenner+
                   tLSystem::Befehlsende+
                   tLSystem::Blank;

  return Gueltig.indexOf(Bezeichner)<0;
}

// ----------------------------------------------------------------------------
const QString tLSystem::BezTiefe  ="Tiefe";
const QString tLSystem::BezAlpha  ="Alpha";
const QString tLSystem::BezAxiom  ="Axiom";
const QString tLSystem::BezSchritt="Schritt";
const QString tLSystem::BezRegeln ="Regeln";
const QString tLSystem::Pfeil     ="->";

// Module mit Zeichenfunktionen
const QChar   tLSystem::Draw      = DRAW;
const QChar   tLSystem::Draw1     = DRAW1;
const QChar   tLSystem::Move      = MOVE;
const QChar   tLSystem::TurnPlus  = TURNPLUS;
const QChar   tLSystem::TurnMinus = TURNMINUS;
const QChar   tLSystem::Push      = PUSH;
const QChar   tLSystem::Pop       = POP;
const QChar   tLSystem::PenWidth  = PENWIDTH;

const QChar   tLSystem::KlammerAuf='(';
const QChar   tLSystem::KlammerZu =')';
const QChar   tLSystem::Trenner=',';
const QChar   tLSystem::Befehlsende=';';
const QChar   tLSystem::Blank=' ';

