#ifndef LSYSTEM_H
#define LSYSTEM_H
#include "mainwin.h"

class tScriptLSystem;

/*! tModul <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
//
// L-Module sind Symbole für das Lindenmayersystem
// Symbole sind Typ Char
// mit oder ohne Parameter vom Typ qreal
//
// Beispiele: F  F()  F(2.3)  X(4.0 ,5.6)
//            [  +    -(30)
//
*/

class tModul
{
public:
  tModul();
  bool read(const QString &Quelle, int &pos);
  // Das nächste Symbol an pos mit Argumenten lesen

  bool isGleich(const tModul &modul);
  // Bezeichner und Anzahl der Argumente

  int     ArgumentAnz() const {return Argumente.count();}
  QChar   Bezeichner()  const {return bezeichner;}
  qreal   Argument(int i) const;

  bool    isValid()     const;
  void    clear();

  QString Info() const;
  QString Fehler() const {return fehler;}

private:
  QChar bezeichner;
  QList <qreal> Argumente;
  bool appendArgument(const QString &Quelle, int p0,int p1);

  QString fehler;

};


//! tLSystem >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
/*! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

  Objekt erstellt und verwaltet Lindenmayer-Systeme:

  Beispiel: Definition eines L-Systems aus einem Script
*//*
    var Grafik  = "Lindenmayer Scripts";
    var Tiefe   = 4;
    var Alpha   = 90.0;
    var Schritt = 1.6;
    var Axiom   = "-F-F-F-F";
    var Regeln  = "F()";


    function F()
    { return "F+F-F-F+F";
    }
*/


class tLSystem
{
public:
  tLSystem();
  void clear();
  bool isValid() {return IsValid;}

  bool    readLDefinitionen(tScriptLSystem *Grafik);
          // L-System Definitonen aus den Scriptvariablen ermitteln.
          // Bei Erfolg isValid()->true

  bool    makeModuleStr();
          // Modulestring des L-System berechnen

  qreal   Schritt() const {return schritt;}
  qreal   Alpha()   const {return alpha;}
  const QString &ModuleStr() const {return moduleStr;} // Module des L-System
  QString Info(tScriptLSystem *Grafik=0) const;              // Systemdefinitionen

private:
  // LSystem Definitionen -------------------------------------
  int     Tiefe;         // Rekursionstiefe
  qreal   alpha;         // Drehwinkel, default
  qreal   schritt;       // Schrittweite, default
  QString Axiom;         // Anfangsstring
  QList <tModul> Regeln; // Funktionsnamen mit Parameteranzahl
                         // für die Regelfunktionen im Script
  // ----------------------------------------------------------

  bool    setRegeln(const QString &ReglnStr);
          // Regeln festlegen und ReglnStr parsen

  int     findRegel(const QChar Bezeichner) const;
          // Regel-Index zu Bezeichner suchen, -1 nicht gefunden

  QString RegelFunktionen() const;

  bool    ersetzeModule(QString &ModulStr);
          // Alle Module durch Aufruf der Regelfunktionen im Script ersetzen

  QString   moduleStr;
  bool      IsValid;
  tScriptLSystem *Grafik; // Kopie für Scriptzugriffe
public:


  static const QString BezTiefe;
  static const QString BezAlpha;
  static const QString BezSchritt;
  static const QString BezAxiom;
  static const QString BezRegeln;
  static const QString Pfeil;

  // Symbole für Module ---------------------------------------------

  static bool  isModulBezeichner(QChar Bezeichner);

  // ungültigen Modulbezeichner
  static const QChar   KlammerAuf;   // (
  static const QChar   KlammerZu;    // )
  static const QChar   Trenner;      // ,
  static const QChar   Befehlsende;  // ;
  static const QChar   Blank;

  // alle anderen ASCII-Zeichen sind gültigen Modulbezeichner

  // Modulbezeichner mit Bedeutung für das Zeichnen
  #define DRAW  'F'
  static const QChar   Draw;      // Schritt vorwärts mit zeichen
  #define DRAW1 'G'
  static const QChar   Draw1;     // Schritt vorwärts mit zeichen

  #define MOVE 'f'
  static const QChar   Move;      // Schritt vorwärts ohne zeichen

  #define TURNPLUS '+'
  static const QChar   TurnPlus;  // Turtle um Alpha im Uhrzeigersinn drehen
  #define TURNMINUS '-'
  static const QChar   TurnMinus; // Turtle um Alpha gegen Uhrzeigersinn drehen

  #define PUSH '['
  static const QChar   Push;      // Zustand der Turtle auf den Stack

  #define POP  ']'
  static const QChar   Pop;       // Zustand der Turtle vom den Stack holen

  #define PENWIDTH  'W'
  static const QChar   PenWidth;  // Stiftstärke der Turtle ändern
};


#endif // LSYSTEM_H
