#ifndef MAINWIN_H
#define MAINWIN_H
#include <QtGui>
#include <QtScript>


/******************************************************************************
turtle: turlegrafik

Copyright (C) 2011	Günther Schardinger,
                    Forstweg 12,
                    8793-Trofaiach, Austria
                    g.schardinger@aon.at

Dieses Programm ist freie Software. Sie können es unter den Bedingungen der
GNU General Public License, wie von der Free Software Foundation veröffentlicht,
weitergeben und/oder modifizieren, entweder gemäß Version 3 der Lizenz oder
jeder späteren Version.

Die Veröffentlichung dieses Programms erfolgt in der Hoffnung, daß es Ihnen von
Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite
Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK.
Details finden Sie in der GNU General Public License.
GNU General Public License siehe <http://www.gnu.org/licenses/>.
*******************************************************************************/



//! Modul- und Objektuebersicht >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
/*! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

Modul: main.cpp

   * QtMsgHandler: Alle Qt-Messages auf die MsgBox umleiten
   * Name und Version des Programms setzen
   * Resourcen einbinden, deutsche Übersetzung
   * tMainWin anlegen und starten

------------------------------------------------------------------------------
Modul: mainwin.h, mainwin.cpp
   Hauptfenster, Formular: mainwin.ui

   class tMainWin:
     Hauptfenster, Menuebefehle, Textdock und MsgBox.
     Script-Texte ins Textdocks laden und speichern.
     Hilfedateien anzeigen.
     Aufraeumarbeitem beim Beenden.

     Statische Objekte:
        class tGalerie:    siehe Modul galerie.h
        class tSteuerung:  siehe Modul steuerung.h
        class tBefehleLst: siehe Modul drawbefehle.h
        class tRundgang:   siehe Modul rundgang.h

   class tDock:
     Fixe und freie Dock-Fenster fuer Texte mit tEdit.
     Textaenderung setzt Flag isGeandert.
     Verbindet Signal doScript() mit Slot on_action_Script_triggered().

   class tEdit:
     QTextEdit fuer Docks:
     Kontextmenue fuer Dock.
     Doppelkick sendet Signal doScript().


------------------------------------------------------------------------------
Modul: galerie.h, galerie.cpp

   C++ Grafiken verwalten, erzeugen
   Dialog zum Anzeigen  von C++ Grafiken und Scriptgrafiken

   * class tGalerie:
     Die Liste der Grafiken wird im Konstruktor definiert.
     Dialog:      Wahl von Grafiken und Scripten.
     show():      C++ Grafik oder Script-Grafik in GDock anzeigen

   * class tGDock:
     Dock-Fenster fuer die Grafikausgaben des Programms.
     Verwendet Widget tGrafik mit tTurtle zum Zeichnen der Grafiken.

-------------------------------------------------------------------------------
Modul: _turtle.h, _turtle.cpp

   Stellt die Turtle-Befehle für die Grafik.draw() zur Verfügung.
   draw() wird von DGock::paintEvent() oder DGock::printPDF() gerufen.

   * Befehle und Script fuer den Zugriff auf QPainter:

     + Setup fuer Papier, Rand, Koordinatensystem, Tranformationsmatizen.
     + Einheiten in mm und Grad, Darstellung mit qreal.
     + mathematisches Koordinatensystem: x-Achse nach rechts, y-Achse nach oben.
     + automatische Anpassung der Abbildung an Fenster oder PDF-Ausgabe.
     + Zeichen-Stift und Malpinsel Pinsel fuer Turtle und Painter.
     + Turtlebefehle: z.B. move(25.5), moveTo(20.0,-12.8), turn(30), turnTo(56)
     + Turtle zum Zeichnen in Polygonen: move(l, poly), moveTo(x,y, poly),
                                         goTo(x,y), last();
     + Turtle fuer Texte: setFont(...), text(x,y, string).
     + Alle Painterbefehle: Painter.drawPolygon(polygon), Painter.dawEllipse(...), usw.

   Verwendung siehe Beispiele in grafiken/xxx.h
   Standardeinstellungen siehe class tSetupTurtle.

-------------------------------------------------------------------------------
Modul: _grafik.h                Header für Objekt-Grafik
       _grafik.cpp              Implementierung für Objekt-Grafik

       grafiken/NameXYZ.h       Abgeleitete Grafiken
       grafiken/NameXYZ.cpp
       ...

   Alle Grafiken werden von class tGrafik und tGrafik_Info abgeleitet.

   * class tGrafik: public tTurtle, Basisklasse für alle Grafiken
     draw():      (virtual) Hier wird die Grafik mit Turtle- und
                  Painterbefehlen erzeugt. Siehe class tTurtle.
     Anleitung(): (virtual) Beschreibung zur Grafik im Text-Dock ausgeben.
     Details siehe grafik/_grafik.h.

   * class tGrafik_Info, nur in grafik/_grafik.h.
     Zu jeder Grafik tNameXYZ muss ein tNameXYZ_Info definiert werden.
     Das Objekt tNameXYZ_Info wird von der Galerie verwendet.
     Details siehe _grafik.h.

-------------------------------------------------------------------------------
Modul: steuerung.h, steuerung.cpp

       class tSteuerung:

       Grafiken und Scripte laden, ausführen
       Grafiken und Scripte automatisch ausführen
       PDF-Ausgabe der Grafiken

-------------------------------------------------------------------------------
Modul: drawbefehle.h, drawbefehle.cpp

       class tBefehleLst:
       Befehlsliste für eigene Befehle

-------------------------------------------------------------------------------
Modul: rundgang.h, rundgang.cpp

       class tRundgang:
       Rundgangssteuerung (Diashow) für Scripte und Grafiken

-------------------------------------------------------------------------------
Modul: setup.cpp, setup.h

   Verwaltet alle Einstellungen und Konstanten. Komfortklasse fuer QSetup.

   * Verwendet ini-Datei: ...\Anwendungsdaten\AppName\AppVers.ini

   * Alle Konstanten: Papierformate, Standard Hintergrundfarben und Fontsize usw.

   * class tSetup:
     Dialog: Fenster fuer Programm-Einstellungen,
             Formular: einstdialog.ui, class tSetupDialog

     Datei-Pfade: Programmverzeichnis, Hilfe, PDF, usw.
     Einstellungen fuer Objekte save/restore: z.B. tDock
                                              restore(tDock *Dock)
                                              save(tDock *Dock)
   * class tSetupTurtle:
     Papierformate und Standardeinstellungen fuer class tTurtle

-------------------------------------------------------------------------------
Hilfmodule:
   utils/msg.h, utils/msg.cpp ...........................................

   * Strings: Spezielle String-Funktionen.
              String-Funktionen fuer die Ausgabe.

   * class tMsgBox
     Anzeigebereich fuer Status- und Fehler-Meldungen im Hauptfenster.
     Qt-Messages werden in das Fenster umgeleitet.


   utils/utils.h, utils/utils.cpp  ......................................

   * class tRandom:  Verschieden Zufallsfunktionen.
   * Vektorbefehle Betrag, wAplpha, Abbildungsmatrix

-------------------------------------------------------------------------------

Ende: Modul- und Objektuebersicht >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<*/


class tDock;
class tGDock;
class tGalerie;
class tSteuerung;
class tBefehleLst;
class tRundgang;
class tToolScriptDlg;


// Hauptfenster >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//
namespace Ui {class tMainWin; class tEdit;} // namespace fuer UserInterface des Designers

class tMainWin : public QMainWindow
{ Q_OBJECT

public:  explicit tMainWin(QWidget *parent = 0); ~tMainWin();

public:
  static tDock      *getDock()     {return Dock;}

  static tGalerie    Galerie;      // Grafiken verwalten
  static tSteuerung  Steuerung;    // Aufrufsteuerung für Grafiken und Scripts
  static tBefehleLst BefehleLst;   // Liste für eigene Script Befehle
  static tRundgang   Rundgang;     // automatische Grafik und Script Aufrufe

  Ui::tMainWin  *ui;               // Userinterface vom Qt-Designer

public slots:

  //Menuebefehle ----------------------------------------------------

  // Datei ----------------------------------------------------------
  void on_action_GalerieDialog_triggered();   // Grafik aus Galerie wählen
  void on_action_PDF_triggered();             // Grafik als PDF ausgeben
  void on_action_LastGrafik_triggered();      // letzte Grafik anzeigen

  void on_action_ScriptLaden_triggered();     // Script laden
  void on_action_Speichern_triggered();       // Script speichern
  void on_action_SpeichernUnter_triggered();  // Script speichern unter
  void on_action_CallEditor_triggered();      // Externen Editor aufrufen

  void on_action_Close_triggered();           // Progammende

  // Scripte --------------------------------------------------------
  void on_action_Script_triggered();          // Script ausführen
  void on_action_ScriptClear_triggered();     // Script löschen
  void on_action_ScriptVorlage_triggered();   // Scriptvorlage einfügen  

  void on_action_ShowBefehle_triggered();     // Scriptbefehle anzeigen
  void on_action_ClearBefehle_triggered();    // Script löschen


  // Tools ----------------------------------------------------------
  void on_action_Scriptbefehle_triggered();   // Tool Scriptbefehle
  void on_action_ToolMischen_triggered();     // Tool Farbmischen ausführen
  void on_action_ToolSpline_triggered();      // Tool Spline ausführen
  void on_action_ToolPenstyle_triggered();    // Tool Penstyle ausführen
  void on_action_ToolBrushstyle_triggered();  // Tool Brushstyle ausführen


  // Extras ---------------------------------------------------------
  void on_action_msgClrs_triggered();         // Msgbox löschen
  void on_action_setup_triggered();           // Einstellungen
  //   ....................................
  void on_action_test1_triggered();           // Test 1 fuer Tests
  void on_action_test2_triggered();           // Test 2 fuer Tests

  // Hilfe ----------------------------------------------------------
  void on_action_Hilfe_triggered();           // Die ersten Schritte
  //   ....................................
  void on_action_HilfeScripte_triggered();    // Hilfe Scripte
  void on_action_HilfeTurtle_triggered();     // Hilfe Turtlebefehle
  void on_action_HilfeGrafik_triggered();     // Hilfe Grafikbefehle
  void on_action_HilfeFarben_triggered();     // Hilfe Farben
  //   ....................................
  void on_action_HilfeCPlus_triggered();      // Hilfe C++
  void on_action_info_triggered();            // Programminfo anzeigen



public:

  // -------------------------------------------------------------------
  bool loadScript(const QString &Path, int CursorPos=0);
       // Script ins Textdock laden
       // Scriptcursor auf CursorPos setzen.
       // Script nicht Ausführen.

  bool saveScript(const QString &Path="",bool mitDialog=false);
       // Textdock speichern
       // Path=="": Dialog speichern immer anzeigen

  void setScriptText(const QString &Anleitung);
       // Anleitung in Textdock anzeigen

  bool showUrl(const QString &Url);
       // Hilfedateien laden

  void setDockFocus();
       // Dock aktiv setzen


protected:
  void closeEvent(QCloseEvent *event);    // Aufräumarbeitem beim Beenden
  //void showEvent (QShowEvent * event ) ;

private:  //intern --------------------------------------------------

  void loadScriptMitDlg(const QString &Path);

  static tDock   *Dock;    // fixes Dockfenster mit QTextEdit
  friend class tGDock;

  static tToolScriptDlg *ToolScript;
  friend class tToolScriptDlg;

};//End Hauptfenster <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<




// tEdit >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//
// QTextEdit fuer tDock:  Behandelt mouseDoubleClickEvent()
//
class tEdit: public QTextEdit
{ Q_OBJECT

public:
  tEdit(QWidget * parent=0): QTextEdit(parent) {}

signals:
  void doScript();  // Script auswerten

protected:
  void contextMenuEvent(QContextMenuEvent *event);
  void mouseDoubleClickEvent(QMouseEvent * event);
};
// End tEdit <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<




// tDock >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//
// Fenster fuer Textausgaben
//
class tDock : public QDockWidget  //QTextEdit Fenster als Anzeige-Dock
{ Q_OBJECT

public:
  enum tDockArt { ScriptFix   =0x01,   // fixes Dock nicht schliessbar
                  xScript     =0x02};  // freies Dock

  tDock(const QString &title, tMainWin *parent, tDockArt Art=ScriptFix);
  ~tDock();

  void clear();          // clear(Backcolor) mit Standart Backcolor zum DockTyp
  bool checkDockSaved();
      // Dock auf Änderungen prüfen und speichern oder verwerfen

  bool isDockSaved() {return !isGeaendert;}

  void setPfadSpeichernUnter(const QString &Pfad);
       // Speicherpfad für 'Speichern unter'

  bool isHtml(); // Html-Text,  AcceptRichText(true);

  const QString &getPfad() const {return tDock::Pfad;}  // Speicherpfad

  void setTxtMode(bool MixedCase=true); // Anzeigemodus fuer Dock
  void setFontSize();                   // Setup Schriftgroesse setzen


  void setText (const QString &s);
       // Dock löschen und Text einfügen
       // 1. Zeichen '<' in den Html-Modus schalten

  void insertPlainText(const QString &s);
       // Text an der Cursorpos einfügen

  void append  (const QString &s);   // Text anfuegen

  void insert_p(const QString &s);   // Html-Absatz in FontSize einfuegen
  void insert_c(QString &s);         // Html-code in FontSize einfuegen

  void moveCursor(QTextCursor::MoveOperation m,
                  QTextCursor::MoveMode   mode=QTextCursor::MoveAnchor);

  int getCursorPos(const QString Marke="");
  // Marke!="":  Cursorposition zuerst auf ScriptBegin setzen
  // Rückgabe: Cursorposition

  void setCursorPos(int pos);
  // Cursorposition im Script setzen. -1 Scriptende.

  QString getScriptText(const QString &EndMarke="");
  // Script-Text zwischen ScriptBegin und ScriptEnd bestimmen.
  // EndMarke==""      : Die Suche beginnt an der Cursorposition.
  // EndMarke=="#marke": Script mit "ScriptEnd#marke" suchen.
  // Setzt ScriptPosAnfang.


  static QString getDockName(tDockArt Art); // Objectname des Docks

  tEdit     *Edit;      // QTextEdit des Docks
  tDockArt   Art;       // Verwendungszweck des Docks

public slots:
  void setGeaendert(bool Ja=true);  // wird von Edit->document() bedient

  void markScriptLine(int Zeilenummer=1);
  // Im atuellen Script die Zeile mit Zeilenummer markieren.
  // Für die Anzeige von Fehlern im Script.


signals:
  void setSpeichernEnabled(bool on);
  void setSpeichernUnterEnabled(bool on);

protected:
  void closeEvent(QCloseEvent *event);  // save() sicherstellen
  //void mouseDoubleClickEvent (QMouseEvent * event);


private:
  void clear(QRgb Backcolor);    // Anzeigeinhalt loeschen, Text-Modus setzen
  void setBackcolor(QRgb Back);  // Hintergrundfarbe

  bool       isGeaendert;        // Flag: Text wurde geaendert
  QString    Pfad;               // Speicherpfad für Speichern unter                                 //
  int        ScriptPosAnfang;    // Position des aktuellen Script-Texts
  tMainWin  *MainWin;            // Pointer zum Hauptfenster

};//End Dock <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<


#endif // MAINWIN_H
