#include "rundgang.h"
#include "utils/msg.h"
#include "mainwin.h"
#include "setup.h"
#include "galerie.h"
#include "steuerung.h"


extern tMsgBox   *MsgBox;
extern tSetup    *Setup;

// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//
tRundgang::tRundgang(): QObject(),
  index(-1),
  stopped(true)
{}


// ----------------------------------------------------------------------------
void tRundgang::clear()
{ index=-1;
  stopped=true;
  Bilder.clear();
  ms=0; 
}

// ----------------------------------------------------------------------------
bool tRundgang::isActive()
{ return !Bilder.isEmpty();
}


// ----------------------------------------------------------------------------
void tRundgang::setBilder(const QString &Bildernamen, int sec)
{
  if (isActive()) { emit closeDlg(); clear();}

  // Bildernamen durch SepLst getrennt
  Bilder=Bildernamen.split( SepLst, QString::SkipEmptyParts );

  if (Bilder.count()<=0) return;

  if (sec<1) sec=0;  ms=sec*1000; QString Sec=Real(ms/1000)+" sec";

  StartScript=   tMainWin::Steuerung.Grafikname();
  StartCursorPos=tMainWin::Steuerung.CursorPos();

  for(int i=0; i<Bilder.count(); i++)
  { Bilder[i]= tMainWin::Steuerung.makeGrafikname(Bilder.at(i));
  }

  MsgBox->clear();
  MsgBox->addLn("\nRundgang: "+Sec, tMsgBox::inf1);

  for(int i=0; i<Bilder.count(); i++)
  { MsgBox->addLn(Bilder.at(i)+";", tMsgBox::inf1);
  }

  // Dialogsteuerung anlegen
  tRundgangDlg *Dlg=new tRundgangDlg(tMainWin::Galerie.MainWin());
  connect(this, SIGNAL(closeDlg()), Dlg, SLOT(close()));
  connect(this, SIGNAL(setGrafikLabel(QString)), Dlg->ui->Path, SLOT(setText(QString) ));
  connect(this, SIGNAL(setPlayButton(bool)), Dlg->ui->PlayButton, SLOT(setEnabled(bool)));
  Dlg->ui->SecLabel->setText(Sec);
  Dlg->show();
  play();
}


// ----------------------------------------------------------------------------
void tRundgang::pause()
{ emit setPlayButton(true);
  stopped=true;
  QTimer::singleShot(0, this, SLOT(showNextBild()));
  MsgBox->addLn("Rundgang: Pause", tMsgBox::inf1);

}

// ----------------------------------------------------------------------------
void tRundgang::prev()
{ if (!stopped) pause();
  int msLast=ms;

  index -=2;
  if (index<-1) index=Bilder.count()-2;

  ms=0; stopped=false; showNextBild();
  ms=msLast; stopped=true;
}

// ----------------------------------------------------------------------------
void tRundgang::next()
{ if (!stopped) pause();
  int msLast=ms;

  if (index>=(Bilder.count()-1)) index=-1;

  ms=0; stopped=false;  showNextBild();
  ms=msLast; stopped=true;
}

// ----------------------------------------------------------------------------
void tRundgang::play()
{ if (stopped)
  { emit setPlayButton(false);
    MsgBox->addLn("Rundgang: Play", tMsgBox::inf1);
    stopped=false;  showNextBild();
  }
}

// ----------------------------------------------------------------------------
void tRundgang::ende()
{ emit closeDlg(); clear();
  tMainWin::Steuerung.load(StartScript, StartCursorPos);
}


// ----------------------------------------------------------------------------
void tRundgang::showNextBild()
{  
  if (stopped) return;

  index++;
  if (index<0) return;
  if (index>=Bilder.count())  { ende();  return; }

  tMainWin::Steuerung.run(Bilder.at(index));

  emit setGrafikLabel(Int(index,0)+", "+Bilder.at(index));

  if (ms>900)
     { QTimer::singleShot(ms, this, SLOT(showNextBild()));
     }
}
// Ende tRundgang <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<




// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
tRundgangDlg::tRundgangDlg(QWidget * parent):
  QDialog(parent, Qt::Window |
                  Qt::CustomizeWindowHint |
                  Qt::WindowCloseButtonHint |
                  Qt::WindowTitleHint |
                  Qt::WindowStaysOnTopHint),
  //Qt::WindowContextHelpButtonHint | Qt::CustomizeWindowHint | Qt::WindowTitleHint |
  ui(new Ui::Controls)
{ setAttribute(Qt::WA_DeleteOnClose);
  ui->setupUi(this);  
}

tRundgangDlg::~tRundgangDlg()
{ MsgBox->addLn("Rundgang: Ende", tMsgBox::inf1);
  delete ui;
}


// ----------------------------------------------------------------------------
void tRundgangDlg::PauseButton()
{ tMainWin::Rundgang.pause();
}


// ----------------------------------------------------------------------------
void tRundgangDlg::PrevButton()
{ tMainWin::Rundgang.prev();

}


// ----------------------------------------------------------------------------
void tRundgangDlg::NextButton()
{ tMainWin::Rundgang.next();

}


// ----------------------------------------------------------------------------
void tRundgangDlg::PlayButton()
{ tMainWin::Rundgang.play();
}

// ----------------------------------------------------------------------------
void tRundgangDlg::reject()
{ tMainWin::Rundgang.ende();

  QDialog::reject();
}
