#include "setup.h"
#include "mainwin.h"
#include "utils/msg.h"
#include "galerie.h"
#include "steuerung.h"
#include "toolscript.h"

extern tMsgBox *MsgBox;  // MsgBox
tSetup *Setup;           // Alle Einstelungen


// tSetupDialog >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

tSetupDialog::tSetupDialog(tMainWin * w):
  QDialog(w),
  ui(new Ui::EinstDialog)
{ tSetupDialog::w=w;
  setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
  ui->setupUi(this);

  // Ordner --------------------------------------------------------------
  ui->DirScripte->setText(Setup->getDirScripte());
  ui->DirPDF->setText(Setup->getDirPDF());
  ui->DirDemoRO->setText(Setup->getDirDemoRO());

  // Format und Koordinatensystem ----------------------------------------
  tSetupTurtle Turtle=Setup->Turtle;
  ui->PapierFormat->addItem( QString("A0 quer"), tSetupTurtle::dinA0quer );
  ui->PapierFormat->addItem( QString("A0 hoch"), tSetupTurtle::dinA0hoch );
  ui->PapierFormat->addItem( QString("A1 quer"), tSetupTurtle::dinA1quer );
  ui->PapierFormat->addItem( QString("A1 hoch"), tSetupTurtle::dinA1hoch );
  ui->PapierFormat->addItem( QString("A2 quer"), tSetupTurtle::dinA2quer );
  ui->PapierFormat->addItem( QString("A2 hoch"), tSetupTurtle::dinA2hoch );
  ui->PapierFormat->addItem( QString("A3 quer"), tSetupTurtle::dinA3quer );
  ui->PapierFormat->addItem( QString("A3 hoch"), tSetupTurtle::dinA3hoch );
  ui->PapierFormat->addItem( QString("A4 quer"), tSetupTurtle::dinA4quer );
  ui->PapierFormat->addItem( QString("A4 hoch"), tSetupTurtle::dinA4hoch );
  ui->PapierFormat->addItem( QString("A5 quer"), tSetupTurtle::dinA5quer );
  ui->PapierFormat->addItem( QString("A5 hoch"), tSetupTurtle::dinA5hoch );
  ui->PapierFormat->addItem( QString("A6 quer"), tSetupTurtle::dinA6quer );
  ui->PapierFormat->addItem( QString("A6 hoch"), tSetupTurtle::dinA6hoch );
  ui->PapierFormat->addItem( QString("A7 quer"), tSetupTurtle::dinA7quer );
  ui->PapierFormat->addItem( QString("A7 hoch"), tSetupTurtle::dinA7hoch );
  ui->PapierFormat->addItem( QString("A8 quer"), tSetupTurtle::dinA8quer );
  ui->PapierFormat->addItem( QString("A8 hoch"), tSetupTurtle::dinA8hoch );
  ui->PapierFormat->addItem( QString("A9 quer"), tSetupTurtle::dinA9quer );
  ui->PapierFormat->addItem( QString("A9 hoch"), tSetupTurtle::dinA9hoch );

  for (int i=0; i<ui->PapierFormat->count(); i++)
  { if (Turtle.Din == ui->PapierFormat->itemData(i).toInt())
       ui->PapierFormat->setCurrentIndex(i);  // aktuelle Wahl
  }
  ui->RandX->setText( Real(Turtle.RandX_mm));          ui->RandX->setValidator(&DoubleValidator);
  ui->RandY->setText( Real(Turtle.RandY_mm));          ui->RandY->setValidator(&DoubleValidator);

  ui->Antialiasing->setChecked(Turtle.AntiAliasing);

  // PDF-Druck -----------------------------------------------------------
  ui->PDF_P0_dX->setText( Real(Turtle.PDF_P0_dX_mm));
  ui->PDF_P0_dY->setText( Real(Turtle.PDF_P0_dY_mm));
  ui->PDF_DPI->setText( Real(Turtle.PDF_DPI_mm));

  ui->Font->setCurrentFont( QFont(Turtle.FontName));
  ui->FontSize_mm->setText(Real(Turtle.getFontSize_mm()));

  // Einstellungen Fontsize ----------------------------------------------
  FontSize_Bakup=Setup->getFontSize();
  ui->FontSize->setText( Int(Setup->getFontSize()));

  // Editorpath  ---------------------------------------------------------
  ui->EditorPath->setText(Setup->getEditorPath() );

  ui->label_Datei->setText(Setup->getIniPfad());
}


// ----------------------------------------------------------------------------
void tSetupDialog::ButtonDirPDF()
{
  ui->DirPDF->setText( QFileDialog::getExistingDirectory(this,
                       "Ordner für PDF-Dateien wählen",
                       ui->DirPDF->text())+"/" );
}

// ----------------------------------------------------------------------------
void tSetupDialog::ButtonDirScripte()
{
  ui->DirScripte->setText( QFileDialog::getExistingDirectory(this,
                           "Ordner für Scripte wählen",
                           ui->DirScripte->text())+"/" );
  if (Setup->isDemoROPath(ui->DirScripte->text() ))
  {
    MsgBox->addLn("Err: Ordner '"+ui->DirScripte->text()+"' ist geschützt",tMsgBox::err);
    Setup->setDirScripte("");
    ui->DirScripte->setText( Setup->getDirScripte() );

  }
}


// ----------------------------------------------------------------------------
void tSetupDialog::ButtonDirDemo()
{
  ui->DirDemoRO->setText( QFileDialog::getExistingDirectory(this,
                           "Ordner für Hilfe und Demos wählen (Ro)",
                           ui->DirDemoRO->text())+"/" );
}



// ----------------------------------------------------------------------------
void tSetupDialog::ButtonEditor()
{
  QString Prog = QFileDialog::getOpenFileName(this,
                                                  "C++ Editor",
                                                   QDir::rootPath(),
                                                   "Programm (*.exe)");
  if (Prog.isEmpty()) return;
  ui->EditorPath->setText(Prog);  
}


// ----------------------------------------------------------------------------
void tSetupDialog::accept() // Taste-Ok
{ QString s;
  QApplication::setOverrideCursor(Qt::WaitCursor);

  // DirPDF --------------------------------------------------------------
  Setup->setDirScripte(ui->DirScripte->text());
  Setup->setDirPDF( ui->DirPDF->text() );
  Setup->setDirDemoRO(ui->DirDemoRO->text() );

  // Format und Koordinatensystem ----------------------------------------
  int din_i=ui->PapierFormat->currentIndex();

  Setup->Turtle.setPage(ui->PapierFormat->itemData(din_i).toInt());
  Setup->Turtle.setMargin(ui->RandX->text().toDouble(), ui->RandY->text().toDouble());
  Setup->Turtle.AntiAliasing=ui->Antialiasing->isChecked();
  Setup->Turtle.FontName=ui->Font->currentFont().family();
  Setup->Turtle.FontSize_mm=ui->FontSize_mm->text().toFloat();

  // PDF-Druck -----------------------------------------------------------
  Setup->Turtle.PDF_P0_dX_mm=ui->PDF_P0_dX->text().toFloat();
  Setup->Turtle.PDF_P0_dY_mm=ui->PDF_P0_dY->text().toFloat();
  Setup->Turtle.PDF_DPI_mm=ui->PDF_DPI->text().toFloat();

  // Textdock Fontsize ---------------------------------------------------
  Setup->setFontSize( ui->FontSize->text() );
  w->getDock()->setFontSize();
  MsgBox->setFontSize(Setup->getMsgFontSize());

  // Editorpath  ---------------------------------------------------------
  Setup->setEditorPath(ui->EditorPath->text());


  MsgBox->addLn("Setup ->ok");
  QApplication::setOverrideCursor(Qt::ArrowCursor); // WaitCursor aus
  QDialog::accept();
}


void tSetupDialog::reject()
{ MsgBox->addLn("Setup ->verworfen");
  Setup->setFontSize(Int(FontSize_Bakup,0));
  QDialog::reject();
}

// Ende tSetupDialog <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<




// tSetup >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

tSetup::tSetup()
{  Setting = new QSettings(QSettings::IniFormat,
                          QSettings::UserScope,
                          QApplication::applicationName(),
                          QApplication::applicationVersion());

   FontSize=default_FontSize;
   DirPrgm=QString(QApplication::applicationDirPath() + "/");
}

tSetup::~tSetup()
{ delete Setting;
}

//-----------------------------------------------------------------------------
void tSetup::Dialog(tMainWin *w)
{
  tSetupDialog * d=new tSetupDialog(w);
  d->exec();
}


// Einstellungen Gruppe Turtle ------------------------------------------

void tSetup::restoreT(tSetupTurtle *Turtle)
{
  Setting->beginGroup("Turtle");

  if (Setting->value("Din").isValid())
  {
  Turtle->setPage( Setting->value("P0").toPointF(), Setting->value("Din").toInt());
  Turtle->setMargin(Setting->value("RandX").toFloat(),Setting->value("RandY").toFloat());

  Turtle->AntiAliasing=Setting->value("Antialising").toBool();
  Turtle->FontName=Setting->value("FontName").toString();
  Turtle->FontSize_mm=Setting->value("FontSize").toFloat();
  }
  else MsgBox->addLn("Turtle->Defaultwerte");
  Setting->endGroup();
}

void tSetup::saveT(tSetupTurtle *Turtle)
{
  Setting->beginGroup("Turtle");
  Setting->setValue("P0", Turtle->P0P1_mm.topLeft());
  Setting->setValue("Din", Turtle->Din);
  Setting->setValue("RandX", Turtle->RandX_mm);
  Setting->setValue("RandY", Turtle->RandY_mm);
  Setting->setValue("Antialising", Turtle->AntiAliasing);
  Setting->setValue("FontName",Turtle->FontName);
  Setting->setValue("FontSize",Turtle->getFontSize_mm());
  Setting->endGroup();
}

// Einstellungen Gruppe ToolScript --------------------------------------
void tSetup::restore(tToolScriptDlg *Dlg)
{ Setting->beginGroup( Dlg->objectName());
  Dlg->restoreGeometry(Setting->value( "Geometry").toByteArray());
  int w=Setting->value("ColumnWidth0").toInt();
  Dlg->ui->HelpTree->setColumnWidth(0,(w<50)?50:w);
  w=Setting->value("ColumnWidth1").toInt();
  Dlg->ui->HelpTree->setColumnWidth(1,(w<50)?100:w);
  Dlg->ui->InsertDock->setChecked(!Setting->value("NoInsertDock").toBool());
  Dlg->ui->DefaultParameter->setChecked(Setting->value("DefaultParameter").toBool());
  Setting->endGroup();
}

void tSetup::save(tToolScriptDlg *Dlg)
{ Setting->beginGroup(Dlg->objectName());
  Setting->setValue("Geometry", Dlg->saveGeometry());
  Setting->setValue("ColumnWidth0", Dlg->ui->HelpTree->columnWidth(0));
  Setting->setValue("ColumnWidth1", Dlg->ui->HelpTree->columnWidth(1));
  Setting->setValue("NoInsertDock",!Dlg->ui->InsertDock->isChecked());
  Setting->setValue("DefaultParameter",Dlg->ui->DefaultParameter->isChecked());
  Setting->endGroup();
}




// Einstellungen Gruppe [MainWin] --------------------------------------
void tSetup::restore(tMainWin *w)
{ // Fenster einlesen
  Setting->beginGroup(w->objectName());
  w->restoreGeometry( Setting->value("Geometry").toByteArray());
  w->restoreState( Setting->value( "WindowState").toByteArray() );
  setFontSize( Setting->value("DockFontsize").toString() );
  qsrand(Setting->value("qrand").toInt());
  Setting->endGroup();

  // Pfade einlesen
  Setting->beginGroup("Pfade");
  if (Setting->value("lastDirScript").isValid())
  { setLastDirScript( Setting->value("lastDirScript").toString());
    setDirScripte( Setting->value("DirScripte").toString());
    setDirPDF(Setting->value("DirPDF").toString());
    setDirDemoRO(Setting->value("DirDemoRO").toString());
    setEditorPath(Setting->value("Editor").toString());
  }
  else
  { // Defaultwerte setzen
    setLastDirScript(getDirPrgm());
    setDirScripte( getDir(QDir::homePath()));
    setDirPDF(getDirPrgm());
    setDirDemoRO(QString());
  }
  Setting->endGroup();
  restoreT(&Turtle);

}

void tSetup::save(tMainWin *w)
{ Setting->beginGroup(w->objectName());
  Setting->setValue("Geometry", w->saveGeometry());
  Setting->setValue("WindowState", w->saveState());
  Setting->setValue("DockFontsize", getFontSize());
  Setting->setValue("qrand",qrand());  // Zufallszahlen zufällig
  Setting->endGroup();

  Setting->beginGroup("Pfade");
  Setting->setValue("lastDirScript", getLastDirScript());
  Setting->setValue("DirScripte", getDirScripte());
  Setting->setValue("DirPDF", getDirPDF());
  Setting->setValue("DirDemoRO", getDirDemoRO());
  Setting->setValue("Editor", getEditorPath());
  Setting->endGroup();

  saveT(&Turtle);
  Setting->sync();
}



// Einstellungen Gruppe GalerieDialog ---------------------------------------
void tSetup::restore(tGalerieDialog *GalerieDialog)
{Setting->beginGroup( GalerieDialog->objectName() );
 GalerieDialog->restoreGeometry(Setting->value( "Geometry").toByteArray() );
 GalerieDialog->setDialogPath( Setting->value("Last").toString());
 Setting->endGroup();
}


void tSetup::save(tGalerieDialog *GalerieDialog, const QString &DialogPfad)
{ Setting->beginGroup(GalerieDialog->objectName());
  Setting->setValue("Geometry", GalerieDialog->saveGeometry());
  if (!DialogPfad.isEmpty()) Setting->setValue("Last", DialogPfad);
  Setting->endGroup();
}


// Einstellungen Gruppe Grafik-Dock ------------------------------------
QRect tSetup::GeometrieStart(QWidget *w, qreal faktor)
{
  int dh=w->geometry().y()-w->y();

  QRect r=QApplication::desktop()->availableGeometry();//screenGeometry();
  r.setHeight((qreal) faktor*r.height() );
  r.setWidth((qreal) faktor*r.width());

  r.moveTo(dh,2*dh);
  return r;
}


void tSetup::restore(tGDock * GDock)
{ Setting->beginGroup( GDock->objectName() );

  if (!GDock->restoreGeometry(Setting->value( "Geometry").toByteArray()))
    GDock->setGeometry(GeometrieStart(GDock));

  GDock->restoreGrafik(Setting);
  Setting->endGroup();
}


void tSetup::save(tGDock * GDock)
{ Setting->beginGroup(GDock->objectName());
  Setting->setValue("Geometry", GDock->saveGeometry());
  GDock->saveGrafik(Setting);
  Setting->endGroup();
}


// Einstellungen Gruppe Text-Docks [DockXX] ----------------------------
void tSetup::restore(tDock * Dock)
{ Setting->beginGroup( Dock->objectName() );
  Dock->restoreGeometry(Setting->value( "Geometry").toByteArray() );
  Dock->setFloating( Setting->value("isFloating").toBool());
  Setting->endGroup();
}

void tSetup::save(tDock * Dock)
{ Setting->beginGroup(Dock->objectName());
  Setting->setValue("Geometry", Dock->saveGeometry());
  Setting->setValue("isFloating", Dock->isFloating());
  Setting->endGroup();
}


// Einstellungen Gruppe Steuerung --------------------------------------
void tSetup::restore(tSteuerung *Steuerung)
{ Setting->beginGroup(Steuerung->objectName());
  Steuerung->setGrafikname(Setting->value("Grafikname").toString());
  Steuerung->setCursorPos(Setting->value("CursorPos").toInt());
  Setting->endGroup();
}

void tSetup::save(tSteuerung *Steuerung)
{ Setting->beginGroup(Steuerung->objectName());
  Setting->setValue("Grafikname", Steuerung->Grafikname());
  Setting->setValue("CursorPos" , Steuerung->CursorPos());
  Setting->endGroup();

}



//----------------------------------------------------------------------
void tSetup::setFontSize(QString s)
{  FontSize=s.toInt();
   if ((FontSize<5) || FontSize>25) FontSize=default_FontSize;
}


//----------------------------------------------------------------------
QString tSetup::StyleSheet()
{
  QString style(
       "QTextEdit { font-family:Courier New; "     //"font-family:Courier New;Arial;
       "font-size:"+Int(getFontSize())+"pt; "
       "background:"+ QColor(BackcolorFix).name()+"; }"
       );


  return style;
}


// Scriptpfade ---------------------------------------------------------

bool tSetup::isDemoROPath(const QString &Path) const
{ if (getDirDemoRO().isEmpty()) return false;
  return (QString(Path+"/").indexOf(getDirDemoRO())==0);
}

//----------------------------------------------------------------------
QString tSetup::getDir(QString Path) const
{ if (Path.isEmpty()) return Path;
  if (!Path.endsWith("/"))
   {  int i=Path.lastIndexOf("/",-1);
      if (i>0) Path=Path.left(i)+"/";
  }
  return Path;
}

//----------------------------------------------------------------------
QString tSetup::getName(QString Path) const
{
  QFileInfo fi(Path);
  return fi.completeBaseName();
}


//----------------------------------------------------------------------
const QString &tSetup::getDirDemoRO() const
{ return DirDemoRO;
}

//----------------------------------------------------------------------
const QString &tSetup::getDirScripte() const
{  return DirScripte;
}

//----------------------------------------------------------------------
const QString &tSetup::getEditorPath() const
{ return EditorPath;
}


//----------------------------------------------------------------------
const QString tSetup::makePath(const QString &Praefix, const QString &Path)
{ QString s=Praefix;

  // "eigene:" oder "demo:" oder "pdf:"
  if (Praefix==PathEigene) { s=getDirScripte(); goto ExitGetPath;}
  if (Praefix==PathDemo)   { s=getDirDemoRO(); goto ExitGetPath;}
  if (Praefix==PathPDF)    { s=getDirPDF(); goto ExitGetPath;}

  s=Praefix;

  ExitGetPath:
  return QDir::cleanPath(s+Path);
}


//----------------------------------------------------------------------
QString tSetup::getPrefixPath(QString Path)
{
  Path.replace(getDirScripte(), PathEigene);
  Path.replace(getDirDemoRO(), PathDemo);
  return Path;
}


//----------------------------------------------------------------------
void tSetup::setDirScripte(const QString &Path)
{  QString Dir=getDir(Path);

   if (Dir.isEmpty() || isDemoROPath(Dir)) Dir=getDir(QDir::homePath());

  DirScripte=Dir;
}


//----------------------------------------------------------------------
void tSetup::setEditorPath(const QString &Path)
{ EditorPath=Path;
}


//----------------------------------------------------------------------
const QString &tSetup::getLastDirScript() const
{  return lastDirScript;
}

//----------------------------------------------------------------------
void tSetup::setLastDirScript(const QString &Path)
{  lastDirScript=getDir(Path);
}


//----------------------------------------------------------------------
void tSetup::setDirDemoRO(const QString &Path)
{  DirDemoRO=getDir(Path);
}



// Einstellungen Gruppe [Pfade] ----------------------------------------

const QString &tSetup::getDirPrgm() const
{ return DirPrgm;
}

const QString tSetup::getUrlHilfe() const
{ return getDirDemoRO()+HilfeDir+"1_Hilfe"+PDFSuffix;
}

const QString tSetup::getUrlScripte() const
{ return getDirDemoRO()+HilfeDir+"2_Scripte"+PDFSuffix;
}

const QString tSetup::getUrlTurtle() const
{ return getDirDemoRO()+HilfeDir+"3_Turtle"+PDFSuffix;
}

const QString tSetup::getUrlGrafik() const
{ return getDirDemoRO()+HilfeDir+"4_Grafik"+PDFSuffix;
}

const QString tSetup::getUrlFarben() const
{ return getDirDemoRO()+HilfeDir+"5_Farben"+PDFSuffix;
}

const QString tSetup::getUrlDoku() const
{ return getDirDemoRO()+HilfeDir+"6_CPlus"+PDFSuffix;
}

const QString tSetup::getUrlToolScript() const
{ return getDirDemoRO()+HilfeDir+ToolScript+ToolScriptSuffix;
}

const QString &tSetup::getDirPDF() const
{ return DirPDF;
}

void tSetup::setDirPDF(const QString &Dir)
{ if (Dir=="") DirPDF=getDir(QDir::homePath());
  else DirPDF=getDir(Dir);
}

const QString tSetup::getPathPDF(const QString &Name) const
{ return getDirPDF() + Name + PDFSuffix;
}

const QString tSetup::getUrlPDF(const QString &Name) const
{ return "file:"+getPathPDF(Name);
}


// tSetupTurtle >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
tSetupTurtle::tSetupTurtle():
  FontName("Courrier"),
  FontSize_mm(2.5),
  Din(dinA4quer),
  RandX_mm(8),
  RandY_mm(8),
  P0P1_mm(-din4/2,din5/2,din4,din5),
  PDF_P0_dX_mm(5.5),
  PDF_P0_dY_mm(4),
  PDF_DPI_mm(24.34)
{
   setPage( getP0_UrspMitte(dinA4quer), dinA4quer);
   AntiAliasing=false;
   Stift.setColor(Qt::black);
   Stift.setWidthF(0);
   Stift.setCapStyle(Qt::RoundCap);
   Stift.setJoinStyle(Qt::RoundJoin);
   Pinsel.setStyle(Qt::SolidPattern);
   Pinsel.setColor(BackcolorPainter);
}


QPointF tSetupTurtle::getP0_UrspMitte(int din)
{ QPointF P0;
  int dinLang=din4, dinKurz=din5;
  checkFormat(din, dinLang, dinKurz);

  P0.setX( -dinLang/2);
  P0.setY(  dinKurz/2 );

  return P0;
}


bool tSetupTurtle::checkFormat(int &din, int &dinLang, int &dinKurz)
{ int i=0;
  while (i<5)  // versucht ein gültiges Format zu bestimmen
  {
    switch (din)
    {case dinA0hoch: dinLang=din1; dinKurz=din0; break;
     case dinA0quer: dinLang=din0; dinKurz=din1; break;

     case dinA1hoch: dinLang=din2; dinKurz=din1; break;
     case dinA1quer: dinLang=din1; dinKurz=din2; break;

     case dinA2hoch: dinLang=din3; dinKurz=din2; break;
     case dinA2quer: dinLang=din2; dinKurz=din3; break;

     case dinA3hoch: dinLang=din4; dinKurz=din3; break;
     case dinA3quer: dinLang=din3; dinKurz=din4; break;

     case dinA5hoch: dinLang=din6; dinKurz=din5; break;
     case dinA5quer: dinLang=din5; dinKurz=din6; break;

     case dinA6hoch: dinLang=din7; dinKurz=din6; break;
     case dinA6quer: dinLang=din6; dinKurz=din7; break;

     case dinA7hoch: dinLang=din8; dinKurz=din7; break;
     case dinA7quer: dinLang=din7; dinKurz=din8; break;

     case dinA8hoch: dinLang=din9; dinKurz=din8; break;
     case dinA8quer: dinLang=din8; dinKurz=din9; break;

     case dinA9hoch: dinLang=din10; dinKurz=din9; break;
     case dinA9quer: dinLang=din9; dinKurz=din10; break;

     case dinA4hoch: dinLang=din5; dinKurz=din4; break;

     case dinA4quer: dinLang=din4; dinKurz=din5; break;

     case dinSetup:
     default: if (i==0) { i++; din=Din; continue; }
              if (i==1) { i++; din=dinA4quer; continue; }
              else { qDebug()<<"Err in checkFormat()"; return false;}
   }
   i=5;
  }
  return true;
}

qreal tSetupTurtle::checkRand(qreal Rand, qreal Laenge, qreal IniRand)
{ qreal maxRand=Laenge/2-4.0;

  if (Rand<0) Rand=IniRand; // Default aus ini-Datei
  if ((Rand<0)||(Rand>maxRand)) Rand=maxRand;

  return Rand;
}


// -----------------------------------------------------------------------------
void tSetupTurtle::setMargin(qreal RandX, qreal RandY)
{ if (!Setup) { RandX_mm=RandMinDefault; RandY_mm=RandMinDefault; return; }

  qreal dx=getP0P1().width(), dy=abs(getP0P1().height());
  qreal maxRandX=dx/2-5.0;
  qreal maxRandY=dy/2-5.0;

  if (RandX<-1)
    { // set quadratisch      
      qreal l=dy-2.0*RandY; RandX=(dx-l)/2.0;
    }
  if (RandX<0) { RandX=Setup->TurtleDefaults().RandX_mm; }
  if ((RandX<0)||(RandX>maxRandX)) RandX=maxRandX;
  RandX_mm=RandX;

  if (RandY<-1)
    { // set quadratisch
      qreal h=dx-2.0*RandX; RandY=(dy-h)/2.0;
    }
  if (RandY<0)  RandY=Setup->TurtleDefaults().RandY_mm;
  if ((RandY<0)||(RandY>maxRandY)) RandY=maxRandY;
  RandY_mm=RandY;

 //qDebug()<<RandX_mm<<RandY_mm;
}


// -----------------------------------------------------------------------------
void tSetupTurtle::setPage(int din)
{ QPointF P0(getP0_UrspMitte(din));
  setPage(P0.x(), P0.y(), din);
}

// -----------------------------------------------------------------------------
void tSetupTurtle::setPage(QPointF P0, int din)
{ setPage(P0.x(), P0.y(), din);
}

// -----------------------------------------------------------------------------
void tSetupTurtle::setPage(qreal x0, qreal y0, int din)
{
  int dinLang=din4, dinKurz=din5;
  if (!checkFormat(din, dinLang,dinKurz)) qDebug()<<"Err in setPage()";

  Din=(tDin) din;
  P0P1_mm.setLeft(x0);
  P0P1_mm.setTop(y0);
  P0P1_mm.setRight(  x0 + dinLang );
  P0P1_mm.setBottom( y0 - dinKurz );

  setMargin();

}

// -----------------------------------------------------------------------------
void tSetupTurtle::setFangRect(QRectF &r, qreal min_mm)
{ qreal d=P0P1_mm.width()/250.0;

  if (min_mm<0) d=-min_mm;
  else if (d<min_mm) d=min_mm;
  r.setRect(-d, -d, d+d, d+d);
}

void tSetupTurtle::setStiftRGB(const QString &rgb)
{ Stift.setColor( QColor(rgb));
}

QString  tSetupTurtle::getStiftRGB()
{ return Stift.color().name();
}


QRectF tSetupTurtle::getPlotRect() const
{ QRectF r=P0P1_mm;
  r.adjust(RandX_mm, -RandY_mm, -RandX_mm, RandY_mm);
  return r.normalized();
}


qreal tSetupTurtle::getInfoPosX()
{ return P0P1_mm.x()+RandX_mm+RandX_mm+2;
}

qreal tSetupTurtle::getInfoPosy()
{ return P0P1_mm.y()- abs(P0P1_mm.height()) + RandY_mm+ RandY_mm+2;

}

qreal tSetupTurtle::getFontSize_mm()
{
  return (FontSize_mm>0) ? FontSize_mm : 2.5;
}

