/******************************************************************************
turtle: turlegrafik

Copyright (C) 2011	Günther Schardinger,
                    Forstweg 12,
                    8793-Trofaiach, Austria
                    g.schardinger@aon.at

Dieses Programm ist freie Software. Sie können es unter den Bedingungen der
GNU General Public License, wie von der Free Software Foundation veröffentlicht,
weitergeben und/oder modifizieren, entweder gemäß Version 3 der Lizenz oder
jeder späteren Version.

Die Veröffentlichung dieses Programms erfolgt in der Hoffnung, daß es Ihnen von
Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite
Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK.
Details finden Sie in der GNU General Public License.
GNU General Public License siehe <http://www.gnu.org/licenses/>.
*******************************************************************************/
#ifndef SETUP_H
#define SETUP_H
#include <QtGui>
#include "ui_einstdialog.h"

class tMainWin;
class tDock;
class tGDock;
class tGalerie;
class tSteuerung;
class tSetupDialog;
class tGalerieDialog;
class tGalerieDialog;
class tToolScriptDlg;


/*! Modul Setup >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

Im Modul Setup findet man alle Konstanten und Einstellungen des Programms.

class tSetup
  Liefert alle Einstellungen. Gespeicherte Einstellungen werden mit
  QSettings in einer ini-Datei gruppiert gespeichert.

  Einstellungen beliebiger Objekte (z.B tMyObject) werden in den Funktionen
  save(tMyObject *w)' und 'restore(tMyObject *w)' behandelt.
  Siehe 'tSetup::save(tMainWin *w)' und 'tSetup::restore(tMainWin *w)'.

  Fuer globale Voreinstellungen steht ein befreundetes
  Dialog-Fenster der Klasse tSetupDialog zur Verfuegung.

class tSetupTurtle
  Grundeinstellungen fuer die Turtle. Siehe auch tTutle.

Modul Setup <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<*/

// Konstanten fuer Application >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//
const QString  AppName("turtle");        // Name des Programms
const QString  AppIcon(":/turtle.ico");
const QString  AppVers("2.20");          // Versionsnummer, Setup-Dateiname
const QString  AppTrDe(":/qt_de.qm");    // deutsche Uebersetzung
const QString  IconCPlus(":/cplus.ico");

//
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<


// Fontsize fuer Text-Dock ------------------------------------------
const int default_FontSize=11;

// FontScale fuer PDF-Printer ---------------------------------------
const qreal PDFFontScale=12.5;

// Hintergrund Farben -----------------------------------------------
        //MsgBox            = 0xCDE5A9
const int BackcolorFix      = 0xf9edd4;  //
const int BackcolorPainter  = 0xFFFFFF;  // Hintergrund, Painter
const int BackcolorGrafik   = 0xc0c0c0;  // c7f1eb
const int BackcolorDefault  = 0x666666;  //

// Hintergrund Farben -----------------------------------------------
const int ColorHeadLine     = 0xDDDDDD;  //
const int ColorLine         = 0xEEEEEE;  //
const int ColorScriptLine   = BackcolorFix ;
const int ColorDemoLine     = 0xe2fbe4;
const int ColorHeller       = 0x111111;


// Formate, Laser: 192x273mm ----------------------------------------
const int din11 = 26.0;   // A11 26x37 mm
const int din10 = 37.0;   // A10 26x37 mm
const int din9  = 52.0;   // A9  37x52 mm
const int din8  = 74.0;   // A8  52x74 mm
const int din7  = 105.0;  // A7  74x105 mm
const int din6  = 148.0;  // A6  105×148 mm
const int din5  = 210.0;  // A5  148×210 mm
const int din4  = 297.0;  // A4  210×297 mm
const int din3  = 420.0;  // A3  297×420 mm
const int din2  = 594.0;  // A2  420×594 mm
const int din1  = 841.0;  // A1  594×841 mm
const int din0  = 1189.0; // A0  841x1189 mm

const qreal RandMinDefault = 8;

// Hilfe ------------------------------------------------------------
const QString HilfeDir("Hilfe/");       // Ordner für Dokus
const QString PDFSuffix(".pdf");        // für PDF's
const QString ScriptSuffix(".txt");     // für Scripts
const QString ToolScriptSuffix(".h");   // für Befehlsliste

// Toolscript -------------------------------------------------------
const QString ToolScript("scriptbefehle");
const QString ToolScriptGruppeBegin("! |");
const QString ToolScriptGruppeEnd("|");
const QString ToolScriptEnd("! End");
const QString ToolScriptRem("//");
const char    ToolScriptTop3('~');
const qint64  MaxLineLen = 200;
const int     BackColorTop1=0x87CEEB;//0x4682B4;//0xA9A9A9;
const int     BackColorTop2=0xCDE5A9;//0x8FBC8F;//0xC9C9C9;
const int     BackColorTop3=0xECE9D8;//0xF5F5DC;//0xFAEBD7;
const int     BackColor1=0xEEEEEE;
const int     BackColor2=0xD3D3D3;

// Scripts ----------------------------------------------------------
// weitere Scriptbezeichner befinden sich in den speziellen Grafiken

const QString ScriptBegin("ScriptBegin");  // Script Anfang
const QString ScriptEnd("ScriptEnd");      // Script Ende
const QString ScriptInit("init");          // Scriptfunktion init()
const QString ScriptDraw("draw");          // Scriptfunktion draw()
const QString ScriptVarGrafik("Grafik");   // Scriptvariable Grafik
const QString ScriptVarSec("sec");         // Scriptvariable sec
const QString ScriptVar("var");            // Javascript var
const QString ScriptConst("const");        // Javascript const
const QString ScriptFunktion("function");  // Javascript function

const QString ScriptBez("Script");
const QString ScriptGrafikDefault("Script Turtle");

      // Pfadbezeichner für Scripte
const QString PathEigene("eigene:");  // Ordner für eigene Scripte
const QString PathDemo("demo:");      // Ordner für Demos, read-only
const QString PathCplus("c++:");      // interne C++ Grafik
const QString PathPDF("pdf:");        // Ordner für eigene PDF

const QString ScriptHinweis("Script mit Doppelklick oder STRG+R aufrufen!");
const QString ScriptLine(   "____________________________________________") ;
const QString ScripName (   ScriptBez+":");

// Sonderzeichen ----------------------------------------------------
const QChar HexFillChar('0');  // Fuellzeicheen fuer Hexzahlen
const QChar SepKlammerAuf('(');// Klammer
const QChar SepKlammerZu(')'); // Klammer
const QChar SepBlockAuf('{'); // Klammer
const QChar SepGleich('=');    // Gleichheitszeichen
const QChar SepHochkomma('"');
const QChar SepOpen('(');      // Trenner fuer Listen
const QChar SepLst(';');       // Trenner fuer Listen
const QChar SepBlank(' ');     // Trenner
const QChar SepWert(':');      // Trenner zwischen Namen:Wert
const QChar SepParam(',');     // Trenner
const QChar SepMarke('#');     // Trenner zwischen Namen#Marke
const QChar SepPfad('/');      // Trenner zwischen Namen#Marke
const QChar SepObjekt('.');    // Trenner füe Objektnamen

//#define TYP_G 'g'

// Realwerte --------------------------------------------------------
const qreal epsilon(0.0001);    // Epsilonumgebung für Turtle
const double Pi = 3.14159265358979323846264338327950288419717;
const double TwoPi = 2.0 * Pi;


// tSetupTurtle >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//
// Spezielles Setupobjekt fuer die Klasse Turtle.
//
class tSetupTurtle
{
 public:
    tSetupTurtle();

    // Din-Formate fuer den Papierbereich
    enum tDin{ dinA0quer=0x00, dinA0hoch=0x00+0x0a,
               dinA1quer=0x01, dinA1hoch=0x01+0x0a,
               dinA2quer=0x02, dinA2hoch=0x02+0x0a,
               dinA3quer=0x03, dinA3hoch=0x03+0x0a,
               dinA4quer=0x04, dinA4hoch=0x04+0x0a,
               dinA5quer=0x05, dinA5hoch=0x05+0x0a,
               dinA6quer=0x06, dinA6hoch=0x06+0x0a,
               dinA7quer=0x07, dinA7hoch=0x07+0x0a,
               dinA8quer=0x08, dinA8hoch=0x08+0x0a,
               dinA9quer=0x09, dinA9hoch=0x09+0x0a,
               dinSetup=0xff};

    // setPage( ... ): Papierbereich, Zeichenbereich und Rand festlegen.
    // Zeichenbereich plus Rand ergibt den Papierbereich.
    // Papierbereich: Festgelegt durch Ecke P0 links oben und P1 rechts unten.
    // dinSetup:     Default-Format aus ini-Datei verwenden oder dinA4quer
    // Rand=-1:       Default aus ini-Datei  oder 0
    void setPage(int din=dinSetup);
    // Ursprung  in Blattmitte
    void setPage(qreal x0, qreal y0, int din=dinSetup);
    void setPage(QPointF P0, int din=dinSetup);
    // Ursprung ergibt sich aus Ecke P0(x0,y0) und dem Format,

    void setMargin(qreal RandX=-1, qreal RandY=-1);
    // RandX=-1 oder RandY=-1: Defaultwert aus ini-Datei
    // RandX=-2: Zeichenrechteck quadratisch


    void setFangRect(QRectF &r, qreal min_mm=0);
    // Fangrechteck setzen.
    // Option: min_mm=0 -> automatisch berechnen
    //         min_mm>0 -> automatisch berechnen, aber nicht kleiner
    //         min_mm<0 -> genau Betrag von min_mm

    tDin    getDin() const {return Din;}        // Papierformat
    QRectF  getP0P1()const {return P0P1_mm;}    // P0,P1 vom Papierbereich mit Rand
    QRectF  getPlotRect() const;                // Zeichenbereich: Papierbereich ohne Rand

    qreal   getPDF_P0_dX() const {return PDF_P0_dX_mm;}
    qreal   getPDF_P0_dY() const {return PDF_P0_dY_mm;}
    qreal   getPDF_DPI()   const {return PDF_DPI_mm;  }
    // Für PDF-Druck: Translation von Ecke P0 in x-Richtung und y-Tichtung
    // getPDF_DPI() liefert die tatsächliche Länge für Dots-Per-Inch Punkte.

    void    setStiftRGB(const QString &rgb);    // Penfarbe setzen
    QString getStiftRGB();                      // Penfarbe lesen

    qreal   getInfoPosX();     // Position fuer Textinfos
    qreal   getInfoPosy();     // Position fuer Textinfos

    qreal   getFontSize_mm();  // Fontsize
    void    setFontSize_mm(qreal mm) {FontSize_mm=mm;} // Fontsize

    QPen        Stift;         // Zeichenstift
    QBrush      Pinsel;        // Hintergrund
    QString     FontName;      // Font
    bool        AntiAliasing;  // Antialiasing

private:
    bool    checkFormat(int &din, int &dinLang, int &dinKurz); // ein gueltiges Format zu bestimmen
    qreal   checkRand(qreal Rand, qreal Laenge, qreal IniRand); // gueltigen Rand bestimmen, Format zuerst bestimmen!
    QPointF getP0_UrspMitte(int din);
                                // Ursprung in Blattmitte zum Format din.
                                // Rueckgabe: P0 Ecke links oben

    qreal       FontSize_mm;   // Fontsize
    tDin        Din;           // Papierformat
    qreal       RandX_mm;      // Rand
    qreal       RandY_mm;      // Rand

    QRectF      P0P1_mm;       // P0 Ecke links oben, P1 Ecke rechts unten

    qreal       PDF_P0_dX_mm;  // PDF-Druck: Translation von P0 in x-Richtung
    qreal       PDF_P0_dY_mm;  // PDF-Druck: Translation von P0 in y-Richtung
    qreal       PDF_DPI_mm;    // PDF-Druck: Tatsächliche Länge für die DPI
                               // Beispiel: 1200 Dots-Per-Inch ergeben 24.34 mm

    friend class tSetup;        // Dialogfenster hat Zugriff auf Setup
    friend class tSetupDialog;  // Dialogfenster hat Zugriff auf Setup

};// Ende tSetupTurtle <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<




// tSetup >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// Alles rund um Einstellungen:
//       eingeben, ueberpruefen, aendern, speichern und verwalten
//
class tSetup
{
public:

  tSetup();
  ~tSetup();

  // Hilfsfunktionen -----------------------------------------------------

  QString getIniPfad() {return Setting->fileName();}
  // Pfad der aktuellen Ini-Datei

  QString getDir(QString Path) const;
          // Dir ohne Dateiname aus Pfad berechnen

  QString getName(QString Path) const;
          // Dateiname aus Pfad berechnen

  const QString makePath(const QString &Praefix, const QString &Path);
                // Präfix "eigene:", "demo:", "c++:", "C:"
                // "c:/ordner/", "../ordner1/" usw.

  QString getPrefixPath(QString Path);
          // Pfadteile durch Präfixe "eigene:", "demo:" ersetzen

  bool    isDemoROPath(const QString &Path) const;
          // Path ist schreibgeschützt.
          // Path liegt im Programmverzeichnis


  // Dialogfenster fuer Einstellungen ------------------------------------
  void Dialog(tMainWin *w);   // Dialogfenster  fuer Setup
  friend class tSetupDialog;  // Dialogfenster hat Zugriff auf Setup


  // Einstellungen Gruppe [MainWin] --------------------------------------
  void   restore(tMainWin *w);
  void   save(tMainWin *w);

  void   setFontSize(QString s);            // Schriftgroesse fuer Docks
  int    getFontSize()    {return FontSize;}
  qreal  getMsgFontSize() {return (qreal)FontSize*0.9;} 

  QString StyleSheet(); // Stylesheet passend zu
                        // font, font-size, background-color

  // Einstellungen Gruppe [Pfade] ----------------------------------------
  const QString &getDirPrgm() const;
                // Programm-Ordner

  const QString &getLastDirScript() const;
  void           setLastDirScript(const QString &Path);
                 // Letzter verwendete Scriptordner

  const QString &getDirScripte() const;
                // Scriptordner für Präfix "eigene:"

  const QString &getDirDemoRO() const;
                // Scriptordner für Präfix "demo:" readonly

  const QString &getEditorPath() const;
                // Externer Editor


  // Hilfedateien --------------------------------------------------------
  const QString getUrlHilfe() const;      // Url fuer Hilfe
  const QString getUrlScripte() const;    // Url fuer Script-Hilfe
  const QString getUrlTurtle() const;     // Turtlebefehle
  const QString getUrlGrafik() const;     // Grafikbefehle
  const QString getUrlFarben() const;     // Url fuer Colors
  const QString getUrlDoku() const;       // Url fuer Doku
  const QString getUrlToolScript() const; // Url ToolScript Hilfe

  const QString &getDirPDF() const;       // Verzeichnis fuer PDF-Dateien
  const QString getPathPDF(const QString &Name) const; // PDF-Pfad
  const QString getUrlPDF (const QString &Name) const; // PDF-Url



  // Einstellungen Gruppe Grafik-Dock ------------------------------------
  void restore(tGDock *GDock);
  void save(tGDock *GDock);  

  // Einstellungen Gruppe GrafikDialog -------------------------------------
  void restore(tGalerieDialog *GalerieDialog);
  void save(tGalerieDialog *GalerieDialog, const QString &DialogPfad);

  // Einstellungen Gruppe Text-Docks [DockXX] ----------------------------
  void restore(tDock *Dock);
  void save(tDock *Dock);

  // Einstellungen Gruppe Steuerung --------------------------------------
  void restore(tSteuerung *Steuerung);
  void save(tSteuerung *Steuerung);

  // Grundeinstellungen Gruppe Turtle -------------------------------------
  void restoreT(tSetupTurtle *Turtle);
  void saveT(tSetupTurtle *Turtle);
  tSetupTurtle const &TurtleDefaults() {return Turtle;}

  // Einstellungen Gruppe ToolScript --------------------------------------
  void restore(tToolScriptDlg *Dlg);
  void save(tToolScriptDlg *Dlg);

  bool getAntiAliasing() {return Turtle.AntiAliasing;}

private:
    void setDirPDF(const QString &Dir);      // Verzeichnis fuer PDF-Dateien setzen
    void setDirScripte(const QString &Path); // Verzeichnis für Scripte
    void setDirDemoRO(const QString &Path);  // Verzeichnis für Hilfe und Demos
    void setEditorPath(const QString &Path); // Externer Editor

    QString DirScripte;      // Ordner für eigene Scripte
    QString lastDirScript;   // letzter verwendeter Scriptordner
    QString DirPrgm;         // Programmordner
    QString DirDemoRO;       // Ordner für Demos schreibgeschützt
    QString EditorPath;      // Externer Texteditor

    QRect GeometrieStart(QWidget *w, qreal faktor=0.8); // Startwert für GDockfenster

    tSetupTurtle Turtle;     // Grundeinstellungen der Turtle
    QString      DirPDF;     // PDF-Ordner
    int          FontSize;   // Schriftgroesse
    QSettings   *Setting;
};// Ende tSetup <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<



// tSetupDialog >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// Dialogfenster fuer Setups
//
class tSetupDialog : public QDialog
{
  Q_OBJECT
public:
   tSetupDialog(tMainWin * w);
  ~tSetupDialog() { delete ui; }


  void accept();                    // Dialog mit Ok beendet
  void reject();                    // Dialog mit ESC beendet

public slots:
  void ButtonDirScripte();
  void ButtonDirPDF();
  void ButtonEditor();
  void ButtonDirDemo();

private:
  QDoubleValidator DoubleValidator;
  int              FontSize_Bakup;
  tMainWin        *w;
  Ui::EinstDialog *ui;                       // Userinterface vom Qt-Designer
};
// Ende tSetupDialog <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<



#endif // SETUP_H
