#include "utils/msg.h"
#include "galerie.h"
#include "steuerung.h"


extern tMsgBox   *MsgBox;
extern tSetup    *Setup;


//-----------------------------------------------------------------------------
tSteuerung::tSteuerung():QObject(0),
  FlagSetDrawEnde(false),
  cursorPos(0),
  geladen(false),
  useScript(false)
{ setObjectName("Steuerung");
}


// ----------------------------------------------------------------------------
void tSteuerung::setMainWin(tMainWin *MainWin)
{ mainWin=MainWin;
  Setup->restore(this);
}


//-----------------------------------------------------------------------------
bool tSteuerung::load()
{
  geladen=false; useScript=false;

  if (pfad.isEmpty()) return geladen;

  if (praefix==PathCplus)
  { // C++ Grafik laden
      tGDock *GDock=mainWin->Galerie.show(pfad);
      if (GDock) geladen=true;
  }
  else
  { // Script oder PDF laden

    if (!pfad.endsWith(ScriptSuffix))
          geladen=mainWin->showUrl( Setup->makePath(praefix,pfad));
    else
    { geladen=mainWin->loadScript( Setup->makePath(praefix,pfad),cursorPos);
      useScript=geladen;
    }
  }

  if (geladen)
  { MsgBox->addLn("'"+praefix+pfad+marke+"' geladen!", tMsgBox::inf1 );    
  }
  else  MsgBox->addLn("Err: Beim Laden'"+praefix+pfad+marke+"'", tMsgBox::err );
 return geladen;
}


//-----------------------------------------------------------------------------
bool tSteuerung::load(const QString &Grafikname, int CursorPos)
{
  geladen=false; cursorPos=CursorPos;

  if (!splitGrafikname(Grafikname)) return geladen;
  return load();
}


//-----------------------------------------------------------------------------
bool tSteuerung::runScript(const QString Marke)
{

  tGDock *GDock=mainWin->Galerie.show(mainWin->getDock(),Marke);
  if (GDock)
  { useScript=true;
    setCursorPos(mainWin->getDock()->getCursorPos());
    return true;
  }
  return false;
}


//-----------------------------------------------------------------------------
bool tSteuerung::run(const QString &aGrafikname, int CursorPos)
{
  if (!load(aGrafikname, CursorPos)) return 0;

  // C++ Grafik
  if (praefix==PathCplus)
   {  if (marke.isEmpty()) return true;
      else
       { // Script zut C++ Grafik ausführen
         useScript=true;
         return runScript(marke);
       }
   }

  // Script
  if (useScript) return runScript(marke);

  // PDF
  return true;
}


//-----------------------------------------------------------------------------
bool tSteuerung::call(tGrafik *Grafik, const QString &nextGrafikname)
{ tCall call;

  if (lastCaller!=Grafikname())
  { // Aktuelle Grafik auf den Stack
    lastCaller=Grafikname();
    call.Grafik=Grafikname(); call.CursorPos=CursorPos();
    stack.push(call);
  }

  // Nächste Grafik auf den Stack
  call.Grafik=nextGrafikname; call.CursorPos=0;
  stack.push(call);

  // Signal tTurtle.DrawEnde einschalten
  FlagSetDrawEnde=true; SetDrawEnde(Grafik);
 return true; 
}


//-----------------------------------------------------------------------------
void tSteuerung::SetDrawEnde(tGrafik *Grafik)
{
  if (FlagSetDrawEnde)
  { connect(Grafik,SIGNAL(DrawEnde(QString)), this, SLOT(DrawEnde(QString)));
    FlagSetDrawEnde=false;
  }
  else ClrDrawEnde();
}


//-----------------------------------------------------------------------------
void tSteuerung::DrawEnde(QString Titel)
{
  tMainWin::Galerie.stopSignals(Titel);
  if (stack.isEmpty())
  {  // keine Signale mehr von Grafik     
     FlagSetDrawEnde=false;
     lastCaller.clear();
  }
  else
  {  // nächste Grafik vom Stack ausführen
     tCall call=stack.pop();
     FlagSetDrawEnde=true;
     run(call.Grafik, call.CursorPos);
  }
}


//-----------------------------------------------------------------------------
void tSteuerung::ClrDrawEnde()
{ FlagSetDrawEnde=false;
  lastCaller.clear();
  stack.clear();
}


//-----------------------------------------------------------------------------
void tSteuerung::printPDF()
{ tGDock *GDock=0; QString PDFName;

  if (useScript)
  { // Scriptgrafik
    // Grafik zum Script laden
    GDock=mainWin->Galerie.show(mainWin->getDock());
    PDFName= Setup->getName(mainWin->getDock()->getPfad());
  }
  else
  { // C++ Grafik anzeigen
    GDock=mainWin->Galerie.show(pfad);  
  }

  if (GDock)
  {  if (PDFName.isEmpty())  PDFName= GDock->objectName();

     MsgBox->addLn("PDF '"+ PathPDF+"' erzeugen ...");

     GDock->printPDF(Setup->getPathPDF(PDFName));

     QUrl url( Setup->getUrlPDF(PDFName),
               QUrl::TolerantMode);
     if (!QDesktopServices::openUrl(url))
          MsgBox->addLn("Err: PDF-Datei '"+url.toString()+"' nicht gefunden",tMsgBox::err );

   }
  else
  MsgBox->addLn("Err: Keine Grafik für PDF-Ausgabe!",tMsgBox::err);
}


//-----------------------------------------------------------------------------
QString tSteuerung::CPlusGrafik()
{
  if (praefix==PathCplus) return pfad;
  else return ScriptGrafikDefault;
}


//-----------------------------------------------------------------------------
QString tSteuerung::Grafikname()
{ return praefix+pfad+marke;
}


//-----------------------------------------------------------------------------
bool tSteuerung::setGrafikname(const QString &Grafikname)
{ return splitGrafikname(Grafikname);
}

//-----------------------------------------------------------------------------
int tSteuerung::CursorPos()
{ return cursorPos;
}


//-----------------------------------------------------------------------------
void tSteuerung::setCursorPos(int pos)
{ cursorPos=pos;
}


//-----------------------------------------------------------------------------
bool tSteuerung::splitGrafikname(const QString &Grafikname)
{
  return splitGrafikname(Grafikname, praefix, pfad, marke);
}


//-----------------------------------------------------------------------------
QString tSteuerung::makeGrafikname(const QString &Grafikname)
{ QString Praefix, Pfad, Marke;
  if (splitGrafikname(Grafikname, Praefix, Pfad, Marke))
    return Praefix+Pfad+Marke;
  else return "";
}

//-----------------------------------------------------------------------------
bool tSteuerung::splitGrafikname(const QString &Grafikname,
      QString &Praefix, QString &Pfad, QString &Marke)
{
  Praefix.clear(); Pfad.clear();  Marke.clear();

  // Grafikname: Scriptbezeichner
  // Grafikname "eigene:verz/.../Name.txt#marke"
  // Grafikname "/verz/.../Name.txt#marke"
  // Grafikname "./verz/.../Name.txt#marke"
  if (Grafikname.isEmpty()) return false;

  QString Name=Grafikname.trimmed();
#ifdef Q_OS_LINUX
  //Linux keine Laufwerksbuchstaben! ------------------------------------------
  //SepObjekt='.'  SepWert=':'  SepMarke='#'
  if (Name.indexOf(SepObjekt)==0)
  { Name=QDir::cleanPath( Setup->getLastDirScript()+Name); }

  QStringList l= Name.split(SepWert,QString::SkipEmptyParts);
   if (!l.isEmpty())
   { if (l.count()==2)
     {// l(0) "eigene:", "demo:", "C:"
      // l(1) "/verz/Name.txt#marke;"
      Praefix=l.at(0)+SepWert;  // "eigene:"
      Name=l.at(1);
     }
   }

   QStringList ll=Name.split(SepMarke,QString::SkipEmptyParts);
   // ll(0) "/verz/Name.txt",
   // ll(1) "marke"

   if (ll.count()<=0) return false;
   Pfad= ll.at(0);      // "verz/Name.txt"

   if (ll.count()==2) Marke= SepMarke+ll.at(1); // "marke"

#else
   // Windows dateinamen mit Laufwerksbuchstaben! -----------------------------
   // SepObjekt='.'  SepWert=':'  SepMarke='#'
   if (Name.indexOf(SepWert)<0)
   { Name=QDir::cleanPath( Setup->getLastDirScript()+Name); }

    QStringList l= Name.split(SepWert,QString::SkipEmptyParts);
    if (l.isEmpty()) return false;

    if (l.count()<2) return false;
    // l(0) "eigene:", "demo:", "C:"
    // l(1) "/verz/Name.txt#marke;"
    Praefix=l.at(0)+SepWert;  // "eigene:"

    QStringList ll=l.at(1).split(SepMarke,QString::SkipEmptyParts);
    // ll(0) "/verz/Name.txt",
    // ll(1) "marke"

    if (ll.count()<=0) return false;
    Pfad= ll.at(0);      // "verz/Name.txt"

    if (ll.count()==2) Marke= SepMarke+ll.at(1); // "marke"
    // Windows ende -----------------------------------------------------------
#endif
   return true;
}


//-----------------------------------------------------------------------------
QString tSteuerung::Info()
{
  return QString(
        "Grafikname: "+praefix+pfad+marke+"\n"
        "CursorPos:" + Int(cursorPos,0)+"\n"
        "UseScript:" + Bool(useScript)+"\n"
        "Stack:"+Int(stack.count())+"\n"
         );
}


//-----------------------------------------------------------------------------
QString tSteuerung::StackInfo()
{
  QString s;
  for (int i=0; i<stack.count(); i++)
   {
    s+=stack.at(i).Grafik+", "+Int(stack.at(i).CursorPos,0) +"\n";
   }

return s;
}


