#ifndef STEUERUNG_H
#define STEUERUNG_H
#include "mainwin.h"

class tGrafik;

//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//
// Aufrufsteuerung für Grafiken und Scriptgrafiken
//
//
//
//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
class tSteuerung : public QObject
{ Q_OBJECT

public:
  tSteuerung();

  struct tCall
  { QString Grafik;
    int CursorPos;
  };

  void setMainWin(tMainWin *MainWin);

  bool load();
       // last Grafik oder PDF laden und anzeigen.
       // last Scripte nur laden nicht ausführen.

  bool load(const QString &Grafikname, int CursorPos=0);
       // Grafik oder PDF laden und anzeigen.
       // Scripte nur laden nicht ausführen.

  bool runScript(const QString Marke="");
       // Script ScriptEnd#Marke ausführen
       // Marke=="": Script an der Cursorposition

  bool run(const QString &Grafikname, int CursorPos=0);
       // C++ Grafik oder Script ausführen

  void printPDF();
       // last Grafik oder Script als PDF ausgeben

  bool splitGrafikname(const QString &Grafikname,
                       QString &Praefix, QString &Pfad, QString &Marke);
       // Grafikname in Praefix, Bezeichner und Marke zerlegen
       // ./ und ../ auflösen bezüglich LastDirScript
       // Präfix: "eigene:", "demo:", "c++:", "C:" usw.
       // Pfad: ordner/.../xyz.xy
       // Marke: #xyz

  QString makeGrafikname(const QString &Grafikname);
       // ./ und ../ bezüglich LastDirScript auflösen

  QString CPlusGrafik();
       // C++ Grafikname. Default: "Script Turtle"

  QString Grafikname();
  bool    setGrafikname(const QString &Grafikname);
  int     CursorPos();
  void    setCursorPos(int pos);

  QString Info();
  QString StackInfo();

  //-------------------------------------------------------
  // Für Rekursive call()'s verwendet die Steuerung einen
  // Aufrufstack für Grafiken.
  //
  // Mit FlagSetDrawEnde=true wird das Signal Grafik.DrawEnde
  // mit dem Slot DrawEnde verbunden.
  //

public slots:
  void DrawEnde(QString Titel);
       // Slot für Signal tTurtle.DrawEnde(Titel).
       // Das Signal wird von Grafik nach draw() gesendet.
       // stack.count()>1 : Pop Grafikname, run(Grafikname)
       // stack.count()==0: stopSignals(Titel)

public:
  bool call(tGrafik *Grafik, const QString &nextGrafikname);
       // Die 'aktuelle' Grafik beenden und Grafik 'nextGrafikname'
       // ausführen. Nach draw() wieder die 'aktuelle' Grafik
       // mit run('aktuelle') ausführen.

  void SetDrawEnde(tGrafik *Grafik);
       // FlagSetDrawEnde auswerten:
       // FlagSetDrawEnde==true: Signal Grafik.DrawEnde mit
       //                        Slot DrawEnde verbinden
       //
       // Wird von Galerie.show(Titel,ScriptEngine) verwendet.

  void ClrDrawEnde();
       // stack löschen und FlagSetDrawEnde=false;

private:
  bool FlagSetDrawEnde;
       // Steuert das Verhalten der Funktion SetDrawEnde()
       // true : Signal Grafik.DrawEnde mit Slot DrawEnde verbinden
       // false: stack löschen

  QString lastCaller;

  //-------------------------------------------------------

private:
  tMainWin *mainWin;   // Hauptfenster

  bool splitGrafikname(const QString &Grafikname);
       // Grafikname in praefix, pfad und marke zerlegen

  QString   praefix;   // Präfix: "eigene:", "demo:", "c++:", "C:" usw.
  QString   pfad;      // Pfad/Name ohne Laufwerk: ordner/.../xyz.xy
  QString   marke;     // Script-Marke: #xyz
  int       cursorPos; // Cursorposition oder 0
  bool      geladen;   // Grafik oder Script geladen

  bool      useScript; // true: Grafik verwendet Script




  QStack <tCall> stack; //


};

#endif // STEUERUNG_H
