#include "toolscript.h"
#include "utils/msg.h"
#include "utils/utils.h"
#include "setup.h"

extern tMsgBox   *MsgBox;
extern tSetup    *Setup;

//-----------------------------------------------------------------------------
tToolScriptDlg::tToolScriptDlg(QWidget *parent):
  QWidget(parent, Qt::Window),
  ui(new Ui::ToolScript),
  LastTopItem(0),
  LastItem(0)
{
  setAttribute(Qt::WA_DeleteOnClose);
  ui->setupUi(this);

  Setup->restore(this);
  //qDebug()<<objectName();

  makeHelpTree();
}

//-----------------------------------------------------------------------------
tToolScriptDlg::~tToolScriptDlg()
{ tMainWin::ToolScript=0;
}

//-----------------------------------------------------------------------------
void tToolScriptDlg::setBefehl()
{
  QString Befehl=ui->HelpTree->currentItem()->text(1);
  bool  isBefehl=ui->HelpTree->currentItem()->data(1,Qt::UserRole).toBool();
  QString modul;

  if (isBefehl)
  {
     QTreeWidgetItem *item=ui->HelpTree->currentItem()->parent();
     if (item) modul=item->text(0);

     FormatBefehl(modul, Befehl, ui->DefaultParameter->isChecked());

     kopieren(Befehl);
     if (ui->InsertDock->isChecked()) insertDock(Befehl);
  }
}



//-----------------------------------------------------------------------------
void tToolScriptDlg::kopieren(const QString &Befehl)
{  QApplication::clipboard()->setText(Befehl);
}



//-----------------------------------------------------------------------------
void tToolScriptDlg::insertDock(const QString &Befehl)
{ tMainWin::getDock()->insertPlainText(Befehl);
}


//-----------------------------------------------------------------------------
QTreeWidgetItem *tToolScriptDlg::makeTopLevelItem(QString &Text, int BackColor)
{
  // Text:  "Objekt.Info"
  QString objekt=stripFirst(Text,SepObjekt);
  if (objekt.isEmpty()) objekt="";
  else objekt=objekt+SepObjekt;

  QTreeWidgetItem *item= new  QTreeWidgetItem(ui->HelpTree);
  item->setTextAlignment(1,Qt::AlignTop);
  item->setText(0,objekt);
  item->setText(1,Text);
  item->setBackground(1,QBrush(BackColor));
  item->setBackground(2,QBrush(BackColor));
  LastTopItem=item;
  LastItem=item;
  return item;
}


//-----------------------------------------------------------------------------
QTreeWidgetItem *tToolScriptDlg::makeItem(QString &Text, QColor Color,
                                          QColor  BackColor, bool isBefehl)
{
  if (!LastItem) return 0;
  QTreeWidgetItem *item= new  QTreeWidgetItem(LastTopItem);
  item->setTextAlignment(1,Qt::AlignTop);
  item->setText(1,Text);
  item->setForeground(1,QBrush(Color) );
  item->setBackground(1,QBrush(BackColor));
  item->setBackground(2,QBrush(BackColor));
  item->setData(1,Qt::UserRole,QVariant(isBefehl));
  LastItem=item;
  return item;
}


//-----------------------------------------------------------------------------
void tToolScriptDlg::makeHelpTree()
{
  //MsgBox->addLn(Setup->getUrlToolScript());

  ui->HelpTree->clear(); LastTopItem=0; LastItem=0;

  QFile file(Setup->getUrlToolScript());
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
    { MsgBox->addLn("Err: '"+Setup->getUrlToolScript()+"' nicht geladen!", tMsgBox::err);
      return;
    }
    QString s; QByteArray puf(MaxLineLen,0);
    bool readScript=false; // true: Zeilen auswerten

    // Befehlsliste zeilenweise lesen
    while (file.readLine(puf.data(),MaxLineLen)>=0)
     {s=puf.simplified();
      s=s.trimmed();
      if (s.isEmpty()) continue;

      if (!readScript)
      { // Scriptanfang testen
        if (s.contains(ToolScriptGruppeBegin)) readScript=true;
        else continue;
      }
      // readScript
      if (s.contains(ToolScriptEnd)) {readScript=false;  continue; }


      if (getSubString(s,ToolScriptGruppeBegin,ToolScriptGruppeEnd))
      { // s=" /*! |Objekt.Titel|***/
        // s=" /*! |Titel|***/
        // s=" /*! |~Untertitel|***/

        // Option s="Objekt.Gruppe"
        if (s.contains(SepObjekt))
         { makeTopLevelItem(s,BackColorTop1);
           isBackColor1=true;
           continue;
         }

        // Option s="~Untertitel"
        if (s.contains(ToolScriptTop3))
        { stripFirst(s,ToolScriptTop3);
          makeItem(s, Qt::darkYellow, BackColorTop3);
          isBackColor1=true;
          continue;
        }

        // Option s="Titel"
        makeItem(s, Qt::red, BackColorTop2);
        isBackColor1=true;
        continue;
      }

      // Zeile ohne s="! |"
      if (s.startsWith(ToolScriptRem))
      { // s ist " // Infos
        if (!LastItem) return;
        getSubString(s,ToolScriptRem,"");

        if (LastItem->text(2).isEmpty()) LastItem->setText(2,s);
          else LastItem->setText(2,LastItem->text(2)+"\n"+s);
        continue;
      }

      // Subitem Befehl anlegen
      int color;     
      { if (s.startsWith("void")) stripFirst(s,' ');
        if (isBackColor1==true)
             { color=BackColor1; isBackColor1=false; }
        else { color=BackColor2; isBackColor1=true;  }
        makeItem(s, Qt::darkGreen, color,true);
      }
     }

}



//-----------------------------------------------------------------------------
void tToolScriptDlg::closeEvent(QCloseEvent * e )
{ Setup->save(this);
  e->accept();
}


//-----------------------------------------------------------------------------
void tToolScriptDlg::FormatBefehl(const QString &Objektname, QString &Definition,
                                  bool AlleParameter, bool Strichpunkt)
{
  Definition=Definition.trimmed();
  if (Definition.startsWith(ScriptVar)) // var ....
     { Definition +="\n";
       return;
     }

  if (Definition.contains(SepBlockAuf)) // function {....
     { Definition +="\n";
       return;
     }

    // for( ){}
  if (Definition.startsWith(ScriptConst))  // const X;
     {  stripFirst(Definition, SepBlank);
        Definition=Objektname + stripFirst(Definition,SepLst);
        return;
     }

   QString s;
  //Definition="qreal f(int i, real r=0.2);" Objektname="t."
  s=stripFirst(Definition, SepKlammerAuf);
  if (s.isEmpty())
   { // keine Funktion
     Definition +="\n"; return;
   }
  //s="qreal f"
  //Definition=int i, real r=0.2);"

  if (!stripFirst(s, SepBlank).isEmpty()) //s=f"
   { // Rückgabe qreal
     Strichpunkt=false;
   }
  s=Objektname+s+SepKlammerAuf;      //s="t.f("

  Definition=stripFirst(Definition,SepKlammerZu);
  //Definition=int i, real r=0.2
  QStringList l=Definition.split(SepParam);

  for (int i=0; i<l.length(); i++)
  { if (l.at(i).isEmpty()) break;

    if (l.at(i).contains(SepGleich))
     {// Check Optionalen  Parameter

      if (!AlleParameter) break;  // Keine Optionalen  Parameter

      // Optionale  Parameter
      stripFirst(l[i],SepGleich);
      if (i>0) s+=SepParam;  // ,

      s+=l.at(i);
     }
    else
     { // Parameter
       if (i>0) s+=SepParam; // ,
       if (l.at(i).contains("QString")) s+=SepHochkomma+SepHochkomma;
       //else  s+=SepBlank+SepBlank;
     }
  }
  if (Strichpunkt) Definition=s+SepKlammerZu+SepLst;
  else Definition=s+SepKlammerZu;
}


