#ifndef TOOLSCRIPT_H
#define TOOLSCRIPT_H
#include <QtGui>
#include "ui_toolscript.h"


// tToolScriptDlg >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//
// Toolfenster für Scriptbefehle
// Formular: "toolscript.ui"
// Befehlsdatei: scriptbefehle.h
//
// Die Befehlsdatei wird zeilenweise gelesen und baut einen QTree
// mit den Feldern:  | Objekt | Überschrift/Befehl | Anleitung |
//
// Format der Datei scriptbefehle.h:
//
//  * Wie in den class-Definitionen unter 'public slots:'
//  * Kommentare // immer in einer eigenen Zeile!
//
// Auswertung von scriptbefehle.h:
//    Die Auswertung wird über folgende Strings gesteuert.
//    Befehlseinträge können angeklickt werden.
//
//    "/*! End", Ende der Auswertung. Restzeile egal, z.B "*/".
//    diese und folgende Zeilen werden ignoriert.
//
//    "/*! |ObjektX.Titel|", Restzeile egal.
//    erzeugt Objekt- und Titel-Eintrag:
//         | ObjektX         | Titel       | ... |
//
//    "/*! |Titel|", Restzeile egal.
//    erzeugt Titel-Eintrag:
//         | letztes ObjektX | Titel       | ... |
//
//    "/*! |~Untertitel|", Restzeile egal.
//    erzeugt Untertitel-Eintrag
//         | letztes ObjektX | Untertitel  | ... |
//
//    " Befehl xxx ", immer die ganze Zeile.
//    erzeugt anklickbaren Befehlseintrag:
//         | letztes ObjektX | Befehl xxx  | ... |
//
//    "// Infos", bis zum Zeilenende.
//    erzeugt Anleitung zu ObjektX, Titel, Untertitel oder Befehl:
//         | letztes ObjektX | letzter Titel/Überschrift | Infos |
//
//
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
class tToolScriptDlg : public QWidget
{
  Q_OBJECT
public:

   tToolScriptDlg(QWidget *parent=0);
   ~tToolScriptDlg();

public slots:

  void setBefehl();    // Befehl übernehmen

  void closeEvent(QCloseEvent * e );

public:

  void kopieren(const QString &Befehl);    // Befehl in Zwischenablage kopieren
  void insertDock(const QString &Befehl);  // Befehl ins Dock kopieren


  static void FormatBefehl(const QString &Objektname,
                                 QString &Definition,
                                 bool AlleParameter=true,
                                 bool Strichpunkt=true );
  // Befehl für das Dock formatieren
  // Input : Modulname="t." , Definition="qreal f(int i, real r=0.2);" , true
  // Output: t.f( );

  // Input : Modulname="" , Definition="qreal f(int i, real r=0.2);" , falsl
  // Output: t.f( ,0.2);

private:
  friend class     tSetup;
  Ui::ToolScript  *ui;       // Ui "toolscript.ui"

  void             makeHelpTree(); // Befehlsbeschreibungen aus ToolScript
                                   // lesen und Help-Tree erzeugen

  QTreeWidgetItem *makeTopLevelItem(QString &Text, int BackColor);
  QTreeWidgetItem *makeItem(QString &Text,
                            QColor Color=Qt::black,
                            QColor BackColor=Qt::white,
                            bool isBefehl=false);

  QTreeWidgetItem *LastTopItem;
  QTreeWidgetItem *LastItem;
  bool isBackColor1;
};



#endif // TOOLSCRIPT_H
