#include "msg.h"
#include "setup.h"

// MsgBox >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//
tMsgBox *MsgBox;  // globaler MsgBox-Pointer


tMsgBox::tMsgBox(QWidget *parent): QTextEdit(parent), out(&temp)
{
}


void tMsgBox::addOut(QRgb rgb, bool flush)
{
  if (out.string()!=0)
   {  addLn(*out.string(), rgb);
      if (flush) {temp.clear(); }
   }
}


void tMsgBox::add(const QString &string, QRgb rgb)
{ setTextColor(QColor(rgb));
  textCursor().insertText(string);
}


void tMsgBox::addLn(const QString &string, QRgb rgb )
{ if (string.size()>0)
  { setTextColor(QColor(rgb) );
    append(string);
  }
  moveCursor( QTextCursor::End );
}


void tMsgBox::addLst(const QStringList &lst, QString Titel, QChar sep)   //Liste ausgeben
{ QString s;
  if (Titel!="") s=Titel+": "+ lst.join(sep);
  else  s=lst.join(sep);
  addLn(s);
}


void tMsgBox::clear()
{ document()->clear();
  temp.clear();
}

void tMsgBox::setFontSize(qreal size)
{
  QFont f=document()->defaultFont();
  f.setPointSizeF(size);
  document()->setDefaultFont(f);
}



// Ende MsgBox <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<



// qDebug >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

void MsgBoxQtOutput(QtMsgType type, const char *msg)
 {   switch (type) {
     case QtDebugMsg:
       MsgBox->addLn("Debug: "+QString(msg),tMsgBox::err);
         break;
     case QtWarningMsg:
       MsgBox->addLn("Warning: "+ QString(msg), tMsgBox::err);
         break;
     case QtCriticalMsg:
       MsgBox->addLn("Critical: "+ QString(msg), tMsgBox::err);
         break;
     case QtFatalMsg:
       MsgBox->addLn("Fatal: "+ QString(msg), tMsgBox::err);
         abort();
     }
 }
// Ende qDebug <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<



// Strings >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//
QString stripFirst(QString &s, QChar Trenner)
{ QString ret="";
  int i=s.indexOf(Trenner);
  if (i<0) return ret;
  if (0<i)  ret= s.mid(0,i);
  i++; if (i<s.size()) s=s.mid(i); else s="";
  return ret;
}



void removeCPlusPlusRemark(QByteArray &ba)
{ int i=0,
      countHochkomma=0,  // Zeichen " zaehlen
      countSlash=0,      // Zeichen / zaehlen
      SlashPos=-1;

  while (i<ba.size())
  {
     switch (ba.at(i))
     { case '\\':  i++;
                   countSlash=0;
                   break;
       case '"' :  countHochkomma++;
                   countSlash=0;
                   break;
       case '/' :  if (0 == (countHochkomma % 2))
                     { // '/' ausserhalb von "..." gefunden
                       if (countSlash==0) { SlashPos=i; countSlash++; }
                       else countSlash=2;
                     }
                   break;
       case '\n':  if (countSlash==2)
                   { for( int j=SlashPos; j<i; j++ ) ba[j]=' ';
                   }
                   countHochkomma=0;
                   countSlash=0;
                   break;
       default:    if (countSlash<2) countSlash=0;
     }
     i++;
   }
}




QString getUnicode(const QString &string)
{
  QString s;
  for(int i=0; i<string.size(); i++)
    {  QChar c=string.at(i);
        s=s+c+"="+Hex(c.unicode(),2)+"|";
    }
  s="|"+s;
  return s;
}

QString getAsciiCode(const QByteArray &ba)
{  QString s;
   for(int i=0; i<ba.size(); i++)
   { s=s+'|'+ba.at(i)+"="+Hex(ba.at(i),2); }
   return s;
}


//String mit int Werten >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
QString si( const QString &s, int i){return s.arg(i);}
QString si( const QString &s, int i, int j){return s.arg(i).arg(j);}
QString si( const QString &s, int i, int j, int k){return s.arg(i).arg(j).arg(k);}
QString si( const QString &s, int i, int j, int k, int l){return s.arg(i).arg(j).arg(k).arg(l);}
QString si( const QString &s, qreal i, qreal j, qreal k, qreal l){return s.arg(i).arg(j).arg(k).arg(l);}

QString rot=("<span style=\"color:#FF0000;\">");
QString gruen=("<span style=\"color:#005500;\">");
QString blau=("<span style=\"color:#000099;\">");
QString off=("</span>");
QString courier=("<span style=\" font-family:'Courier New'\">");

QString Hex(int i, int w) { QString s("%1"); return s.arg(i,w,16,HexFillChar).toUpper(); }
QString Bin(int i, int w) { QString s("%1"); return s.arg(i,w,2,HexFillChar).toUpper(); }
QString Int(int i, int w) { QString s("%1"); return s.arg(i,w,10); }
QString Real(qreal x, int w, int p) { QString s("%1"); return s.arg(x,w,'g',p); } // &L1 für ','
QString Bool(bool b) { if (b)  return "true"; else return "false"; }
QString Str(const QString &s, int w)
{ if (s.size()>=w) return s;
  QString b=" ";   return s+b.repeated(w-s.size());
}
QString OnOff(int i)      { if (i==0) return QString ("AUS"); else  return QString ("EIN");}
QString OnOff(int i, QString s ) { if (i==0) return "-"; else  return s;}

QString Punkt(QPointF p) { return QString("("+Real(p.x())+","+Real(p.y())+")"); }
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
// Ende Strings <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<




