#include "utils.h"
#include "msg.h"
#include "setup.h"

extern tMsgBox   *MsgBox;

// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// Teilstring bestimmen

bool getSubString(QString &Str,const QString &MarkeL, const QString &MarkeR)
{ int i0,i1;
  if (Str.isEmpty()) return false;
  if (MarkeL.isEmpty()) i0=0;
  else i0=Str.indexOf(MarkeL);
  if (i0<0) return false;
  i0+=MarkeL.length();
  if (MarkeR.isEmpty())
  {
    i1=Str.length()-1;
  }
  else i1=Str.indexOf(MarkeR,i0)-1;

  if (i1<0) return false;
  if (i0>i1) {Str.clear(); return false; }
  Str=Str.mid(i0, i1-i0+1);
  return true;
}


// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// Subtraktive Farbmischung
//

QString subRgb(int cyan, int magenta, int yellow)
{ QColor c(255-cyan, 255-magenta, 255-yellow);
  return c.name();
}

// ----------------------------------------------------------------------------
void subRgb(QString Rgb, int &cyan, int &magenta, int &yellow)
{ QColor c(Rgb);
  cyan=255-c.red();
  magenta=255-c.green();
  yellow=255-c.blue();
}


// ----------------------------------------------------------------------------
QString komplRgb(QString Rgb)
{   QColor c(Rgb);
    int alpha=c.alpha();
    int kc= 0xFFFFFF ^ c.rgb();
    c.setRgb(kc);
    c.setAlpha(alpha);
    return c.name();
}



// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// Zufallfuntionen mit qrand()
//
tRnd::tRnd(int seed)
{ if (seed>0) qsrand(seed);
}

tRnd::tRnd(qreal x0, qreal x1, int Samples)
{ setIntervall(x0,x1,Samples);
}


// ----------------------------------------------------------------------------
tRnd::tRndIntervall::tRndIntervall(qreal x0, qreal x1, int Samples):
  r0(0),
  r1(0)
{ if ((x1<x0)) { this->x0=x1; this->x1=x0; }
  else         { this->x0=x0; this->x1=x1; }

  if (w>0) this->w=Samples;
  else w=1;
}


// ----------------------------------------------------------------------------
void tRnd::clear(int n)
{
  if (n<0) RndIntervalle.clear();
   else if (n<RndIntervalle.count()) RndIntervalle.removeAt(n);
}


// ----------------------------------------------------------------------------
void tRnd::setIntervall(qreal x0, qreal x1, int Samples)
{
  RndIntervalle.append(tRndIntervall(x1,x0,Samples));
  setRndIntervalle();

}


// ----------------------------------------------------------------------------
qreal tRnd::qnext(qreal max)
{ qreal r=RAND_MAX;  r= qrand() / r;  // 0 <= r <= 1
  return r * max;                     // 0 <= x <= max
}


// ----------------------------------------------------------------------------
int tRnd::qnext(int max)
{ if (max==0) return 0;
  else return qrand() % max;
}


// ----------------------------------------------------------------------------
qreal tRnd::qnext() const
{ int r=qrand();

  if (RndIntervalle.isEmpty()) return r;
  else
  { int i=0;

    while (i<RndIntervalle.length())
    { const int  &r1=RndIntervalle.at(i).r1;

      if (r<=r1)
      { const int   &r0=RndIntervalle.at(i).r0;
        const qreal &x0=RndIntervalle.at(i).x0;
        const qreal &x1=RndIntervalle.at(i).x1;

        if (r0==r1) return x0;
        else        return x0+ (x1-x0)* (qreal)(r-r0)/(qreal)(r1-r0);
      }
      i++;
    }

  return 0;
  }
}

// ----------------------------------------------------------------------------
QString tRnd::Info(bool kurz) const
{ QString s;

  if (RndIntervalle.isEmpty())
  { s="Rnd von 0 bis "+Int(RAND_MAX,0);
  }
  else
  { s="Rnd-Werte\n";
    for (int i=0; i<RndIntervalle.length(); i++)
    {
     const qreal &x0=RndIntervalle.at(i).x0;
     const qreal &x1=RndIntervalle.at(i).x1;
     const int   &r0=RndIntervalle.at(i).r0;
     const int   &r1=RndIntervalle.at(i).r1;
     const int   &w =RndIntervalle.at(i).w;

     qreal wProzent=100.0*(qreal)(r1-r0)/(qreal)RAND_MAX;

     s+="["+Real(x0)+", "+Real(x1)+"] "+Real(wProzent,4,4)+"%";
     if (kurz) s+=+"\n";
     else s+=" Definition: ["+ Real(r0)+", "+Real(r1)+"] w="+ Real(w)+"\n";

    }
  }
  return s;
}


// ----------------------------------------------------------------------------
void tRnd::setRndIntervalle()
{
  // Summe der Wahrscheinlichkeiten
  qreal wSumme(0);
  for (int i=0; i<RndIntervalle.length(); i++) wSumme+=RndIntervalle.at(i).w;

  qreal faktor= (qreal) RAND_MAX / wSumme;

  int r0=-1, r1=-1;
  for (int i=0; i<RndIntervalle.length(); i++)
  {
    r0=r1+1; if (r0>RAND_MAX) r1=RAND_MAX;
    r1=r0+(faktor* (qreal)RndIntervalle.at(i).w); if (r1>RAND_MAX) r1=RAND_MAX;
    RndIntervalle[i].r0=r0;
    RndIntervalle[i].r1=r1;
  }
}


// ----------------------------------------------------------------------------
qreal tRndN::rndN(int n)
{
  if ((n>-1) && (n<size())) return at(n).qnext();

  return tRnd::qnext(RAND_MAX);
}


// ----------------------------------------------------------------------------
void tRndN::clrRndN(int n)
{
  if (n<0) clear();
  else  if (n<size()) removeAt(n);
}


// ----------------------------------------------------------------------------
void tRndN::setRndN(int n, qreal x0, qreal x1, int wProzent)
{
  if (n<0) return;
  //const tRndN &N=*this;

  if (n<size()) operator [](n).setIntervall(x0,x1,wProzent);
  else
    {
      append(tRnd(x0,x1,wProzent));
    }
}


// ----------------------------------------------------------------------------
QString tRndN::Info(int n)
{ QString s;
  if (n<0)
    { for (int i=0;i<size();i++)
    { s+="Rnd"+Int(i,0)+"\n";
      s+=at(i).Info();
     }
    }
  else if (n<size()) s=at(n).Info();

  return s;
}
//


// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// Zufallsfarbe
tRndFarbe::tRndFarbe()
{ setRGB();
  LighterFaktor=0; LighterMaxIncr=0; //80,70
}


// ----------------------------------------------------------------------------
void tRndFarbe::clear()
{ setRGB();
  LighterFaktor=0; LighterMaxIncr=0;
}


// ----------------------------------------------------------------------------
void tRndFarbe::setRGB(const QString &RGB1, const QString &RGB2)
{  QColor A,B;
   if (RGB1=="") A.setRgb(70,190,90);
   else  A.setNamedColor(RGB1);

   if (RGB2=="") B.setRgb(0,0,0);
   else  B.setNamedColor(RGB2);

   Ar = A.red();
   Ag = A.green();
   Ab = A.blue();
   ABr= B.red()  -Ar;
   ABg= B.green()-Ag;
   ABb= B.blue() -Ab;
}


// ----------------------------------------------------------------------------
void tRndFarbe::setLighter(int Faktor, int maxIncrement)
{ LighterFaktor=Faktor;
  LighterMaxIncr=maxIncrement;
}


// ----------------------------------------------------------------------------
QString tRndFarbe::rgb1()
{ QColor c(qnext(Ar),qnext(Ag),qnext(Ab));
  return c.lighter(LighterFaktor+qnext(LighterMaxIncr)).name();
}


// ----------------------------------------------------------------------------
QString tRndFarbe::rgb2()
{  QColor x; qreal r=RAND_MAX;  r= qrand() / r;
   x.setRed(   Ar + ABr*r);
   x.setGreen( Ag + ABg*r);
   x.setBlue(  Ab + ABb*r);
   return x.lighter(LighterFaktor+qnext(LighterMaxIncr)).name() ;
}

// ----------------------------------------------------------------------------
QString tRndFarbe::rgbRGB(qreal rnd)
{ QColor x;
  x.setRed  ( Ar + ABr*rnd);
  x.setGreen( Ag + ABg*rnd);
  x.setBlue ( Ab + ABb*rnd);
  return x.lighter(LighterFaktor+qnext(LighterMaxIncr)).name();
}

// ----------------------------------------------------------------------------
int tRndFarbe::qnext(int max)
{ if (max==0) return 0;
  else return qrand() % max;
}

// ----------------------------------------------------------------------------
qreal tRndFarbe::qnext(qreal max)
{ qreal r=RAND_MAX;  r= qrand() / r;  // 0 <= r <= 1
  return r * max;                     // 0 <= x <= max
}


// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<



// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// Zufallsfunktionen
//
tRandom::tRandom()
{ setSeed(12345678);
}

tRandom::tRandom(int seed)
{ setSeed(seed);
}


void tRandom::setSeed(int seed)
{ const int b=31415821;
  a[0] =seed;
  for (j=1; j<=54; j++)
      a[j]=( mult(a[j-1], b) + 1) %m;

  qsrand(seed);
}


int tRandom::mult (int p, int q)
{ int p1,p0,q1,q0;
  p1=p/m1; p0=p%m1;
  q1=q/m1; q0=q%m1;

  return (((p0*q1 + p1*q0)% m1 )* m1 + p0*q0) % m;
}

int tRandom::next(int max)
{
    j= (j+1) % 55 ;
    a[j] = ( a[(j+23) % 55] + a[(j+54) % 55]) % m;
    return ((a[j]/m1)*max)/m1;
}

qreal tRandom::next(qreal max)
{ j= (j+1) % 55 ;
  a[j] = ( a[(j+23) % 55] + a[(j+54) % 55]) % m;
  qreal r=qreal(a[j])/ qreal(m);
  return r*max;
}

int tRandom::qnext(int max)
{ if (max==0) return 0;
  else return qrand() % max;
}

qreal tRandom::qnext(qreal max)
{
  qreal r=RAND_MAX;  r= qrand() / r;  // 0 <= r <= 1
  return r * max;                     // 0 <= x <= max
}


qreal tRandom::chiSquare()
{
  int N=10000, max=100;
  int i, t, f[1000];
  for (i=0; i<max; i++) f[i]=0;
  for (i=0; i<N; i++) f[next(max)]++;
  for (i=0, t=0; i<max; i++) t +=f[i]*f[i];
  return qreal( (max*t/N) -N );
}

qreal tRandom::qchiSquare()
{
  int N=10000, max=100;
  int i, t, f[1000];
  for (i=0; i<max; i++) f[i]=0;
  for (i=0; i<N; i++) f[qnext(max)]++;
  for (i=0, t=0; i<max; i++) t +=f[i]*f[i];
  return qreal( (max*t/N) -N );
}

// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//
// Vektoren und Punkte
//
//-----------------------------------------------------------------------------
double Betrag(const QPointF &P)
{  return sqrt(P.x()*P.x()+P.y()*P.y());
}


//-----------------------------------------------------------------------------
double Betrag(const QPointF &A, const QPointF &B)
{  QPointF P=B-A;
   return sqrt(  P.x()*P.x()+P.y()*P.y());
}


//-----------------------------------------------------------------------------
qreal wAlpha(const QPointF &A, const QPointF &B)
{ return wAlpha( QPointF (B-A) );
}


//-----------------------------------------------------------------------------
qreal wAlpha(const QPointF &P)
{ double l=Betrag(P), w=0;

  if (l<epsilon)
  { MsgBox->addLn("Err: Winkel nicht definiert!", tMsgBox::err);
    return w;
  }

  w= asin(P.y()/l)*180.0/Pi;

  if (P.x()>0)
     if (P.y()>0) return w;
     else  return w;
  else
     if (P.y()>0) return 180-w;
     else  return -180-w;
}


//-----------------------------------------------------------------------------
qreal wAlpha(const QPointF &A, const QPointF &B, const QPointF &C)
{
  QPointF AB=B-A, AC=C-A;
  return  wAlpha(AC)-wAlpha(AB);
}

//-----------------------------------------------------------------------------
QPointF vektorNormal(const QPointF &A, const QPointF &B)
{
  QPointF AB=B-A;
  return QPointF(-AB.y(),AB.x());
}


//-----------------------------------------------------------------------------
QPointF vektorENormal(const QPointF &A, const QPointF &B, bool &Ok)
{
  QPointF AB=B-A;
  qreal l= sqrt(  AB.x()*AB.x()+AB.y()*AB.y());
  if (l<epsilon) {Ok=false; return QPointF(0,0); }
  AB=(1/l)*AB; Ok=true;
  return QPointF(-AB.y(),AB.x());
}




//-----------------------------------------------------------------------------
void setInsertMatrix(QMatrix &m, const QPointF &A, const QPointF &B, const QPointF &A1, const QPointF &B1)
{ qreal w=0;
  QPointF AB=B-A, A1B1=B1-A1;
  double l=Betrag(AB), l1=Betrag(A1B1);

  m.reset();
  m.translate(A1.x(),A1.y());

  if ((l>epsilon)&&(l1>epsilon))
  { w= wAlpha(A1B1)-wAlpha(AB);
    qreal s=l1/l; m.scale(s,s);
    m=m.rotate(w);
  }

  m.translate(-A.x(),-A.y());
}


//-----------------------------------------------------------------------------
QPointF &Raster(qreal dl, QPointF &P)
{ if (dl>epsilon)
  { P.rx()=round(P.rx()/dl)*dl;
    P.ry()=round(P.ry()/dl)*dl;
  }
  return P;
}


//-----------------------------------------------------------------------------
void cutPolyLine(QPolygonF &Poly, const QPolygonF &cutPoly)
{
  if ((Poly.count()<2)||(cutPoly.count()<2)) return;

  for(int j=1; j<cutPoly.count(); j++)
  {  QLineF cLine(cutPoly.at(j-1),cutPoly.at(j));

     for(int i=1; i<Poly.count(); i++)
     { QPointF P;
       QLineF pLine(Poly.at(i-1), Poly.at(i));
       if (QLineF::BoundedIntersection==pLine.intersect(cLine,&P))
        { Poly[i]=P;
          Poly.resize(i+1);
          return;
        }
     }
  }
}


//-----------------------------------------------------------------------------
qreal boundedIntersection(const QPolygonF &Poly, const QPointF &P0, const QPointF &P1)
{
  QLineF cLine(P0,P1); qreal lmin=-1;
  if ((Poly.count()<2)||(cLine.length()<epsilon)) return lmin;

  for(int i=1; i<Poly.count()-1; i++)
  {
    QLineF pLine(Poly.at(i-1), Poly.at(i)); QPointF X;
    if (QLineF::BoundedIntersection==pLine.intersect(cLine,&X))
    { qreal l=QLineF (P0,X).length();
      if ((l<lmin)||(lmin==-1)) lmin=l;
    }
  }
  return lmin;
}

//-----------------------------------------------------------------------------
void addPolgonRndLine(const QPointF &, const QPointF &, const tRnd &,
               QPolygonF &)
{

}
