#ifndef UTILS_H
#define UTILS_H
#include <QtCore>
#include <QtGui>


// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// Teilstring bestimmen
bool getSubString(      QString &Str,     // String
                  const QString &MarkeL,  // Linke Begrenzung
                  const QString &MarkeR); // Rechte Begrenzung
     // Rückgabe true : Str ist der Teilstring
     //          false: Str unverändert



// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// Subtraktive Farbmischung
//

QString subRgb(int cyan, int magenta, int yellow);
        // Farbezeichner zu cyan, magenta, yellow
        // für subtraktive Farbmischung

void    subRgb(QString Rgb, int &cyan, int &magenta, int &yellow);
        // Farben cyan, magenta, yellow zu Farbstring Rgb
        // für subtraktive Farbmischung

QString komplRgb(QString Rgb);
        // Komplementäfarbe


// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// tRnd und tRndN: Zufallfuntionen mit qrand()
//
// Zufallszahlen aus Intervallen mit gewählten Wahrscheinlichkeiten.
//
class tRnd
{
  class tRndIntervall
  { public:
    tRndIntervall(qreal x0, qreal x1, int Samples);
    qreal x0,x1;   // Intervall
    qreal w;       // Wahrscheinlichkeit in Samples
    int   r0,r1;   // Rndwerte von zum Intervall
                   // setRndIntervalle()
  };

 public:
  tRnd(int seed=0);   // seed>0 qrand() neu initialisieren
  tRnd(qreal x0, qreal x1, int Samples);

  void clear(int n=-1);

  void setIntervall(qreal x0, qreal x1, int Samples);
       //Zufallszahl im Intervall [x0,x1] mit der
       //Wahrscheinlichkeit von Samples

  static qreal qnext(qreal max); // 0<= Zufallswert <=max
  static int   qnext(int   max); // 0<= Zufallswert <=max

  qreal qnext() const;
        // Zufällige Werte aus RndIntervalle
        // mit den gewählten Wahrscheinlichkeiten

  QString Info(bool kurz=true) const;

private:
  QList <tRndIntervall> RndIntervalle;

  void setRndIntervalle();

};

// ----------------------------------------------------------------------------
// Liste von tRnd Intervallen
//
class tRndN : public QList<tRnd>
{
  public:
  inline  tRndN(){}
  void    setRndN(int n, qreal x0, qreal x1, int wProzent);
  qreal   rndN(int n);
  void    clrRndN(int n);
  QString Info(int n=-1);

};
// End tRnd <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<


// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// Zufallsfarbe
//
class tRndFarbe
{public:
  tRndFarbe();
  void    clear();

  int     qnext(int max);    // Zufallswert <=max
  qreal   qnext(qreal max);  // Zufallswert <=max

  void    setRGB(const QString &RGB1="", const QString &RGB2="black");
          // Farbübergang von RGB1 nach RGB2 definieren
  void    setLighter(int Faktor=100, int maxIncrement=0);
          // Ergebnis um Faktor aufhellen
          // maxIncrement<0: Faktor zufällig vergrößern

  QString rgb1(); // Zufällige Farbe von RGB1 mit setLighter(...)
  QString rgb2(); // Zufällige Farbe zwischen RGB1 und RGB2
                  // mit setLighter(...)

  QString rgbRGB(qreal rnd); // Farbe zu zufallswert rnd 0<=rnd<=1

 private:
  int Ar, Ag, Ab;     // Anfangsfarbe
  int ABr,ABg,ABb;    // Endfarbe-Anfangsfarbe
  int LighterFaktor,
      LighterMaxIncr;
};
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<




// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//
// Zufallsfunktionen, Programmiert
//
//
class tRandom
{ public:
    tRandom();
    tRandom(int seed);

    void  setSeed(int seed);

    int   next(int max);
    qreal next(qreal max);

    int   qnext(int max);
    qreal qnext(qreal max);

    qreal chiSquare();
    qreal qchiSquare();


  private:
    static const int m =100000000;
    static const int m1=10000;

    int a[55],j;
    int mult(int p, int q);
};
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<


// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//
// Vektoren, Punkte
//
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
double Betrag(const QPointF &P);
double Betrag(const QPointF &A, const QPointF &B);
qreal  wAlpha(const QPointF &P);
qreal  wAlpha(const QPointF &A, const QPointF &B);
qreal  wAlpha(const QPointF &A, const QPointF &B, const QPointF &C);

QPointF vektorNormal(const QPointF &A, const QPointF &B);
        //Normalvektor zu AB
QPointF vektorENormal(const QPointF &A, const QPointF &B, bool &Ok);
        //Einheits-Normalvektor zu AB

void setInsertMatrix(
          QMatrix &m,                            // Abbildung
          const QPointF &A, const QPointF &B,    // Strecke
          const QPointF &A1, const QPointF &B1); // Bildstrecke

QPointF &Raster(qreal dl, QPointF &P);
        //nächster Rasterpunkt zu P

void cutPolyLine(QPolygonF &Poly, const QPolygonF &cutPoly);
     // Schnittpunkte von Poly mit cutPoly bestimmen
     // Rückgabe: Poly von P[0] bis zum ersten Schnittpunkt

qreal boundedIntersection(const QPolygonF &Poly, const QPointF &P0, const QPointF &P1);
     // Innerer Schnittpunkt X von Poly mit Strecke (P0,P1)
     // Rückgabe: 0<=t<=1 Schnittpunkt P0 + t(P0,P1)
     //           t=-1 Schnittpunkt undefiniert


void addPolgonRndLine(const QPointF &A, const QPointF &B, const tRnd &rnd,
               QPolygonF &P);
     //Polygon P mit zufälliger Linie von A nach B erweitern.
     //rnd Zufallswahrscheinlichkeiten



#endif // UTILS_H
